<?php

namespace App\Console\Commands\DocumentExpiryCron;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Carbon;
use App\Jobs\AdminPanel\SendEmailJob;
use Mail;
use App\Models\CronJob;
use App\Interfaces\DocumentHcpInterface;
use App\Interfaces\HcpInterface;
use App\Interfaces\HcpProfessionRequestInterface;
use App\Interfaces\HcpProfessionInterface;

class DocumentExpiryCron extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'documentExpiryCron:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Hcp documents expiry cron';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct(DocumentHcpInterface $documentHcp, HcpInterface $hcp, HcpProfessionRequestInterface $hcpProfessionRequest, HcpProfessionInterface $hcpProfession)
    {
        parent::__construct();
        $this->document_hcp = $documentHcp;
        $this->hcp = $hcp;
        $this->hcp_profession_request = $hcpProfessionRequest;
        $this->hcp_profession = $hcpProfession;
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        config(['logging.channels.cronlog.path' => storage_path('logs/cron-logs/document-expiry-cron.log')]);

        // Current date
        $currentDate = Carbon::now()->toDateString();
        $currentTime = Carbon::now();
        //-------------

        $documentUpdate = null;

        // Get cron status
        $cronJob = CronJob::where('name', '=', 'document-expiry-cron')->where('status', 0)->first();
        //----------------

        if (!empty($cronJob)) {

            // Update cron status in processing
            $cronJob->status = 1;
            $cronJob->start_time = Carbon::now()->toDateTimeString();
            $cronJob->save();
            //---------------------------------

            DB::beginTransaction();
            try {

                // Get hcps
                $hcps = $this->hcp->get_hcp_expiry_document('expiry', $currentDate);
                //---------

                if (!empty($hcps)) {

                    Log::channel('cronlog')->info("-----------------------");
                    Log::channel('cronlog')->info("documents expiry cron start");
                    Log::channel('cronlog')->info("documents expiry cron processing");

                    foreach($hcps as $hcp) {

                        // Send email document notification
                        $emailTemplate = config('constants.admin_email_path') . 'document-expiry-email';
                        $emailData = [
                            'template' => $emailTemplate,
                            'data' => [
                                'name' => ucwords($hcp['first_name']),
                                'email' => $hcp['user']['email'],
                                'subject' => __('subjects.document_expiry'),
                                'documents' => $hcp['hcp_documents'],
                            ],
                        ];

                        if(config('app.server') == 'AWS' ) {
                            SendEmailJob::dispatch($emailData)->delay(now()->addSeconds(5));
                        } else {
                            $data = $emailData['data'];
                            Mail::send($emailTemplate, $data, function ($message) use ($data) {
                                $message->to($data['email']);
                                $message->subject($data['subject']);
                            });
                        }
                        //---------------------------------

                        // Update Send Notification
                        foreach($hcp['hcp_documents'] as $document){
                            $documentName = ucwords(str_replace("-", " ", preg_replace('/[0-9]+/', '', $document->document['name'])));

                            // Send push notification
                            send_document_expiry_push_notification($document->hcp_id, $documentName, $document->document['doc_type'], 'document-expired');
                            //-----------------------

                            // Update Hcp Profession
                            if (!empty($document['profession_id']) && $document->document['mandatory_status'] == 'yes') {
                                $this->hcp_profession->update([
                                    'status' => 'expired'
                                ], [
                                    'hcp_id' => $document->hcp_id,
                                    'profession_id' => $document['profession_id'],
                                ]);
                            }
                            //----------------------

                            // // Store Hcp Profession Request
                            // if (!empty($document['profession_id'])) {
                            //     $this->hcp_profession_request->store([
                            //         'hcp_id' => $document->hcp_id,
                            //         'hcp_profession_id' => null,
                            //         'category_id' => $document['profession']['category_id'],
                            //         'profession_id' => $document['profession']['id'],
                            //         'leve_id' => null,
                            //         'hcp_document_id' => $document->id,
                            //         'type' => 'document-expired',
                            //         'status' => 'pending',
                            //         'created_by' => $document->hcp->user_id,
                            //     ]);
                            // }
                            // //-----------------------------

                            $document->document_approval = 'expired';
                            $document->show_in_document_approval = 1;
                            $document->show_in_notification = 1;
                            $document->save();
                        }
                        //-------------------------


                    }

                    Log::channel('cronlog')->info("documents expiry cron completed");
                    Log::channel('cronlog')->info("-----------------------");

                } else {
                    Log::channel('cronlog')->info("hcp expiry documents not found.");
                }

                // Update cron status to completed
                $cronJob->status = 0;
                $cronJob->end_time = Carbon::now()->toDateTimeString();
                $cronJob->save();
                //---------------------------------


                DB::commit();

            } catch (\Exception $e) {
                Log::channel('cronlog')->error($e->getMessage());
            }

        } else {
            Log::channel('cronlog')->warning('cron stuck in processing state...');
        }


    }

}
