<?php

namespace App\Console\Commands\DocumentExpiryCron;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Carbon;
use App\Jobs\AdminPanel\SendEmailJob;
use Mail;
use App\Models\CronJob;
use App\Interfaces\DocumentHcpInterface;
use App\Interfaces\HcpInterface;

class DocumentExpiryNotificationCron extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'documentExpiryNotification:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Hcp documents expiry notification cron';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct(DocumentHcpInterface $documentHcp, HcpInterface $hcp)
    {
        parent::__construct();
        $this->document_hcp = $documentHcp;
        $this->hcp = $hcp;
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        config(['logging.channels.cronlog.path' => storage_path('logs/cron-logs/document-expiry-notification-cron.log')]);

        // Current date
        $currentDate = Carbon::now()->toDateString();
        $currentTime = Carbon::now();
        //-------------

        $documentUpdate = null;

        // Get cron status
        $cronJob = CronJob::where('name', '=', 'document-expiry-notification-cron')->where('status', 0)->first();
        //----------------

        if (!empty($cronJob)) {

            // Update cron status in processing
            $cronJob->status = 1;
            $cronJob->start_time = Carbon::now()->toDateTimeString();
            $cronJob->save();
            //---------------------------------

            DB::beginTransaction();
            try {

                // Get hcps
                $hcps = $this->hcp->get_hcp_expiry_document('notification', $currentDate);
                //---------

                if (!empty($hcps)) {
                    Log::channel('cronlog')->info("-----------------------");
                    Log::channel('cronlog')->info("documents expiry notifications cron start");
                    Log::channel('cronlog')->info("documents expiry notifications cron processing");

                    foreach($hcps as $hcp) {

                        // Send email document notification
                        $emailTemplate = config('constants.admin_email_path') . 'document-expiry-notification-email';
                        $emailData = [
                            'template' => $emailTemplate,
                            'data' => [
                                'name' => ucwords($hcp['first_name']),
                                'email' => $hcp['user']['email'],
                                'subject' => __('subjects.document_expiry_notification'),
                                'documents' => $hcp['hcp_documents'],
                            ],
                        ];

                        if(config('app.server') == 'AWS' ) {
                            SendEmailJob::dispatch($emailData)->delay(now()->addSeconds(5));
                        } else {
                            $data = $emailData['data'];
                            Mail::send($emailTemplate, $data, function ($message) use ($data) {
                                $message->to($data['email']);
                                $message->subject($data['subject']);
                            });
                        }
                        //---------------------------------

                        foreach($hcp['hcp_documents'] as $document){
                            $documentName = ucwords(str_replace("-", " ", preg_replace('/[0-9]+/', '', $document->document['name'])));

                            // Add next 7 days to date
                            $date = Carbon::createFromFormat('Y-m-d', $document['send_notification_date']);
                            $daysToAdd = 7;
                            $notificationDate = $date->addDays($daysToAdd);
                            //-------------------------

                            // Update Send Notification Date
                            $document->show_in_notification = 1;
                            $document->send_notification_date = $notificationDate;
                            $document->save();
                            //-----------------------------

                            // Send push notification
                            send_document_expiry_push_notification($document->hcp_id, $documentName, $document->document['doc_type'], 'document-expiry-soon');
                            //-----------------------

                        }
                    }

                    Log::channel('cronlog')->info("documents expiry notifications cron completed");

                } else {
                    Log::channel('cronlog')->info("hcp expiry documents not found.");
                }

                // Update cron status to completed
                $cronJob->status = 0;
                $cronJob->end_time = Carbon::now()->toDateTimeString();
                $cronJob->save();
                //---------------------------------

                DB::commit();

            } catch (\Exception $e) {
                Log::channel('cronlog')->error($e->getMessage());
            }

        } else {
            Log::channel('cronlog')->warning('cron stuck in processing state...');
        }


    }

}
