<?php

namespace App\Console\Commands\FacilityPlanCron;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Carbon;
use Mail;
use App\Jobs\AdminPanel\SendEmailJob;
use App\Models\CronJob;
use App\Interfaces\FacilityPlanInterface;
use App\Interfaces\FacilityInterface;
use Stripe;

class PlanExpiryNotificationCron extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'planExpiryNotificationCron:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Plan expiry notification cron';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct(FacilityPlanInterface $facilityPlan, FacilityInterface $facility)
    {
        parent::__construct();
        $this->facility_plan = $facilityPlan;
        $this->facility = $facility;
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        config(['logging.channels.cronlog.path' => storage_path('logs/cron-logs/facility-plan-cron.log')]);

        // Current date
        $currentDate = Carbon::now()->toDateString();
        $currentTime = Carbon::now();
        //-------------

        // Get cron status
        $cronJob = CronJob::where('name', '=', 'facility-plan-expiry-notification-cron')->where('status', 0)->first();
        //----------------

        if (!empty($cronJob)) {

            // Update cron status in processing
            $cronJob->status = 1;
            $cronJob->start_time = Carbon::now()->toDateTimeString();
            $cronJob->save();
            //---------------------------------

            DB::beginTransaction();
            try {

                // Get Facility Plan
                $requestArray = ['notification_date' => $currentDate];
                $facilityPlans = $this->facility_plan->cron_get_facility_plan($requestArray);
                //------------------

                if (count($facilityPlans)) {
                    Log::channel('cronlog')->info("-----------------------");
                    foreach($facilityPlans as $facilityPlan) {
                        $planDetails = json_decode($facilityPlan['plan_details'], true);

                        // Get Current Plan
                        if (!empty($facilityPlan) && $facilityPlan['plan_id'] != 4 && !empty($facilityPlan['stripe_subscriptions_id'])) {
                            $stripe = new \Stripe\StripeClient(env('STRIPE_SECRET'));
                            $subscription = $stripe->subscriptions->retrieve(
                                $facilityPlan['stripe_subscriptions_id'],
                                []
                            );
                        } else {
                            $subscription = null;
                        }
                        //-----------------

                        // Send email notification
                        $emailTemplate = config('constants.admin_email_path') . 'facility-plan-expiry-notification-email';
                        $emailData = [
                            'template' => $emailTemplate,
                            'data' => [
                                'name' => ucwords($facilityPlan->facility['name']),
                                'email' => $facilityPlan->facility['user']['email'],
                                'subject' => __('subjects.facility_plan_expiry_notification'),
                                'facilityPlan' => $facilityPlan,
                                'subscription' => $subscription,
                            ],
                        ];

                        if(config('app.server') == 'AWS' ) {
                            SendEmailJob::dispatch($emailData)->delay(now()->addSeconds(5));
                        } else {
                            $data = $emailData['data'];
                            Mail::send($emailTemplate, $data, function ($message) use ($data) {
                                $message->to($data['email']);
                                $message->subject($data['subject']);
                            });
                        }
                        //------------------------

                        // Send push notification
                        send_facility_plan_notification('faclity-plan-expiry-soon', $facilityPlan->facility['id'], $planDetails['name']);
                        //-----------------------

                        // Update Next Notification
                        $notificationDate = null;
                        if(!empty($subscription) && $facilityPlan['plan_cycle'] == 'yearly') {
                            if(!empty($subscription) && !empty($subscription['current_period_end'])) {
                                $notificationDate =  date('Y-m-d', strtotime('-7 days', $subscription['current_period_end']));
                            } else {
                                if(!empty($facilityPlan['expiry_date'])) {
                                    $notificationDate =  date('Y-m-d', strtotime('-7 days', strtotime($facilityPlan['expiry_date'])));
                                }
                            }
                        } else {
                            if(!empty($subscription) && !empty($subscription['current_period_end'])) {
                                $notificationDate =  date('Y-m-d', strtotime('-1 days', $subscription['current_period_end']));
                            } else {
                                if(!empty($facilityPlan['expiry_date'])) {
                                    $notificationDate =  date('Y-m-d', strtotime('-1 days', strtotime($facilityPlan['expiry_date'])));
                                }
                            }
                        }
                        $this->facility_plan->update(['notification_date' => $notificationDate], ['id' => $facilityPlan->id]);
                        //-------------------------

                        Log::channel('cronlog')->info("Expiry Soon Notification Sent To: ". $facilityPlan->facility['facility_prefix'].$facilityPlan->facility['id']);

                    }
                    Log::channel('cronlog')->info("-----------------------");
                }

                // Update cron status to completed
                $cronJob->status = 0;
                $cronJob->end_time = Carbon::now()->toDateTimeString();
                $cronJob->save();
                //---------------------------------

                DB::commit();

            } catch (\Exception $e) {
                Log::channel('cronlog')->info("-----------------------");
                Log::channel('cronlog')->error('Expiry Soon Notification error: '.$e->getMessage());
                Log::channel('cronlog')->info("-----------------------");
            }

        } else {
            Log::channel('cronlog')->warning('cron stuck in processing state...');
        }


    }

}
