<?php

namespace App\Console\Commands\GenerateInvoiceCrons;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Carbon;
use App\Models\CronJob;
use App\Interfaces\TransactionInterface;
use App\Jobs\AdminPanel\SendEmailJob;
use Mail;

class InvoiceOverdueCron extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'invoiceOverdueCron:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Invoice overdue cron';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct(TransactionInterface $transaction)
    {
        parent::__construct();
        $this->transaction = $transaction;
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        config(['logging.channels.cronlog.path' => storage_path('logs/cron-logs/generate-invoice.log')]);

        // Current date
        $currentDate = Carbon::now()->toDateString();
        $currentTime = Carbon::now()->format('d-m-y H:i');
        //-------------

        $shiftUpdate = null;
        $shiftActivity = null;

        // Get cron status
        $cronJob = CronJob::where('name', '=', 'overdue-invoice-cron')->where('status', 0)->first();
        //----------------

        if (!empty($cronJob)) {

            // Update cron status in processing
            $cronJob->status = 1;
            $cronJob->start_time = Carbon::now()->toDateTimeString();
            $cronJob->save();
            //---------------------------------

            DB::beginTransaction();
            try {
                // General Configuration
                $generalConfiguration = get_general_configuration();
                //----------------------

                // facility Invoice
                $filter = [
                    'overdue_date' => date('Y-m-d'),
                    'amount_for' => 'facility',
                ];
                $invoices = $this->transaction->get_overdue_invoice($filter);
                if (count($invoices)) {
                    Log::channel('cronlog')->info("-----------------------");
                    Log::channel('cronlog')->info("facility overdue invoice cron start");

                    foreach ($invoices as $invoice) {
                        Log::channel('cronlog')->info("invoice id: ". $invoice['invoice_id']);
                        $facility = $invoice['facility'];

                        // Update Transaction
                        $array = [
                            'status' => 'overdue',
                        ];
                        $where = ['invoice_id' => $invoice['invoice_id']];
                        $this->transaction->update($array,  $where);
                        //-------------------

                        // Send email notification
                        $filename = $invoice['invoice_file_name'];

                        if (config('filesystems.default') == config('constants.file_systems.PUBLIC.value')) {
                            $attach = url('public/storage/' . config('constants.facility.invoice_path')) . '/'.urlencode($filename);
                        } elseif (config('filesystems.default') == config('constants.file_systems.S3.value')) {
                            $bucket = env('AWS_BUCKET');
                            $region = env('AWS_DEFAULT_REGION');
                            // $bucketBaseUrl = 'https://s3.' . $region . '.amazonaws.com/'. $bucket.'/';
                            // $attach = $bucketBaseUrl.config('constants.facility.invoice_path').urlencode($filename);

                            $S3NewURL = env('AWS_CloudFront_URL').'/'.config('constants.facility.invoice_path').urlencode($filename);
                            $attach = str_replace('#', '%23', $S3NewURL);

                        }

                        $emailTemplate = config('constants.admin_email_path') . 'facility-invoice-overdue-email';
                        $emailData = [
                            'template' => $emailTemplate,
                            'data' => [
                                'name' => ucwords($facility['name']),
                                'email' => $facility['user']['email'],
                                'subject' => __('subjects.facility_shift_invoice_overdue', ['InvoiceNumber' => $invoice['invoice_prefix'].$invoice['invoice_id'], 'InvoiceDate' => date('Y-m-d')]),
                                'invoiceNumber' => $invoice['invoice_prefix'].$invoice['invoice_id'],
                                'StatementDuration' => $invoice['invoice_generated_date_range'],
                                'invoiceOverdueDate' => $invoice['invoice_overdue_date'],
                                'cc' => $generalConfiguration['facillity_cust_care_email'],
                                'filename' => $attach,
                                'attach' => $attach,
                            ],
                        ];

                        if(config('app.server') == 'AWS' ) {
                            SendEmailJob::dispatch($emailData)->delay(now()->addSeconds(5));
                        } else {
                            $data = $emailData['data'];
                            Mail::send($emailTemplate, $data, function ($message) use ($data) {
                                $message->to($data['email']);
                                $message->subject($data['subject']);
                                $message->cc($data['cc']);
                                $message->attach($data['filename']);
                            });
                        }
                        //------------------------

                    }

                    Log::channel('cronlog')->info("facility overdue invoice cron end");
                    Log::channel('cronlog')->info("-----------------------");
                }
                //-----------------

                // hcp Invoice
                // $hcpFilter = [
                //     'overdue_date' => date('Y-m-d'),
                //     'amount_for' => 'hcp',
                // ];
                // $hcpInvoices = $this->transaction->get_overdue_invoice($hcpFilter);
                // if (count($hcpInvoices)) {
                //     Log::channel('cronlog')->info("-----------------------");
                //     Log::channel('cronlog')->info("hcp overdue invoice cron start");

                //     foreach ($hcpInvoices as $invoice) {
                //         Log::channel('cronlog')->info($invoice);
                //         Log::channel('cronlog')->info("statement id: ". $invoice['statement_id']);
                //         $hcp = $invoice['hcp'];

                //         // Update Transaction
                //         $array = [
                //             'hcp_payment_status' => 'overdue',
                //         ];
                //         $where = ['statement_id' => $invoice['statement_id']];
                //         $this->transaction->update($array,  $where);
                //         //-------------------
                //     }

                //     Log::channel('cronlog')->info("hcp overdue invoice cron end");
                //     Log::channel('cronlog')->info("-----------------------");
                // }
                //------------

                DB::commit();

            } catch (\Exception $e) {
                Log::channel('cronlog')->info("-----------------------");
                Log::channel('cronlog')->error('overdue invoice cron error: '.$e->getMessage());
                Log::channel('cronlog')->info("-----------------------");
            }

            // Update cron status to completed
            $cronJob->status = 0;
            $cronJob->end_time = Carbon::now()->toDateTimeString();
            $cronJob->save();
            //---------------------------------

        } else {
            Log::channel('cronlog')->info("-----------------------");
            Log::channel('cronlog')->warning('overdue invoice cron already running...');
            Log::channel('cronlog')->info("-----------------------");
        }
    }
}
