<?php

namespace App\Console\Commands\ShiftCrons;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Carbon;
use App\Jobs\AdminPanel\SendEmailJob;
use App\Interfaces\ShiftInterface;
use App\Interfaces\ShiftActivityInterface;
use App\Interfaces\BuddyShiftRequestInterface;
use App\Interfaces\TransactionInterface;
use App\Interfaces\CancelledShiftInterface;
use App\Models\CronJob;
use Mail;

class ShiftCancelCron extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'shiftCancelCron:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Hcp shift cancel cron';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct(ShiftInterface $shift, ShiftActivityInterface $shiftActivity, BuddyShiftRequestInterface $buddyShiftRequest, TransactionInterface $transaction, CancelledShiftInterface $cancelledShift)
    {
        parent::__construct();
        $this->shift = $shift;
        $this->shift_activity = $shiftActivity;
        $this->buddy_shift_request = $buddyShiftRequest;
        $this->transaction = $transaction;
        $this->cancelled_shift = $cancelledShift;
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        config(['logging.channels.cronlog.path' => storage_path('logs/cron-logs/shift-cron.log')]);

        // Current date
        $currentDate = Carbon::now()->toDateString();
        $currentTime = Carbon::now();
        //-------------

        $shiftUpdate = null;
        $shiftActivity = null;

        // Get cron status
        $cronJob = CronJob::where('name', '=', 'shift-cancel-cron')->where('status', 0)->first();
        //----------------

        if (!empty($cronJob)) {

            // Update cron status in processing
            $cronJob->status = 1;
            $cronJob->start_time = Carbon::now()->toDateTimeString();
            $cronJob->save();
            //---------------------------------

            DB::beginTransaction();
            try {

                // Get Shifts
                $shiftWhere = ['shift_status' => '0'];
                $shifts = $this->shift->get_shift_for_cron($shiftWhere);
                //-----------

                if(count($shifts)){
                    foreach($shifts as $shift){
                        // Set time zone
                        date_default_timezone_set($shift->state->time_zone_name);
                        //--------------

                        $currentTime = Carbon::now($shift->state->time_zone_name);

                        if(strtotime($currentTime) > strtotime($shift->end_time)) {
                            Log::channel('cronlog')->info("-----------------------");
                            Log::channel('cronlog')->info("shift cancel cron start");
                            Log::channel('cronlog')->info("current time: ". $currentTime);

                            // Update Shift
                            $shiftData = ['shift_status' => '3', 'cancel_by' => 'admin', 'cancel_time' => $currentTime, 'cancel_reason' => 'No HCP Accepted'];
                            $shiftUpdate = $this->shift->update($shiftData, ['id' => $shift->id]);
                            //-------------

                            // Update facility transaction
                            $transactionUpdate = $this->transaction->update(['status' => 'cancelled'], ['shift_id' => $shift->id, 'facility_id' => $shift->facility_id]);
                            //----------------------------

                            // Cancel Shift
                            $cancelledShiftData = [
                                'shift_id'      => $shift->id,
                                'hcp_id'        => null,
                                'cancel_by'     => 'admin',
                                'cancel_time'   => $currentTime,
                                'cancel_reason' => __('messages.cron_shift_cancel'),
                                'negative_points' => null,
                                'penalty_amount' => null,
                                'created_by'    => $shift->created_by,
                                'updated_by'    => $shift->updated_by,
                            ];
                            $cancelledShift = $this->cancelled_shift->store($cancelledShiftData);
                            //-------------

                            // Shift Activity
                            $shiftActivityData = [
                                'hcp_id'        => $shift->hcp_id,
                                'shift_id'      => $shift->id,
                                'activity'      => 'shift-cancel',
                                'created_by'    => $shift->created_by,
                                'updated_by'    => $shift->updated_by,
                            ];
                            $shiftActivity = $this->shift_activity->store($shiftActivityData, ['id' => $shift->id]);
                            //---------------

                            // Send Admin Notification
                            send_admin_notification('admin-shift-cancelled', 'admin', $shift->id, null, null, __('messages.cron_shift_cancel'));

                            send_admin_notification('facility-shift-cancelled', 'facility', $shift->id, null, $shift->facility_id, __('messages.cron_shift_cancel'));
                            //------------------------

                            Log::channel('cronlog')->info("shift id: ". $shift->shift_prefix.$shift->id.' cancelled');
                            Log::channel('cronlog')->info("shift cancel cron completed");
                            Log::channel('cronlog')->info("-----------------------");
                        }
                    }

                }

                DB::commit();

            } catch (\Exception $e) {
                Log::channel('cronlog')->info("-----------------------");
                Log::channel('cronlog')->error('shift cancel cron error: '.$e->getMessage());
                Log::channel('cronlog')->info("-----------------------");
            }

            // Update cron status to completed
            $cronJob->status = 0;
            $cronJob->end_time = Carbon::now()->toDateTimeString();
            $cronJob->save();
            //---------------------------------

        } else {
            Log::channel('cronlog')->info("-----------------------");
            Log::channel('cronlog')->warning('shift cancel cron already running...');
            Log::channel('cronlog')->info("-----------------------");
        }
    }
}
