<?php

namespace App\Console\Commands\ShiftCrons;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Carbon;
use App\Jobs\AdminPanel\SendEmailJob;
use App\Interfaces\ShiftInterface;
use App\Interfaces\ShiftActivityInterface;
use App\Interfaces\ShiftTransferInterface;
use App\Interfaces\ShiftSwapRequestInterface;
use App\Models\CronJob;
use Mail;

class ShiftStartCron extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'shiftStartCron:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Hcp shift start cron';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct(ShiftInterface $shift, ShiftActivityInterface $shiftActivity, ShiftTransferInterface $shiftTransfer, ShiftSwapRequestInterface $shiftSwapRequest)
    {
        parent::__construct();
        $this->shift = $shift;
        $this->shift_activity = $shiftActivity;
        $this->shift_transfer = $shiftTransfer;
        $this->shift_swap_request = $shiftSwapRequest;
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        config(['logging.channels.cronlog.path' => storage_path('logs/cron-logs/shift-cron.log')]);

        // Current date
        $currentDate = Carbon::now()->toDateString();
        $currentTime = Carbon::now();
        //-------------

        $shiftUpdate = null;
        $shiftActivity = null;
        $distance = null;
        $hcpReached = false;

        // Get cron status
        $cronJob = CronJob::where('name', '=', 'shift-start-cron')->where('status', 0)->first();
        //----------------

        if (!empty($cronJob)) {

            // Update cron status in processing
            $cronJob->status = 1;
            $cronJob->start_time = Carbon::now()->toDateTimeString();
            $cronJob->save();
            //---------------------------------

            DB::beginTransaction();
            try {

                // General Configuration
                $generalConfiguration = get_general_configuration();
                $radius = $generalConfiguration['radius_km_for_shift_start'];
                //----------------------

                // Get Shifts
                $shiftWhere = ['shift_status' => '1'];
                $shifts = $this->shift->get_shift_for_cron($shiftWhere);
                //-----------

                if(count($shifts)){
                    foreach($shifts as $shift){
                        // Set time zone
                        date_default_timezone_set($shift->state->time_zone_name);
                        //--------------

                        // Current date
                        $currentDate = Carbon::now($shift->state->time_zone_name)->toDateString();
                        $currentTime = Carbon::now($shift->state->time_zone_name);
                        //-------------

                        if (!empty($shift['shift_hcp_loaction'])) {
                            $lat1 = $shift['facility']['latitude'];
                            $lon1 = $shift['facility']['longitude'];
                            $lat2 = $shift['shift_hcp_loaction']['latitude'];
                            $lon2 = $shift['shift_hcp_loaction']['longitude'];
                            $unit = 'M';
                            $distance = get_facility_and_hcp_distance($lat1, $lon1, $lat2, $lon2, $unit);

                            $shiftTime = Carbon::parse($shift->start_time)->setTimezone($shift->state->time_zone_name)->subMinutes(5);

                            if(!empty($distance) && $shift->hcp_reached_at_location_notification == 'no' && $currentDate == Carbon::parse($shift->start_time)->toDateString() && $distance <= $radius && strtotime($currentTime) >= strtotime($shiftTime)){

                                if (!empty($shift['hcp_id']) && $shift->hcp_reached_at_location_notification == 'no') {
                                    $hcpId = $shift['hcp_id'];
                                    send_shift_push_notification($shift->id, 'hcp-reached-at-location', $hcpId);
                                }

                                if (!empty($shift['hcp_id']) && $shift->hcp_reached_at_location_notification == 'no') {
                                    // Shift Activity
                                    $shiftActivityData = [
                                        'hcp_id'        => $shift->hcp_id,
                                        'shift_id'      => $shift->id,
                                        'activity'      => 'hcp-reached-at-location',
                                        'latitude'      => $lat2,
                                        'longitude'     => $lon2,
                                        'created_by'    => $shift->created_by,
                                        'updated_by'    => $shift->updated_by,
                                    ];
                                    $shiftActivity = $this->shift_activity->store($shiftActivityData, ['id' => $shift->id]);
                                    //---------------
                                }

                                $shiftDataNew = ['hcp_reached_at_location_notification' => 'yes'];
                                $this->shift->update($shiftDataNew, ['id' => $shift->id]);

                                Log::channel('cronlog')->info("-----------------------");
                                Log::channel('cronlog')->info("shift hcp reached at facility location, distance: ".$distance);
                                Log::channel('cronlog')->info("-----------------------");
                                $hcpReached = false;
                            } else {
                                $hcpReached = false;
                            }
                        }

                        if($hcpReached == true && $shift->start_time <= $currentTime) {
                            Log::channel('cronlog')->info("-----------------------");
                            Log::channel('cronlog')->info("current time: ". $currentTime);
                            Log::channel('cronlog')->info("shift start time: ". $shift->start_time);

                            // Check Shift Transfer Request
                            $shiftTransfer = $this->shift_transfer->find_shift_transfer(['shift_id' => $shift->id, 'status' => 'pending']);
                            if(!empty($shiftTransfer)){

                                $shiftTransferUpdate = $this->shift_transfer->update(['status' => 'cancelled'], ['id' => $shiftTransfer['id']]);

                                if (!empty($shift['hcp_id'])) {
                                    $hcpId = $shift['hcp_id'];
                                    send_shift_push_notification($shift->id, 'shift-transfer-cancelled', $hcpId);
                                }

                                Log::channel('cronlog')->info("shift id: ". $shift->shift_prefix.$shift->id.' transfer request cancelled.');
                            }
                            //-----------------------------

                            // Check Shift Swap Request
                            $shiftSwap = $this->shift_swap_request->find_shift_swap_request(['shift_id' => $shift->id, 'status' => 'pending']);
                            if(!empty($shiftSwap)){
                                $this->shift_swap_request->update(['status' => 'cancelled'], ['id' => $shiftSwap->id]);

                                if (!empty($shift['hcp_id'])) {
                                    $hcpId = $shift['hcp_id'];
                                    send_shift_push_notification($shift->id, 'shift-swap-cancelled', $hcpId);
                                }

                                Log::channel('cronlog')->info("shift id: ". $shift->shift_prefix.$shift->id.' swap request cancelled.');
                            }
                            //-------------------------

                            // Update Shift
                            $shiftData = ['shift_status' => '2', 'hcp_shift_start_time' => $currentTime];
                            $shiftUpdate = $this->shift->update($shiftData, ['id' => $shift->id]);
                            //-------------

                            // Shift Activity
                            $shiftActivityData = [
                                'hcp_id'        => $shift->hcp_id,
                                'shift_id'      => $shift->id,
                                'activity'      => 'shift-started',
                                'created_by'    => $shift->created_by,
                                'updated_by'    => $shift->updated_by,
                            ];
                            $shiftActivity = $this->shift_activity->store($shiftActivityData, ['id' => $shift->id]);
                            //---------------

                            if (!empty($shift['hcp_id'])) {
                                $hcpId = $shift['hcp_id'];
                                send_shift_push_notification($shift->id, 'shift-start', $hcpId);
                            }

                            Log::channel('cronlog')->info("shift id: ". $shift->shift_prefix.$shift->id.' started');
                            Log::channel('cronlog')->info("shift start cron completed");
                            Log::channel('cronlog')->info("-----------------------");
                        }
                    }
                }

                DB::commit();

            } catch (\Exception $e) {
                Log::channel('cronlog')->info("-----------------------");
                Log::channel('cronlog')->error('shift start cron error: '.$e->getMessage());
                Log::channel('cronlog')->info("-----------------------");
            }

            // Update cron status to completed
            $cronJob->status = 0;
            $cronJob->end_time = Carbon::now()->toDateTimeString();
            $cronJob->save();
            //---------------------------------

        } else {
            Log::channel('cronlog')->info("-----------------------");
            Log::channel('cronlog')->warning('shift start cron already running...');
            Log::channel('cronlog')->info("-----------------------");
        }
    }
}
