<?php

namespace App\Http\Controllers\API\Auth;

use App\Http\Controllers\Controller;
use App\Http\Requests\API\Auth\ForgotPassword;
use App\Http\Requests\API\Auth\Login;
use App\Http\Requests\API\Auth\SocialLogin;
use App\Http\Requests\API\UuidToken;
use App\Jobs\AdminPanel\SendEmailJob;
use App\Http\Traits\GetUserData;
use App\OneTimePassword;
use App\Role;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Mail;
use Mockery\Expectation;

class LoginController extends Controller
{
    use GetUserData;

    /**
     * Create an controller instance.
     */
    public function __construct()
    {
        $this->middleware('auth:api')->only(['logout', 'verifyToken']);
    }

    /**
     * Authenticate user.
     *
     * @param  Login  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function authenticate(Login $request)
    {
        $responseCode = 200;

        // Set credentials
        $credentials = [
            'email' => $request['email'],
            'password' => $request['password'],
        ];
        //----------------

        // Attempt login
        if (auth()->attempt($credentials)) {
            $user = auth()->user();

            if ($user->hcp->status != 'archived' && $user->role_name == 'hcp') {

                $token = $user->createToken(config('app.key'));
                // Find Device Id
                User::where('device_id', $request['device_id'])->update(['device_id' => null]);
                //---------------

                // Update login
                $user->last_login_at = Carbon::now()->toDateTimeString();
                $user->last_login_ip = $request->getClientIp();
                $user->device_id = $request['device_id'];
                $user->device_type = $request->header('device-type') ?? null;
                $user->device_name = $request->header('device-name') ?? null;
                $user->device_version = $request->header('device-version') ?? null;
                $user->save();
                //-------------

                // Set response
                $response = [
                    'status' => true,
                    'message' => __('messages.logged_in'),
                    'data' => [
                        'access_token' => $token->accessToken,
                        'user' => $this->getUserAllData($user->fresh(), true),
                    ],
                ];
                //--------------
            } else {
                // Set response
                $response = [
                    'status' => false,
                    'message' => __('messages.account_inactive'),
                    'data' => null,
                ];
                //-------------
            }
        } else {
            // Set response
            $response = [
                'status' => false,
                'message' => __('messages.login_failed'),
                'data' => null,
                'errors' => [
                    'email' => [
                        __('messages.login_failed'),
                    ],
                ],
            ];
            //--------------

            // Set response code
            $responseCode = 422;
            //------------------
        }
        //--------------

        return response()->json($response, $responseCode);
    }

    /**
     * Logout user.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function logout()
    {
        $user = auth()->user();

        // Find Device Id
        User::where('id', $user->id)->update(['device_id' => null]);
        //---------------

        // Logout user and set response
        if (auth()->user()->token()->delete()) {
            $response = [
                'status' => true,
                'message' => __('messages.logged_out'),
                'data' => null,
            ];
        } else {
            $response = [
                'status' => false,
                'message' => __('messages.logging_out_failed'),
                'data' => null,
            ];
        }
        //-----------------------------

        return response()->json($response, 200);
    }

    /**
     * Verify token.
     *
     * @param  Request  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function verifyToken(Request $request)
    {
        // Set response
        $response = [
            '_status' => true,
            '_message' => __('messages.token_verified'),
            '_data' => [
                'access_token' => $request['token'],
            ],
        ];
        //-------------

        return response()->json($response, 200);
    }

    /**
     * Get token.
     *
     * @param  mixed  $user
     * @return mixed
     */
    public function getToken($user)
    {
        auth()->login($user);

        $token = auth()->user()->createToken(config('app.key'));

        return $token->accessToken;
    }

    /**
     * forgot password
     *
     * @param email
     * @return $response
     * @author Sumit
     * @created_at 29 Sep 2020
     */
    public function forgotPassword(ForgotPassword $request)
    {
        $user = null;
        $one_time_password = null;
        $email_one_time_password = true;
        $randomPassword = str_random(8);
        $errorMessage = null;

        // Set where
        $where = [
            'email' => $request['email'],
        ];
        //----------

        try {

            // Get user
            $user = User::where($where)->first();
            //---------

            if(!empty($user)) {

                // Update password
                $user->password = bcrypt($randomPassword);
                $user->save();
                //----------------

                $emailData = [];
                $emailData['message'] = __('messages.forgot_password_mail_message');
                $emailData['subject'] = __('subjects.forgot_password');
                $emailData['sender'] = __('messages.forgot_password_mail_sender');
                $emailData['email'] = $user->email;
                $emailData['receiver'] = $user->name;
                $emailData['password'] = $randomPassword;

                // Send forgot password email
                $email_data = [
                    'template' => 'emails.forgot-password-email',
                    'data' => $emailData,
                ];


                if(config('app.server') == 'AWS' ) {
                    SendEmailJob::dispatch($email_data)->delay(now()->addSeconds(5));
                } else {
                    $data = $email_data['data'];
                    Mail::send($email_data['template'], $data, function ($message) use ($data) {
                        $message->to($data['email']);
                        $message->subject($data['subject']);
                    });
                }
                //---------------------------


            } else {
                $user = null;
            }

        } catch (\Exception $e) {
            $errorMessage = $e->getMessage();
            $user = null;
        }

        if (!is_null($user)) {

            // Set response
            $response = [
                'status' => true,
                'message' => __('messages.forgot_password_email_sent'),
                'data' => null,
            ];
            //-------------

        } elseif (is_null($user) && is_null($errorMessage)) {
            // Set response
            $response = [
                'status' => false,
                'message' => __('messages.email_address_not_exists'),
                'data' => null,
            ];
            //-------------
        } else {
            // Set response
            $response = [
                'status' => false,
                'message' => __('messages.email_something_went_wrong'),
                'data' => null,
            ];
            //-------------
        }

        return response()->json($response, 200);
    }

    /**
     * To guest user login
     * @param uuid_token
     * @return access_token
     * @author Sumit
     * @created_at 29 Sep 2020
     */
    public function guestLogin(UuidToken $request)
    {
        try {
            $user = User::where('uuid', $request['uuid_token'])->where(function ($query) use ($request) {
                $query->where('email', null);
                $query->where('mobile_number', null);
                $query->where('social_id', null);
            })->first();

            if (is_null($user)) {
                $user = new User;
            }

            $user->uuid = $request['uuid_token'];
            $user->platform = $request['platform'];
            $user->save();

            // Get student role
            $role = Role::where([
                'type' => config('constants.users.types.STUDENT.value'),
                'status' => config('constants.statuses.ACTIVE.value'),
            ])->first();
            //-----------------

            // Assign student role to user
            if (!is_null($user) && !is_null($role)) {
                $user->assignRole($role->id);
            }
            //----------------------------

            $response = [
                '_status' => true,
                '_message' => __('messages.success'),
                '_data' => [
                    'access_token' => $this->getToken($user),
                ],
            ];
        } catch (Expectation $e) {
            $response = [
                '_status' => false,
                '_message' => __('messages.something_went_wrong'),
                '_data' => null,
            ];
        }

        return response()->json($response, 200);
    }
}
