<?php

namespace App\Http\Controllers\API\Auth;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Laravel\Socialite\Facades\Socialite;
use Illuminate\Support\Carbon;
use App\User;
use App\Role;

class SocialLoginController extends Controller
{
    /**
     * Create an controller instance.
     */
    public function __construct()
    {
        //
    }

    /**
     * Redirect user to service (Google, Facebook).
     * 
     * @return Laravel\Socialite\Facades\Socialite;
     */
    public function redirect()
    {
        // Check if valid service, then use the service
        if ($this->isValidService(request('service'))) {
            return Socialite::driver(request('service'))->stateless()->redirect();
        }
        //---------------------------------------------

        return redirect(env('APP_URL').'social-callback?error='.title_case(request('service')).' service not available. Please try again!');
    }

    /**
     * Get the token and return to the client app the token.
     * 
     * @return mixed 
     */
    public function callback()
    {
        $user        = null;
        $social_user = null;
        $token       = null;

        try {
            // Get user
            $social_user = Socialite::driver(request('service'))->stateless()->user();
            //---------

            // Get user
            $user = $this->getUser($social_user->getEmail());
            //---------
        } catch (\Exception $e) {
            $user        = null;
            $social_user = null;

            return redirect(env('APP_URL').'social-callback?error=Unable to login using '.request('service').'. Please try again!');
        }

        // Get token
        if (is_null($user)) {
            $token = $this->createUser($social_user);
        } else {
            // Update user
            $user_updated = $this->updateUser($user, $social_user);
            //------------

            if ($user_updated) {
                $token = $this->getToken($user);
            }
        }
        //----------

        if (! is_null($token)) {
            return redirect()->to(env('APP_URL').'social-callback?token='.$token);
        }

        return redirect()->to(env('APP_URL').'social-callback?error=Unable to login in server. Please try again!');
    }

    /**
     * Get user.
     * 
     * @return object
     */
    private function getUser($email)
    {
        $user = User::where('email', $email)->first();

        return $user; 
    }

    /**
     * Create user.
     * 
     * @return mixed
     */
    private function createUser($social_user)
    {
        $new_user     = null;
        $access_token = null;

        try {
            // Store user 
            $new_user = User::create([
                'name'                     => $social_user->name,
                'email'                    => $social_user->email,
                'email_verified_at'        => Carbon::now()->toDateTimeString(),
                'password'                 => null,
                'social_id'                => $social_user->id,
                'social_image_url'         => $social_user->avatar,
                'account_service_provider' => request('service'),
                'platform'                 => config('constants.platforms.WEB.value'),
            ]);
            //-----------

            // Get role
            $role = Role::where([
                'type'   => config('constants.users.types.STUDENT.value'),
                'status' => config('constants.statuses.ACTIVE.value')
            ])->first();
            //---------

            // Assign role
            if (! is_null($new_user) && ! is_null($role)) {
                $new_user->assignRole($role->id);
            }
            //-----------
        } catch (\Exception $e) {
            $new_user = null;
        }

        if (! is_null($new_user)) {
            auth()->login($new_user); 

            $token = auth()->user()->createToken(config('app.key'));

            $access_token = $token->accessToken;
        }

        return $access_token;
    }

    /**
     * Update user.
     * 
     * @param  object  $user
     * @param  object  $social_user
     * @return boolean
     */
    private function updateUser($user, $social_user)
    {
        $updated = User::where('id', $user->id)->update([
            'name'                     => $social_user->name,
            'email'                    => $social_user->email,
            'email_verified_at'        => Carbon::now()->toDateTimeString(),
            'social_id'                => $social_user->id,
            'social_image_url'         => $social_user->avatar,
            'account_service_provider' => request('service'),
            'platform'                 => config('constants.platforms.WEB.value')
        ]);
        
        return $updated;
    }

    /**
     * Get token.
     * 
     * @param  mixed  $user
     * @return mixed
     */
    public function getToken($user)
    {
        auth()->login($user);
        
        $token = auth()->user()->createToken(config('app.key'));

        return $token->accessToken;
    }

    /**
     * Is valid service.
     */
    private function isValidService($service)
    {
        $is_valid = false;

        // Check if service is valid
        if (
            key_exists($service, array_column(
                config('constants.account_service_providers'), 'caption', 'value')
            )
        ) {

            $is_valid = true;
        }
        //--------------------------

        return $is_valid;
    }
}
