<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Http\Requests\API\Hcp\ValidateHcpRegister;
use App\Interfaces\CountryInterface;
use App\Interfaces\CityInterface;
use App\Interfaces\StateInterface;
use App\Interfaces\SuburbInterface;
use App\Interfaces\CategoryInterface;
use App\Interfaces\ProfessionInterface;
use App\Interfaces\LevelInterface;
use App\Interfaces\DocumentInterface;
use App\Interfaces\GeneralConfigInterface;
use App\Interfaces\SupportingPageInterface;
use App\Interfaces\HcpProfessionInterface;
use App\Models\DevConfiguration;
use App\Models\LmsUrl;
use Illuminate\Support\Facades\Hash;
use Carbon\Carbon;
use App\Models\Shift;

class CommonContentController extends Controller
{
    private $country, $state, $city, $suburb, $category, $level, $document;

    /**
     * Create an controller instance.
     */
    public function __construct(CountryInterface $country, StateInterface $state, CityInterface $city, SuburbInterface $suburb, CategoryInterface $category, ProfessionInterface $profession, LevelInterface $level, DocumentInterface $document, GeneralConfigInterface $generalConfiguration, SupportingPageInterface $supportingPage, HcpProfessionInterface $hcpProfession){
        $this->country = $country;
        $this->state = $state;
        $this->city = $city;
        $this->suburb = $suburb;
        $this->category = $category;
        $this->profession = $profession;
        $this->level = $level;
        $this->document = $document;
        $this->general_configuration = $generalConfiguration;
        $this->supporting_page = $supportingPage;
        $this->hcp_profession = $hcpProfession;

        $this->middleware('auth:api')->except(['getCountries', 'getStates', 'getCities', 'getSuburbs', 'getCategories', 'getLevels', 'getCommonContent','getShiftsAndHoursCompleted']);
    }

    /**
     * Get countries.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getCountries(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $countries = null;
        $where = [];

        DB::beginTransaction();
        try {

            $countries = $this->country->getAll($where)->toArray();

            DB::commit();
        } catch (\Exception $e) {
            $countries = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($countries)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Country']),
                'data' => [
                    'countries'  => $countries,
                ],
            ];

        } else if(empty($countries) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Country']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get states.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getStates(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $states = null;
        $where = [];

        DB::beginTransaction();
        try {

            if(!empty($request['country_id'])){
                $where = ['country_id' => $request['country_id']];
            }
            $states = $this->state->api_get_states($where)->toArray();

            DB::commit();
        } catch (\Exception $e) {
            $states = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($states)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'State']),
                'data' => [
                    'states'  => $states,
                ],
            ];

        } else if(empty($states) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'State']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get cities.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getCities(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $cities = null;
        $where = [];

        DB::beginTransaction();
        try {

            if(!empty($request['state_id'])){
                $where = ['state_id' => $request['state_id']];
            }
            $cities = $this->city->getAll($where, true)->toArray();

            DB::commit();
        } catch (\Exception $e) {
            $cities = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($cities)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'City']),
                'data' => [
                    'cities'  => $cities,
                ],
            ];

        } else if(empty($cities) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'City']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get suburbs.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getSuburbs(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $suburbs = null;
        $where = [];

        DB::beginTransaction();
        try {

            if(!empty($request['city_id'])){
                $where = ['city_id' => $request['city_id']];
            }
            $suburbs = $this->suburb->getAll($where, true)->toArray();

            DB::commit();
        } catch (\Exception $e) {
            $suburbs = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($suburbs)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Suburb']),
                'data' => [
                    'suburbs'  => $suburbs,
                ],
            ];

        } else if(empty($suburbs) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Suburb']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get categories.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getCategories(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $categories = null;
        $where = [];

        DB::beginTransaction();
        try {

            $where = ['status' => 'active'];
            $categories = $this->category->getAll($where, true)->toArray();

            DB::commit();
        } catch (\Exception $e) {
            $categories = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($categories)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Category']),
                'data' => [
                    'categories'  => $categories,
                ],
            ];

        } else if(empty($categories) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Category']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get professions.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getProfessions(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $professions = null;
        $where = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Get Hcp Profession
            $hcpProfessionRecord = $this->hcp_profession->getAll(['hcp_id' => $user->hcp->id], true);
            //-------------------

            if(count($hcpProfessionRecord)) {
                $request['hcp_profession_ids'] = $hcpProfessionRecord->pluck('profession_id');
            }

            if(!empty($request['category_id'])){
                $where['category_id'] = $request['category_id'];
            }

            // Request
            $request = $request->all();
            //--------

            $where['status'] = 'active';
            $professions = $this->profession->getAll($where, true, $request)->toArray();

            DB::commit();
        } catch (\Exception $e) {
            $professions = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($professions)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Profession']),
                'data' => [
                    'professions'  => $professions,
                ],
            ];

        } else if(empty($professions) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Profession']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get levels.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getLevels(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $levels = null;
        $where = [];

        DB::beginTransaction();
        try {

            $where = [
                'category_id' => $request['category_id'] ?? null,
                'profession_id' => $request['profession_id'] ?? null,
                'status' => 'active'
            ];

            $levels = $this->level->getAll($where, true)->toArray();

            DB::commit();
        } catch (\Exception $e) {
            $levels = null;
            $errorMessage = $e->getMessage();

            DB::rollback();
        }

        // Set response
        if (!empty($levels)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Level']),
                'data' => [
                    'levels'  => $levels,
                ],
            ];

        } else if(empty($levels) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Level']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get common content.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getCommonContent(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $document = null;
        $commonContent = null;
        $documentWhere = [];
        $imagePath = [];

        DB::beginTransaction();
        try {

            $documentWhere = [['required_for', '=', 'hcp'], ['doc_type', '!=', 'profession']];
            $documents = $this->document->get($documentWhere, true)->toArray();

            // Get image path
            if (config('filesystems.default') == config('constants.file_systems.PUBLIC.value')) {

                $imagePath['hcp_path'] = url('public/storage/' . config('constants.hcps.hcps_path')) . '/';
                $imagePath['supporting_page_path'] = url('public/storage/' . config('constants.supporting_pages.supporting_pages_file_path')) . '/';
                $imagePath['invoice_path'] = url('public/storage/' . config('constants.hcps.invoice_path')) . '/';

                $imagePath['facility_path'] = url('public/storage/' . config('constants.facility.facility_path')) . '/';

            } elseif (config('filesystems.default') == config('constants.file_systems.S3.value')) {
                $bucket = env('AWS_BUCKET');
                $region = env('AWS_DEFAULT_REGION');

                $bucketBaseUrl = 'https://s3.' . $region . '.amazonaws.com/'. $bucket.'/';

                $S3NewURL = env('AWS_CloudFront_URL').'/';

                $imagePath['hcp_path'] = $S3NewURL.config('constants.hcps.hcps_path');
                $imagePath['supporting_page_path'] = $S3NewURL.config('constants.supporting_pages.supporting_pages_file_path');
                $imagePath['invoice_path'] = $S3NewURL.config('constants.hcps.invoice_path');
                $imagePath['facility_path'] = $S3NewURL.config('constants.facility.facility_path');
            }
            //---------------

            // Get Config Data
            $generalConfiguration = $this->general_configuration->get_app_config();
            //----------------

            // Get Dev Config Data
            $devConfiguration = DevConfiguration::all();
            //---------------------

            // Get Supporting Page
            $where = [];
            $supportingPage = $this->supporting_page->getAll($where);
            //--------------------

            // Get LMS
            $lms = LmsUrl::first();
            //----------

            $commonContent = true;

            DB::commit();
        } catch (\Exception $e) {
            $commonContent = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($commonContent)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Common Content']),
                'data' => [
                    'documents'  => $documents,
                    'image_path'  => $imagePath,
                    'configuration'  => $generalConfiguration,
                    'supporting_page'  => $supportingPage[0],
                    'dev_configuration'  => $devConfiguration,
                    'lms'  => $lms,
                ],
            ];

        } else if(empty($commonContent) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Common Content']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }


    public function getShiftsAndHoursCompleted(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $shifts = 0;
        $hours = 0;
        $where = [];

        DB::beginTransaction();
        try {

            $shifts = Shift::where('shift_status',4)->count();
            $hours = Shift::where('shift_status',4)->sum('hours');

            DB::commit();
        } catch (\Exception $e) {
            $shifts = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($shifts)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Record']),
                'data' => [
                    'completed_shifts'  => $shifts,
                    'completed_hours'  => $hours.' Hours',
                ],
            ];

        } else if(empty($shifts) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Record']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }
}
