<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Http\Requests\API\Hcp\ValidateHcpRegister;
use App\Http\Requests\API\Hcp\ValidatePersonalDetails;
use App\Http\Requests\API\Hcp\ValidateRegistrationNumber;
use App\Http\Requests\API\Hcp\ValidateAddUpdateDocument;
use App\Http\Requests\API\Hcp\ValidateAddUpdateProfession;
use App\Http\Requests\API\Hcp\ValidateAvailableForJobStatus;
use App\Http\Requests\API\Hcp\ValidateHcpDocuments;
use App\Http\Requests\API\Hcp\ValidateUpdateDeviceId;
use App\Http\Requests\API\Hcp\ValidateAddUpdateProfessionNew;
use App\Http\Requests\API\Hcp\ValidateGetProfessionDocuments;
use App\Http\Requests\API\Hcp\ValidateAddUpdateProfessionDocument;
use App\Http\Requests\API\Hcp\ValidateUpdateHcpProfessionStatus;
use App\Http\Requests\API\Hcp\ValidateAccountDetails;
use App\Http\Requests\API\Hcp\ValidateDeleteHcpProfession;
use App\Http\Requests\API\Hcp\ValidateUpdateAppRegistrationScreen;
use App\Http\Requests\API\Hcp\ValidateSendEmailVerification;
use App\Http\Requests\API\Hcp\ValidateSendMobileVerification;
use App\Http\Requests\API\Hcp\ValidateVerify;
use App\Http\Requests\API\Hcp\ValidateChangePassword;
use App\Interfaces\HcpInterface;
use App\Interfaces\DocumentHcpInterface;
use App\Interfaces\UserInterface;
use App\Interfaces\NotificationInterface;
use App\Interfaces\ReferralInterface;
use App\Interfaces\HcpProfessionInterface;
use App\Interfaces\ShiftInterface;
use App\Interfaces\TransactionInterface;
use App\Interfaces\HcpActivityInterface;
use App\Interfaces\ProfessionInterface;
use App\Interfaces\CategoryInterface;
use App\Interfaces\LevelInterface;
use App\Interfaces\HcpProfessionRequestInterface;
use App\Interfaces\RatingReviewInterface;
use App\Interfaces\FacilityInterface;
use Illuminate\Support\Facades\Hash;
use App\Jobs\AdminPanel\SendEmailJob;
use App\Http\Traits\UploadImage;
use App\Http\Traits\UploadFile;
use Mail;
use Illuminate\Support\Str;
use Carbon\Carbon;
use Twilio\Rest\Client;
use App\Models\OneTimePassword;
use App\Models\Document;
use App\Models\DocumentHcp;

class HCPController extends Controller
{
    use UploadImage, UploadFile;

    private $hcp, $user, $documentHcp, $notification, $referral, $hcpProfession, $transaction, $shift, $hcpActivity;

    /**
     * Create an controller instance.
     */
    public function __construct(HcpInterface $hcp, UserInterface $user, DocumentHcpInterface $documentHcp, NotificationInterface $notification, ReferralInterface $referral, HcpProfessionInterface $hcpProfession, ShiftInterface $shift, TransactionInterface $transaction, HcpActivityInterface $hcpActivity, ProfessionInterface $profession, CategoryInterface $category, LevelInterface $level, HcpProfessionRequestInterface $hcpProfessionRequest, RatingReviewInterface $ratingReview, FacilityInterface $facility){
        $this->hcp = $hcp;
        $this->user = $user;
        $this->document_hcp = $documentHcp;
        $this->notification = $notification;
        $this->referral = $referral;
        $this->hcp_profession = $hcpProfession;
        $this->shift = $shift;
        $this->transaction = $transaction;
        $this->hcp_activity = $hcpActivity;
        $this->profession = $profession;
        $this->category = $category;
        $this->level = $level;
        $this->hcp_profession_request = $hcpProfessionRequest;
        $this->ratingReview = $ratingReview;
        $this->facility = $facility;
        $this->middleware('auth:api')->except(['register', 'sendEmailVerification', 'verifyEmail', 'sendMobileVerification', 'verifyMobile']);
    }

    /**
     * Register hcp.
     *
     * @param  ValidateRegister  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function register(ValidateHcpRegister $request)
    {
        $responseCode = 200;
        $user = null;
        $hcp = null;
        $referral = null;
        $errorMessage = null;
        $userData = [];
        $hcpData = [];
        $referralData = [];
        $referrerHcp = [];
        $customMessage = null;

        DB::beginTransaction();
        try {

            if(!empty($request['invitation_code'])) {
                $referrerHcp = $this->hcp->getAll(['referral_code' => $request['invitation_code']], true);
                if (count($referrerHcp)) {
                    // General Configuration
                    $generalConfiguration = get_general_configuration();
                    //----------------------

                    // Find Device Id
                    $this->user->update(['device_id' => null], ['device_id' => $request['device_id']]);
                    //---------------

                    $userData = [
                        'email'     => $request['email'],
                        'password'  => bcrypt($request['password']),
                        'device_id' => $request['device_id'],
                        'role_name' => 'hcp',
                        'device_type' => $request->header('device-type') ?? null,
                        'device_name' => $request->header('device-name') ?? null,
                        'device_version' => $request->header('device-version') ?? null,
                    ];

                    $user = $this->user->store($userData);

                    // Assign role
                    $user->assignRole('HCP');
                    //------------

                    $hcpData = [
                        'user_id'   => $user->id,
                        'registered_date'  => date('Y-m-d'),
                        'referral_code'  => Str::upper(generate_random_string(8)),
                        'show_approved_screen'  => 1,
                        'agree_terms_and_conditions'  => $request['agree_terms_and_conditions'],
                        'points'  => $generalConfiguration['hcp_signup_awarded_points'],
                        'maximum_distance'  => $generalConfiguration['radius_km_for_shift_broadcast'],
                    ];

                    $hcp = $this->hcp->store($hcpData);

                    if(!empty($request['invitation_code'])) {
                        if(count($referrerHcp)){
                            $referralData = [
                                'referrer_hcp_id'  => $referrerHcp[0]->id,
                                'referee_hcp_id'   => $hcp->id,
                                'referrer_amount'  => $generalConfiguration['referrer_amount'],
                                'referee_amount'   => $generalConfiguration['referee_amount'],
                                'referral_date'    => Carbon::now()->toDateString(),
                            ];
                            $referral = $this->referral->store($referralData);
                        }
                    }

                    // Update App Registration Screen
                    if (isset($request['app_registration_screen']) && $request['app_registration_screen'] != '') {
                        if ($request['app_registration_screen'] == 0) {
                            $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                            $hcpUpdate = $this->hcp->update($hcpData, ['id' => $hcp->id]);
                        } else if($request['app_registration_screen'] > $hcp->app_registration_screen) {
                            $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                            $hcpUpdate = $this->hcp->update($hcpData, ['id' => $hcp->id]);
                        } else if($hcp->app_registration_screen == null || $hcp->app_registration_screen == '') {
                            $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                            $hcpUpdate = $this->hcp->update($hcpData, ['id' => $hcp->id]);
                        }
                    }
                    //-------------------------------

                    // Hcp Activity
                    create_hcp_activity($hcp->id, 'account-created', __('hcp_activities.account_created'), $user->id);
                    //-------------

                    $token = $user->createToken(config('app.key'));

                    // Send Admin Notification
                    send_admin_notification('admin-new-hcp-registered', 'admin', null, $hcp->id);
                    //------------------------
                } else {
                    $customMessage = __('messages.invitation_code_invalid');
                    $hcp = null;
                }
            } else {

                // General Configuration
                $generalConfiguration = get_general_configuration();
                //----------------------

                // Find Device Id
                $this->user->update(['device_id' => null], ['device_id' => $request['device_id']]);
                //---------------

                $userData = [
                    'email'     => $request['email'],
                    'password'  => bcrypt($request['password']),
                    'device_id' => $request['device_id'],
                    'role_name' => 'hcp',
                ];

                $user = $this->user->store($userData);

                // Assign role
                $user->assignRole('HCP');
                //------------

                $hcpData = [
                    'user_id'   => $user->id,
                    'registered_date'  => date('Y-m-d'),
                    'referral_code'  => Str::upper(generate_random_string(8)),
                    'show_approved_screen'  => 1,
                    'agree_terms_and_conditions'  => $request['agree_terms_and_conditions'],
                    'points'  => $generalConfiguration['hcp_signup_awarded_points'],
                    'app_registration_screen'  => $request['app_registration_screen'],
                    'maximum_distance'  => $generalConfiguration['radius_km_for_shift_broadcast'],
                ];

                $hcp = $this->hcp->store($hcpData);
                $token = $user->createToken(config('app.key'));

                // Update App Registration Screen
                if (isset($request['app_registration_screen']) && $request['app_registration_screen'] != '') {
                    if ($request['app_registration_screen'] == 0) {
                        $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                        $hcpUpdate = $this->hcp->update($hcpData, ['id' => $hcp->id]);
                    } else if($request['app_registration_screen'] > $hcp->app_registration_screen) {
                        $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                        $hcpUpdate = $this->hcp->update($hcpData, ['id' => $hcp->id]);
                    } else if($hcp->app_registration_screen == null || $hcp->app_registration_screen == '') {
                        $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                        $hcpUpdate = $this->hcp->update($hcpData, ['id' => $hcp->id]);
                    }
                }
                //-------------------------------

                // Hcp Activity
                create_hcp_activity($hcp->id, 'account-created', __('hcp_activities.account_created'), $user->id);
                //-------------

                // Send Admin Notification
                send_admin_notification('admin-new-hcp-registered', 'admin', null, $hcp->id);
                //------------------------
            }

            DB::commit();
        } catch (\Exception $e) {
            $user = null;
            $hcp = null;
            $customMessage = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcp)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_saved', ['records' => 'Hcp']),
                'data' => [
                    'hco_id'  => $hcp->id,
                    'access_token'  => $token->accessToken,
                ],
            ];

        } else if(is_null($hcp) && is_null($errorMessage) && is_null($customMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_saving_failed', ['records' => 'Hcp']),
                'data' => null,
            ];

        } else if(is_null($hcp) && is_null($errorMessage) && !empty($customMessage)) {

            $response = [
                'status' => false,
                'message' => $customMessage,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Update Personal Details.
     *
     * @param  ValidatePersonalDetails  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function updatePersonalDetails(ValidatePersonalDetails $request)
    {
        $responseCode = 200;
        $user = null;
        $hcp = null;
        $errorMessage = null;
        $userUpdate = null;
        $hcpUpdate = null;
        $userData = [];
        $hcpData = [];
        $imageName = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {
            $acceptLowerjobStatus = get_accept_lower_job_checkbox_status();

            // Update User
            $userData = [
                'name'  => $request['first_name'] . ' ' . $request['last_name'],
            ];
            $userUpdate = $this->user->update($userData, ['id' => $user->id]);
            //------------

            // Upload hcp image
            if ($request->hasFile('image')) {
                $path  = config('constants.hcps.hcps_path') . '/' . $user->hcp->hcp_prefix . $user->hcp->id . '/image/';
                $image = $this->uploadImage($request->file('image'), $path, 75, strtolower($request['first_name']));
                if ($image['_status']) {
                    $imageName = $image['_data'];
                }
            }
            //-----------------

            // Update Hcp
            $hcpData = [
                'first_name'  => $request['first_name'],
                'last_name'   => $request['last_name'],
                'country_id'  => $request['country'],
                'state_id'    => $request['state'],
                'address'     => $request['address'],
                'latitude'    => $request['latitude'],
                'longitude'   => $request['longitude'],
                'city_name'        => $request['city'],
                'suburb_name'      => $request['suburb'],
                'post_code'   => $request['post_code'],
                'about_me'    => $request['about_me'],
                'contact_number'    => $request['contact_number'],
                'maximum_distance'    => $request['maximum_distance'],
                'date_of_birth'  => $request['date_of_birth'],
            ];

            if(!is_null($imageName)){
                $hcpData['image'] = $imageName;
            }

            $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
            //-----------

            // Update Hcp
            if (isset($request['app_registration_screen']) && $request['app_registration_screen'] != '') {
                if ($request['app_registration_screen'] == 0) {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                } else if($request['app_registration_screen'] > $user->hcp->app_registration_screen) {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                } else if($user->hcp->app_registration_screen == null || $user->hcp->app_registration_screen == '') {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                }
            }
            //-------------

            // Check Accept Lower Level Job
            if($acceptLowerjobStatus['show_accept_lower_level_job_checkbox'] == '1') {
                $this->hcp->update(['accept_lower_level_job' => $request['accept_lower_level_job']], ['id' => $user->hcp->id]);
                if ($request['accept_lower_level_job'] == 1) {
                    $hcpProfessions = $this->hcp_profession->getAll(['hcp_id' => $user->hcp->id], true);
                    if (count($hcpProfessions)) {
                        foreach($hcpProfessions as $hcpProfession){
                            if (!is_null($hcpProfession->level_id)) {
                                $levelWhere = [
                                    'category_id' => $hcpProfession->category_id,
                                    'profession_id' => $hcpProfession->profession_id,
                                    ['id', '<=', $hcpProfession->level_id]
                                ];
                                $hcplevels = $this->level->getAll($levelWhere, true)->pluck('id');
                                if (count($hcplevels)) {
                                    $this->hcp_profession->update(['accept_level_ids' => implode(',', $hcplevels->toArray())], ['id' => $hcpProfession->id]);
                                }
                            }
                        }
                    }
                } else {
                    $hcpProfessions = $this->hcp_profession->getAll(['hcp_id' => $user->hcp->id], true);
                    if(count($hcpProfessions)) {
                        foreach($hcpProfessions as $hcpProfession){
                            if (!is_null($hcpProfession->level_id)) {
                                $this->hcp_profession->update(['accept_level_ids' => $hcpProfession->level_id], ['id' => $hcpProfession->id]);
                            }
                        }
                    }
                }
            }
            //-----------------------------

            // Hcp Activity
            create_hcp_activity($user->hcp->id, 'update-personal-details', __('hcp_activities.update_personal_details'), $user->id);
            //-------------

            DB::commit();
        } catch (\Exception $e) {
            $hcpUpdate = null;
            $userUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcpUpdate)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_saved', ['records' => 'Hcp']),
                'data' => [],
            ];

        } else if(is_null($hcpUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_saving_failed', ['records' => 'Hcp']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Update Registration Number.
     *
     * @param  ValidateRegistrationNumber  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function updateRegistrationNumber(ValidateRegistrationNumber $request)
    {
        $responseCode = 200;
        $hcp = null;
        $errorMessage = null;
        $userUpdate = null;
        $hcpUpdate = null;
        $cvName = null;
        $hcpData = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {
            $acceptLowerjobStatus = get_accept_lower_job_checkbox_status();

            // Upload hcp cv
            if ($request->hasFile('cv')) {
                $path  = config('constants.hcps.hcps_path') . '/' . $user->hcp->hcp_prefix . $user->hcp->id . '/cv/';
                $file = $this->uploadFile($request->file('cv'), $path, 75, strtolower('cv'));

                if ($file['_status']) {
                    $cvName = $file['_data'];
                }
            }
            //--------------

            // Update Hcp
            $hcpData = [
                'registration_number'  => $request['registration_number'],
                'tfn_number'  => $request['tfn_number'],
                'abn_number'  => $request['abn_number'],
                'cv'  => $cvName,
            ];
            $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
            //-----------

            // Update Hcp
            if (isset($request['app_registration_screen']) && $request['app_registration_screen'] != '') {
                if ($request['app_registration_screen'] == 0) {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                } else if($request['app_registration_screen'] > $user->hcp->app_registration_screen) {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                } else if($user->hcp->app_registration_screen == null || $user->hcp->app_registration_screen == '') {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                }
            }
            //-------------


            // Check Accept Lower Level Job
            if($acceptLowerjobStatus['show_accept_lower_level_job_checkbox'] == '1') {
                $this->hcp->update(['accept_lower_level_job' => $request['accept_lower_level_job']], ['id' => $user->hcp->id]);

                if ($request['accept_lower_level_job'] == 1) {
                    $hcpProfessions = $this->hcp_profession->getAll(['hcp_id' => $user->hcp->id], true);
                    if (count($hcpProfessions)) {
                        foreach($hcpProfessions as $hcpProfession){
                            if (!is_null($hcpProfession->level_id)) {
                                $levelWhere = [
                                    'category_id' => $hcpProfession->category_id,
                                    'profession_id' => $hcpProfession->profession_id,
                                    ['id', '<=', $hcpProfession->level_id]
                                ];
                                $hcplevels = $this->level->getAll($levelWhere, true)->pluck('id');
                                if (count($hcplevels)) {
                                    $this->hcp_profession->update(['accept_level_ids' => implode(',', $hcplevels->toArray())], ['id' => $hcpProfession->id]);
                                }
                            }
                        }
                    }
                } else {
                    $hcpProfessions = $this->hcp_profession->getAll(['hcp_id' => $user->hcp->id], true);
                    if(count($hcpProfessions)) {
                        foreach($hcpProfessions as $hcpProfession){
                            if (!is_null($hcpProfession->level_id)) {
                                $this->hcp_profession->update(['accept_level_ids' => $hcpProfession->level_id], ['id' => $hcpProfession->id]);
                            }
                        }
                    }
                }
            }
            //-----------------------------

            DB::commit();
        } catch (\Exception $e) {
            $hcpUpdate = null;
            $userUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcpUpdate)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_saved', ['records' => 'Hcp']),
                'data' => [],
            ];

        } else if(is_null($hcpUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_saving_failed', ['records' => 'Hcp']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Update Thank You Screen Status.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function updateThankYouScreenStatus(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $hcpUpdate = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Update Hcp
            $hcpData = ['show_approved_screen' => 0];
            $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
            //-----------

            DB::commit();
        } catch (\Exception $e) {
            $hcpUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcpUpdate)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_saved', ['records' => 'Hcp']),
                'data' => [],
            ];

        } else if(is_null($hcpUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_saving_failed', ['records' => 'Hcp']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Add/Update Document.
     *
     * @param  ValidateAddUpdateDocument  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function addUpdateDocument(ValidateAddUpdateDocument $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $hcpDocumentUpdate = null;
        $documentsData = [];
        $documentName = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            $documentHcp = $this->document_hcp->find_hcp_document($request['document_id'], $user->hcp->id);

            if(!empty($documentHcp)){
                $documentApproval = 'pending-reapproval';
            } else {
                $documentApproval = 'pending';
            }

            $name = strtolower(str_replace(' ', '-', $request['document_name']));

            // Upload hcp file
            if (!empty($request['document_file']) && file_exists($request['document_file'])) {
                $path = config('constants.hcps.hcps_path') . '/' . $user->hcp->hcp_prefix . $user->hcp->id . '/documents/';
                $file = $this->uploadFile($request['document_file'], $path, 75, $name);

                if ($file['_status']) {
                    $documentName = $file['_data'];
                }
            } else {
                $documentName = ($request['document_file'] ?? null);
            }
            //----------------

            if ( isset($request['expiry_date']) && !empty($request['expiry_date'])) {
                if (strtotime($request['expiry_date']) > strtotime(date('Y-m-d', strtotime(now() . ' + 60 days')))) {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date'] . ' - 60 days'));
                } else if(strtotime($request['expiry_date']) > strtotime(date('Y-m-d', strtotime(now() . ' + 30 days')))) {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date'] . ' - 30 days'));
                } else if (strtotime($request['expiry_date']) > strtotime(date('Y-m-d', strtotime(now() . ' + 7 days')))) {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date'] . ' - 7 days'));
                } else if (strtotime($request['expiry_date']) > strtotime(date('Y-m-d', strtotime(now() . ' + 1 days')))) {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date'] . ' - 1 days'));
                } else {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date']));
                }
            } else {
                $sendNotificationDate = null;
            }

            if (!empty($request['document_file'])) {
                $where = [
                    'document_id' => isset($request['document_id']) ? $request['document_id'] : null,
                    'hcp_id' => $user->hcp->id,
                ];

                $array = [
                    'document_name' => $documentName,
                    'expiry_date' => $request['expiry_date'] ? date('Y-m-d', strtotime($request['expiry_date'])) : null,
                    'send_notification_date' => $sendNotificationDate,
                    'document_approval' => $documentApproval,
                    'show_in_document_approval' => 'yes',
                    'show_in_notification' => 'no',
                    'created_by' => $user->id,
                    'updated_by' => $user->id,
                    'created_at' => now(),
                    'updated_at' => now(),
                ];

                $hcpDocumentUpdate = $this->document_hcp->update_or_create($where, $array);

                // Hcp Activity
                create_hcp_activity($user->hcp->id, 'mandatory-document-updated', __('hcp_activities.mandatory_document_updated', ['documentName' => $request['document_name']]), $user->id);
                //-------------

                // Update Hcp
                if(!empty($documentHcp) && !empty($documentHcp['document']['mandatory_status'] == 'yes')){
                    $this->hcp->update(['status' => 'pending-approval'], ['id' => $user->hcp->id]);
                }
                //-----------
            }

            // Update Hcp
            if (isset($request['app_registration_screen']) && $request['app_registration_screen'] != '') {
                if ($request['app_registration_screen'] == 0) {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                } else if($request['app_registration_screen'] > $user->hcp->app_registration_screen) {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                } else if($user->hcp->app_registration_screen == null || $user->hcp->app_registration_screen == '') {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                }
            }
            //-----------

            DB::commit();
        } catch (\Exception $e) {
            $hcpDocumentUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcpDocumentUpdate)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_saved', ['records' => 'Document']),
                'data' => [],
            ];

        } else if(is_null($hcpDocumentUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_saving_failed', ['records' => 'Document']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Add/Update Profession. (Old)
     *
     * @param  ValidateAddUpdateProfession  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function addUpdateProfession(ValidateAddUpdateProfession $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $hcpDocumentUpdate = null;
        $hcpProfessionUpdate = null;
        $documentName = null;
        $documentsData = [];
        $professionData = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Add/Update hcp professions
            $professionData = [];
            if (!empty($request['profession_id'])) {
                if (isset($request['id']) && $request['id'] !='') {
                    $where = [
                        'id' => isset($request['id']) ? $request['id'] : null,
                        'hcp_id' => $user->hcp->id,
                    ];
                } else {
                    $where = [
                        'profession_id' => $request['profession_id'],
                        'hcp_id' => $user->hcp->id,
                    ];
                }


                $array = [
                    'category_id' => $request['category_id'],
                    'profession_id' => $request['profession_id'],
                    'level_id'  => $request['level_id'],
                    'created_by' => $user->id,
                    'updated_by' => $user->id,
                    'created_at' => now(),
                    'updated_at' => now(),
                ];
                $hcpProfessionUpdate = $this->hcp_profession->update_or_create($where, $array);
            }
            //---------------------------

            // Add/Update hcp documents
            $documentHcp = $this->document_hcp->find_hcp_document($request['document_id'], $user->hcp->id);

            if(!empty($documentHcp)){
                $documentApproval = 'pending-reapproval';
            } else {
                $documentApproval = 'pending';
            }

            // Upload hcp file
            if (!empty($request['document_file']) && file_exists($request['document_file'])) {
                $name = strtolower(str_replace(' ', '-', $request['document_name']));
                $path = config('constants.hcps.hcps_path') . '/' . $user->hcp->hcp_prefix . $user->hcp->id . '/documents/';
                $file = $this->uploadFile($request['document_file'], $path, 75, $name);

                if ($file['_status']) {
                    $documentName = $file['_data'];
                }
            } else {
                $documentName = ($request['document_file'] ?? null);
            }
            //----------------

            if ( isset($request['expiry_date']) && !empty($request['expiry_date'])) {
                if (strtotime($request['expiry_date']) > strtotime(date('Y-m-d', strtotime(now() . ' + 60 days')))) {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date'] . ' - 60 days'));
                } else if(strtotime($request['expiry_date']) > strtotime(date('Y-m-d', strtotime(now() . ' + 30 days')))) {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date'] . ' - 30 days'));
                } else if (strtotime($request['expiry_date']) > strtotime(date('Y-m-d', strtotime(now() . ' + 7 days')))) {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date'] . ' - 7 days'));
                } else if (strtotime($request['expiry_date']) > strtotime(date('Y-m-d', strtotime(now() . ' + 1 days')))) {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date'] . ' - 1 days'));
                } else {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date']));
                }
            } else {
                $sendNotificationDate = null;
            }

            if (!empty($request['document_file'])) {
                $where = [
                    'document_id' => isset($request['document_id']) ? $request['document_id'] : null,
                    'profession_id' => $request['profession_id'],
                    'hcp_id' => $user->hcp->id,
                ];

                $array = [
                    'profession_id' => $request['profession_id'],
                    'document_name' => $documentName,
                    'expiry_date' => $request['expiry_date'] ?? null,
                    'send_notification_date' => $sendNotificationDate,
                    'document_approval' => $documentApproval,
                    'show_in_document_approval' => 'yes',
                    'show_in_notification' => 'no',
                    'created_by' => $user->id,
                    'updated_by' => $user->id,
                    'created_at' => now(),
                    'updated_at' => now(),
                ];

                $hcpDocumentUpdate = $this->document_hcp->update_or_create($where, $array);
            }
            //-------------------------

            // Update Hcp
            if (isset($request['app_registration_screen']) && $request['app_registration_screen'] != '') {
                if ($request['app_registration_screen'] == 0) {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                } else if($request['app_registration_screen'] > $user->hcp->app_registration_screen) {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                } else if($user->hcp->app_registration_screen == null || $user->hcp->app_registration_screen == '') {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                }
            }
            //-----------

            // Hcp Activity
            create_hcp_activity($user->hcp->id, 'professions-updated', __('hcp_activities.professions_updated'), $user->id);
            //-------------

            DB::commit();
        } catch (\Exception $e) {
            $hcpDocumentUpdate = null;
            $hcpProfessionUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcpProfessionUpdate)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_saved', ['records' => 'Profession']),
                'data' => [],
            ];

        } else if(is_null($hcpProfessionUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_saving_failed', ['records' => 'Profession']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Profile.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function profile(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $hcp = null;
        $where = [];

        DB::beginTransaction();
        try {

            // Get User
            $user = auth()->user();
            //---------

            $hcp = $this->hcp->find_hcp($user->hcp->id);

            DB::commit();
        } catch (\Exception $e) {
            $hcp = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcp)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Hcp']),
                'data' => [
                    'hcp'  => $hcp,
                ],
            ];

        } else if(empty($hcp) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Hcp']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Update Available For Job Status.
     *
     * @param  ValidateAvailableForJobStatus  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function updateAvailableForJobStatus(ValidateAvailableForJobStatus $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $hcpUpdate = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            if($request['status'] != $user->hcp->available_for_job) {

                // Update Hcp
                $hcpData = ['available_for_job' => $request['status']];
                $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                //-----------

                if($request['status'] == 1) {
                    $status = 'Available';
                } else {
                    $status = 'Unavailable';
                }

                // Hcp Activity
                create_hcp_activity($user->hcp->id, 'update-available-for-job-status', __('hcp_activities.update_available_for_job_status', ['status' => $status]), $user->id);
                //-------------

            } else {
                $hcpUpdate = null;
            }

            DB::commit();
        } catch (\Exception $e) {
            $hcpUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcpUpdate)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_updated', ['record' => 'Job Status']),
                'data' => [],
            ];

        } else if(is_null($hcpUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_updation_failed', ['record' => 'Job Status']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get Hcps.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getHcps(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $hcps = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            $pagination_limit="";
            // Get Facility
            if(isset($request['facility_id']) && !empty($request['facility_id'])) {
                $facility = $this->facility->api_find_facility($request['facility_id']);
                if(!empty($facility)) {
                    $request['facility_latitude'] = $facility['latitude'];
                    $request['facility_longitude'] = $facility['longitude'];
                }
            }
            //-------------

            if(isset($request['shift_id']) && !empty($request['shift_id'])) {
                $request['shift_id'] = $request['shift_id'];
                $request['check_transfer_case'] = 'true';
                $request['check_swap_case'] = 'true';
                $pagination_limit=200;
            }

            $hcps = $this->hcp->api_get_hcps($request, $user,$pagination_limit);

            DB::commit();
        } catch (\Exception $e) {
            $hcps = [];
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (count($hcps) > 0) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Hcps']),
                'data' => [
                    'hcps'  => $hcps,
                ],
            ];

        } else if(count($hcps) <= 0 && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Hcps']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get Hcp Documents.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getHcpDocuments(ValidateHcpDocuments $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $documentHcps = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            $documentHcps = $this->document_hcp->api_hcp_documents($user->hcp->id, $request);

            DB::commit();
        } catch (\Exception $e) {
            $documentHcps = [];
            $errorMessage = $e->getMessage();

            DB::rollback();
        }

        // Set response
        if (count($documentHcps) > 0) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Document']),
                'data' => [
                    'hcps'  => $documentHcps,
                ],
            ];

        } else if(count($documentHcps) <= 0 && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Document']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get Hcp Notifications.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getHcpNotifications(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $notifications = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            $notifications =  $this->notification->api_get_notifications($user, $request);

            DB::commit();
        } catch (\Exception $e) {
            $notifications = [];
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (count($notifications) > 0) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Notification']),
                'data' => [
                    'hcps'  => $notifications,
                ],
            ];

        } else if(count($notifications) <= 0 && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Notification']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Refer And Earn.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function referAndEarn(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $referrals = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // General Configuration
            $generalConfiguration = get_general_configuration();
            //----------------------

            // Referral And Earn
            $referralAndEarnMessage = __('messages.referral_and_earn', ['playStoreLink' => $generalConfiguration['android_app_link'], 'iosStoreLink' => $generalConfiguration['ios_app_link'], 'referalCode' => $user->hcp->referral_code]);
            //------------------

            $where = ['referrer_hcp_id' => $user->hcp->id, 'status' => 'completed'];
            $referrals =  $this->referral->getAll($where, true);

            DB::commit();
        } catch (\Exception $e) {
            $referrals = [];
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($user)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Referrals']),
                'data' => [
                    'my_referral_code'  => $user->hcp->referral_code,
                    'referral_and_earn_message'  => $referralAndEarnMessage,
                    'referrals_count'  => count($referrals),
                    'referral_total_amount'  => $referrals->sum('referrer_amount'),
                    'referrals' => $referrals,
                ],
            ];

        } else if(!empty($user) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Referrals']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get My Buddy Shift.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getMyBuddyShift(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $myBuddyShifts = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            $myBuddyShifts =  $this->hcp_profession->api_get_hcp_professions($user);

            DB::commit();
        } catch (\Exception $e) {
            $myBuddyShifts = [];
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (count($myBuddyShifts) > 0) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'My Buddy Shifts']),
                'data' => [
                    'my_buddy_shifts'  => $myBuddyShifts,
                ],
            ];

        } else if(count($myBuddyShifts) <= 0 && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'My Buddy Shifts']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Hcp Dashboard.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function dashboard(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $availableShiftCount = 0;
        $upcomingShiftCount = 0;
        $currentFortnight = 0;
        $currentYear = 0;

        // Get User
        $user = auth()->user();
        //---------
	
        DB::beginTransaction();
        try {

            // General Configuration
            $generalConfiguration = get_general_configuration();
            //----------------------

            // Get available shift
            $request['shift_status'] = 'available';
            $availableShifts = $this->shift->api_get_shifts($request, $user);
            if (count($availableShifts) > 0) {
                $availableShiftCount = $availableShifts['available_shifts']->total() + $availableShifts['transfer_shifts']->total() + $availableShifts['swap_shifts']->total();
            } else {
                $availableShiftCount = 0;
            }
            //--------------------

            // Get upcoming shift
            $request['shift_status'] = 'upcoming';
            $upcomingShifts = $this->shift->api_get_shifts($request, $user);
            if (count($upcomingShifts) > 0) {
                $upcomingShiftCount = $upcomingShifts['upcoming_shifts']->total() + $upcomingShifts['running_shifts']->total();
            } else {
                $upcomingShiftCount = 0;
            }
            //-------------------

            // Get schedule shift
            $request['shift_status'] = 'scheduled';
            $scheduledShifts = $this->shift->api_get_shifts($request, $user);
            if (count($scheduledShifts) > 0) {
                $scheduledShiftCount = $scheduledShifts->total();
            } else {
                $scheduledShiftCount = 0;
            }
            //-------------------

            // Get Current Year
            $currentYearTransactionWhere = [];
            $currentYearTransactionFilter = [
                'hcp_id' => $user->hcp->id,
                'amount_for' => 'hcp',
                'hcp_payment_status' => 'completed',
                'current_year' => 'yes',
            ];
            $currentYeartransactions = $this->transaction->get_transaction_last_days($currentYearTransactionWhere, $currentYearTransactionFilter);
            $currentYear = $currentYeartransactions->sum('hcp_amount');
            //-----------------

            // fortnight
            $startDate = Carbon::now($user->hcp->state->time_zone_name);
            if($startDate->format('d') <= '15') {
                $firstDay = $startDate->firstOfMonth()->format('Y-m-d');
                $lastDay = Carbon::parse($firstDay)->addWeeks(2)->format('Y-m-d');
            } else {
                $firstDay = Carbon::parse('16-'.$startDate->format('m').'-'.$startDate->format('Y'))->format('Y-m-d');
                $lastDay = Carbon::now($user->hcp->state->time_zone_name)->lastOfMonth()->format('Y-m-d');
            }
            //----------


            // Get Current Fort Night
            $currentFortnightTransactionWhere = [];
            $currentFortnightTransactionFilter = [
                'hcp_id' => $user->hcp->id,
                'amount_for' => 'hcp',
                'hcp_payment_status' => 'completed',
                'start_date' => $firstDay,
                'end_date' => $lastDay,
            ];
            $currentFortnightTransactions = $this->transaction->get_transaction_last_days($currentFortnightTransactionWhere, $currentFortnightTransactionFilter);
            $currentFortnight = $currentFortnightTransactions->sum('hcp_amount');
            //-----------------------

            // Get Current Week Data
            $weekStartDate = date("Y-m-d", strtotime("last friday"));
            $weekEndDate = date("Y-m-d", strtotime("thursday this week"));

            // $weekStartDate = date("Y-m-d", strtotime("friday this week"));
            // $weekEndDate = date("Y-m-d", strtotime("next thursday"));

            $currentWeekTransactionWhere = [];
            $currentWeekTransactionFilter = [
                'hcp_id' => $user->hcp->id,
                'amount_for' => 'hcp',
                'hcp_payment_status' => 'completed',
                'start_date' => $weekStartDate,
                'end_date' => $weekEndDate,
            ];
            $currentWeekTransactions = $this->transaction->get_transaction_last_days($currentWeekTransactionWhere, $currentWeekTransactionFilter);
            $currentWeek = $currentWeekTransactions->sum('hcp_amount');
            //----------------------

            // Hcp Point
            $currentPoint = $user->hcp->points;
            $pointMessageArr = null;
            if ($currentPoint <= 0) {
                $pointMessageArr = [
                    'title' => __('messages.hcp_point_title'),
                    'message' => __('messages.hcp_point_message'),
                    'call' => $generalConfiguration['hcp_cust_care_no'],
                    'email' => $generalConfiguration['hcp_cust_care_email'],
                ];
            }
            //----------

            // Referral And Earn
            $referralAndEarnMessage = __('messages.referral_and_earn', ['playStoreLink' => $generalConfiguration['android_app_link'], 'iosStoreLink' => $generalConfiguration['ios_app_link'], 'referalCode' => $user->hcp->referral_code]);
            //------------------

            // Hcp Messages
            $hcpMessages = [
                'point_message' => $pointMessageArr,
                'referral_and_earn' => $referralAndEarnMessage,
            ];
            //-------------

            // Get Hcp Profession Documents
            $pendingProfessionDocuments = $this->hcp_profession->api_get_pending_mandatory_profession_documents($user);
            if (count($pendingProfessionDocuments)) {
                $pendingProfessions = [];
                foreach($pendingProfessionDocuments as $pendingProfessionDocument) {
                    if(!empty($pendingProfessionDocument['profession'])) {
                        if($pendingProfessionDocument['profession']['hcp_documents_count'] < $pendingProfessionDocument['profession']['profession_documents_count']) {
                            $pendingProfessions[] = [
                                'hcp_profession_id' => $pendingProfessionDocument['id'],
                                'category_id' => $pendingProfessionDocument['profession']['category_id'],
                                'profession_id' => $pendingProfessionDocument['profession']['id'],
                                'profession_name' => $pendingProfessionDocument['profession']['name'],
                                'level_id' => $pendingProfessionDocument['level_id'],
                            ];
                        }
                    }
                }
            } else {
                $pendingProfessions = [];
            }
            //-----------------------------


            $data = [
                'available_shifts'  => $availableShiftCount,
                'upcoming_shifts'  => $upcomingShiftCount,
                'scheduled_shifts'  => $scheduledShiftCount,
                'current_fortnight'  => round($currentWeek, 2),
                'current_week'  => round($currentWeek, 2),
                'current_year'  => round($currentYear, 2),
                'hcp_current_point'  => $currentPoint,
                'hcp_messages'  => $hcpMessages,
                'fortnight_start_date'  => $firstDay,
                'fortnight_end_date'  => $lastDay,
                'week_start_date'  => $weekStartDate,
                'week_end_date'  => $weekEndDate,
                'pending_profession_documents'  => $pendingProfessions,

            ];

            DB::commit();
        } catch (\Exception $e) {
            $user = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!is_null($user)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Dashboard']),
                'data' => $data,
            ];

        } else if(is_null($user) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Dashboard']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Update device Id
     *
     * @param ValidateUpdateDeviceId $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     *
     */
    public function updateDeviceId(ValidateUpdateDeviceId $request)
    {
        $errorMessage = null;
        $responseCode = 200;
        $customMessage = null;
        $user = null;

        // Get auth user
        $authUser = auth()->user();
        //--------------

        DB::beginTransaction();
        try {

            // Find Device Id
            $this->user->update(['device_id' => null], ['device_id' => $request['device_id']]);
            //---------------

            // Update Device Id
            $userData = ['device_id'  => $request['device_id']];
            $where = ['id' => $authUser->id];
            $user = $this->user->update($userData, $where);
            //-----------------

            DB::commit();
        } catch (\Exception $e) {
            $user = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($user)) {

            $response = [
                'status' => true,
                'message' => $customMessage ?? __('messages.records_updated', ['record' => 'Device Id']),
                'data' => null,
            ];

        } else if (is_null($user) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $customMessage ?? __('messages.records_updation_failed', ['record' => 'Device Id']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Add/Update Profession (New).
     *
     * @param  ValidateAddUpdateProfessionNew  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function addUpdateProfessionNew(ValidateAddUpdateProfessionNew $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $hcpProfessionUpdate = null;
        $professionData = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {
            $acceptLowerjobStatus = get_accept_lower_job_checkbox_status();

            // Add/Update hcp professions
            $professions = json_decode($request['profession_json'], true);

            if(!empty($professions)) {
                foreach ($professions as $profession) {

                    $categoryRecord = $this->category->find_category($profession['category_id']);
                    $professionRecord = $this->profession->find_profession($profession['profession_id']);

                    if(!empty($categoryRecord) && !empty($professionRecord) && $professionRecord['category_id'] == $categoryRecord['id']) {

                        // Hcp Activity
                        if (isset($profession['hcp_profession_id']) && $profession['hcp_profession_id'] !='') {

                            $hcpProfessionRecord = $this->hcp_profession->api_find_hcp_professions($profession['hcp_profession_id']);
                            $categoryRecord = $this->category->find_category($profession['category_id']);
                            $professionRecord = $this->profession->find_profession($profession['profession_id']);

                            if (isset($profession['level_id']) && !empty($profession['level_id'])) {
                                $levelRecord = $this->level->find_level($profession['level_id']);
                            } else {
                                $levelRecord = null;
                            }

                            create_hcp_activity($user->hcp->id, 'professions-updated', __('hcp_activities.profession_updated', ['categoryName' => $hcpProfessionRecord['category']['name'], 'professionName' => $hcpProfessionRecord['profession']['name'], 'levelName' => isset($hcpProfessionRecord['level']) ? $hcpProfessionRecord['level']['name'] : '', 'newCategoryName' => $categoryRecord['name'], 'newProfessionName' => $professionRecord['name'], 'newLevelName' => isset($levelRecord) ? $levelRecord['name'] : '']), $user->id);

                        } else {
                            $categoryRecord = $this->category->find_category($profession['category_id']);
                            $professionRecord = $this->profession->find_profession($profession['profession_id']);

                            if (isset($profession['level_id']) && !empty($profession['level_id'])) {
                                $levelRecord = $this->level->find_level($profession['level_id']);
                            } else {
                                $levelRecord = null;
                            }

                            create_hcp_activity($user->hcp->id, 'professions-added', __('hcp_activities.profession_added', ['categoryName' => $categoryRecord['name'], 'professionName' => $professionRecord['name'], 'levelName' => isset($levelRecord) ? $levelRecord['name'] : '']), $user->id);
                        }
                        //-------------

                        if (isset($profession['hcp_profession_id']) && $profession['hcp_profession_id'] !='') {
                            $currentHcpProfession = $this->hcp_profession->api_find_hcp_professions($profession['hcp_profession_id']);

                            if(!empty($currentHcpProfession)) {
                                $hcpProfessionRequestWhere = [
                                    'hcp_profession_id' => !empty($currentHcpProfession['id']) ? $currentHcpProfession['id'] : null,
                                ];

                                if($currentHcpProfession['level_id'] != $profession['level_id']) {
                                    $hcpProfessionRequestArray = [
                                        'hcp_id' => $user->hcp->id,
                                        'hcp_profession_id' => $profession['hcp_profession_id'],
                                        'category_id' => $profession['category_id'],
                                        'profession_id' => $profession['profession_id'],
                                        'previous_level_id' => $currentHcpProfession['level_id'],
                                        'level_id' => isset($profession['level_id']) ? $profession['level_id'] : null,
                                        'type' => 'level-update',
                                        'status' => 'pending',
                                        'created_by' => $user->user_id,
                                    ];

                                    $professionRequest = $this->hcp_profession_request->update_or_create($hcpProfessionRequestWhere, $hcpProfessionRequestArray);

                                    // Update Hcp Profession
                                    $this->hcp_profession->update(['status' => 'pending-approval'], ['hcp_id' => $user->hcp->id, 'id' => $profession['hcp_profession_id']]);
                                    //----------------------
                                }
                            }

                            $where = [
                                'id' => isset($profession['hcp_profession_id']) ? $profession['hcp_profession_id'] : null,
                                'hcp_id' => $user->hcp->id,
                            ];
                        } else {
                            $where = [
                                'profession_id' => $profession['profession_id'],
                                'category_id' => $profession['category_id'],
                                'hcp_id' => $user->hcp->id,
                            ];
                        }

                        $array = [
                            'category_id' => $profession['category_id'],
                            'profession_id' => $profession['profession_id'],
                            'level_id'  => isset($profession['level_id']) ? $profession['level_id'] : null,
                            'created_by' => $user->id,
                            'updated_by' => $user->id,
                            'created_at' => now(),
                            'updated_at' => now(),
                        ];

                        $hcpProfessionUpdate = $this->hcp_profession->update_or_create($where, $array);

                        if (isset($request['edit_screen']) && !empty($request['edit_screen'])) {

                            $getHcpProfession = $this->hcp_profession_request->get_hcp_profession_request(['category_id' => $profession['category_id'], 'profession_id' => $profession['profession_id'], 'hcp_id' => $user->hcp->id]);

                            if(empty($getHcpProfession)) {

                                // Store Request
                                $this->hcp_profession_request->store([
                                    'hcp_id' => $user->hcp->id,
                                    'hcp_profession_id' => $hcpProfessionUpdate['id'],
                                    'category_id' => $profession['category_id'],
                                    'profession_id' => $profession['profession_id'],
                                    'previous_level_id' => isset($profession['level_id']) ? $profession['level_id'] : null,
                                    'level_id' => isset($profession['level_id']) ? $profession['level_id'] : null,
                                    'type' => 'profession-added',
                                    'status' => 'pending',
                                    'created_by' => $user->id
                                ]);
                                //--------------

                                // Update Hcp Profession
                                $this->hcp_profession->update(['status' => 'pending-approval'], ['hcp_id' => $user->hcp->id, 'id' => $hcpProfessionUpdate['id']]);
                                //----------------------
                            }
                        }

                    } else {
                        $response = [
                            'status' => false,
                            'message' => __('messages.records_saving_failed', ['records' => 'Profession']),
                            'data' => null,
                        ];
                    }
                }

                // Send Notification To Admin
                send_admin_notification('hcp-update-professions', 'admin', null, $user->hcp->id);
                //---------------------------
            }
            //---------------------------

            // Update Hcp
            if (isset($request['app_registration_screen']) && $request['app_registration_screen'] != '') {
                if ($request['app_registration_screen'] == 0) {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                } else if($request['app_registration_screen'] > $user->hcp->app_registration_screen) {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                } else if($user->hcp->app_registration_screen == null || $user->hcp->app_registration_screen == '') {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                }
            }
            //-------------

            // Update Hcp
            if($acceptLowerjobStatus['show_accept_lower_level_job_checkbox'] == '1') {
                $hcpData = ['accept_lower_level_job' => $request['accept_lower_level_job']];
                $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
            }
            //-----------

            // Check Accept Lower Level Job
            if($acceptLowerjobStatus['show_accept_lower_level_job_checkbox'] == '1') {
                if ($request['accept_lower_level_job'] == 1) {
                    $hcpProfessions = $this->hcp_profession->getAll(['hcp_id' => $user->hcp->id], true);
                    if (count($hcpProfessions)) {
                        foreach($hcpProfessions as $hcpProfession){
                            if (!is_null($hcpProfession->level_id)) {
                                $levelWhere = [
                                    'category_id' => $hcpProfession->category_id,
                                    'profession_id' => $hcpProfession->profession_id,
                                    ['id', '<=', $hcpProfession->level_id]
                                ];
                                $hcplevels = $this->level->getAll($levelWhere, true)->pluck('id');
                                if (count($hcplevels)) {
                                    $this->hcp_profession->update(['accept_level_ids' => implode(',', $hcplevels->toArray())], ['id' => $hcpProfession->id]);
                                }
                            }
                        }
                    }
                } else {
                    $hcpProfessions = $this->hcp_profession->getAll(['hcp_id' => $user->hcp->id], true);
                    if(count($hcpProfessions)) {
                        foreach($hcpProfessions as $hcpProfession){
                            if (!is_null($hcpProfession->level_id)) {
                                $this->hcp_profession->update(['accept_level_ids' => $hcpProfession->level_id], ['id' => $hcpProfession->id]);
                            }
                        }
                    }
                }
            }
            //-----------------------------

            DB::commit();
        } catch (\Exception $e) {
            $hcpProfessionUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcpProfessionUpdate)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_saved', ['records' => 'Profession']),
                'data' => [],
            ];

        } else if(is_null($hcpProfessionUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_saving_failed', ['records' => 'Profession']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get Hcp Professions.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getHcpProfessions(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $professions = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            $professions =  $this->hcp_profession->api_get_hcp_professions_with_document($user, $request);

            DB::commit();
        } catch (\Exception $e) {
            $professions = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (count($professions) > 0) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Professions']),
                'data' => [
                    'professions'  => $professions,
                ],
            ];

        } else if(count($professions) <= 0 && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Professions']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get Profession Documents.
     *
     * @param  ValidateGetProfessionDocuments  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getProfessionDocuments(ValidateGetProfessionDocuments $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $professions = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            $request['hcp_id'] = $user->hcp->id;
            $professions =  $this->profession->api_get_profession_with_documents($request);

            DB::commit();
        } catch (\Exception $e) {
            $professions = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (count($professions) > 0) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Professions Documents']),
                'data' => [
                    'professions'  => $professions,
                ],
            ];

        } else if(count($professions) <= 0 && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Professions Documents']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Add/Update Profession Document.
     *
     * @param  ValidateAddUpdateProfessionDocument  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function addUpdateProfessionDocument(ValidateAddUpdateProfessionDocument $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $hcpDocumentUpdate = null;
        $documentName = null;
        $documentsData = [];
        $sendNotificationDate = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Add/Update hcp documents
            $documentHcp = $this->document_hcp->api_get_hcp_document(['document_id' => $request['document_id'], 'profession_id' => $request['profession_id'], 'hcp_id' => $user->hcp->id]);

            if(!empty($documentHcp)){
                $documentApproval = 'pending-reapproval';
            } else {
                $documentApproval = 'pending';
            }

            // Upload hcp file
            if (!empty($request['document_file']) && file_exists($request['document_file'])) {
                $name = strtolower(str_replace(' ', '-', $request['document_name']));
                $path = config('constants.hcps.hcps_path') . '/' . $user->hcp->hcp_prefix . $user->hcp->id . '/documents/';
                $file = $this->uploadFile($request['document_file'], $path, 75, $name);

                if ($file['_status']) {
                    $documentName = $file['_data'];
                }
            } else {
                $documentName = ($request['document_file'] ?? null);
            }
            //----------------

            if (isset($request['expiry_date']) && !empty($request['expiry_date'])) {
                if (strtotime($request['expiry_date']) > strtotime(date('Y-m-d', strtotime(now() . ' + 60 days')))) {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date'] . ' - 60 days'));
                } else if(strtotime($request['expiry_date']) > strtotime(date('Y-m-d', strtotime(now() . ' + 30 days')))) {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date'] . ' - 30 days'));
                } else if (strtotime($request['expiry_date']) > strtotime(date('Y-m-d', strtotime(now() . ' + 7 days')))) {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date'] . ' - 7 days'));
                } else if (strtotime($request['expiry_date']) > strtotime(date('Y-m-d', strtotime(now() . ' + 1 days')))) {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date'] . ' - 1 days'));
                } else {
                    $sendNotificationDate = date('Y-m-d', strtotime($request['expiry_date']));
                }
            } else {
                $sendNotificationDate = null;
            }

            if (!empty($request['document_file'])) {
                $where = [
                    'document_id' => isset($request['document_id']) ? $request['document_id'] : null,
                    'profession_id' => $request['profession_id'],
                    'level_id' => isset($request['level_id']) ? $request['level_id'] : null,
                    'hcp_id' => $user->hcp->id,
                ];

                $array = [
                    'profession_id' => $request['profession_id'],
                    'level_id' => isset($request['level_id']) ? $request['level_id'] : null,
                    'document_name' => $documentName,
                    'expiry_date' => $request['expiry_date'] ? date('Y-m-d', strtotime($request['expiry_date'])) : null,
                    'send_notification_date' => $sendNotificationDate,
                    'document_approval' => $documentApproval,
                    'show_in_document_approval' => 'yes',
                    'show_in_notification' => 'no',
                    'created_by' => $user->id,
                    'updated_by' => $user->id,
                    'created_at' => now(),
                    'updated_at' => now(),
                ];

                $hcpDocumentUpdate = $this->document_hcp->update_or_create($where, $array);
            }
            //-------------------------

            // Store Hcp Profession Request
            if (isset($request['edit_screen']) && !empty($request['edit_screen']) && !empty($documentHcp) && !empty($documentHcp['profession_id']) && !empty($documentHcp['document']['mandatory_status'] == 'yes')) {

                $hcpProfessionRequest = $this->hcp_profession_request->get_hcp_profession_request(['profession_id' => $documentHcp['profession_id'], 'hcp_id' => $user->hcp->id, 'status' => 'pending', 'type' => 'document-expired']);

                $documentHcpIds = null;
                if(!empty($hcpProfessionRequest) && !empty($hcpProfessionRequest['hcp_document_ids'])) {
                    $documentHcpIds = explode(',', $hcpProfessionRequest['hcp_document_ids']);
                    array_push($documentHcpIds, $documentHcp->id);
                }

                $hcpProfessionRequestWhere = [
                    'id' => !empty($hcpProfessionRequest) ? $hcpProfessionRequest['id'] : null,
                ];

                $hcpProfessionRequestArray = [
                    'hcp_id' => $user->hcp->id,
                    'hcp_profession_id' => null,
                    'category_id' => $documentHcp['profession']['category_id'],
                    'profession_id' => $documentHcp['profession']['id'],
                    'leve_id' => null,
                    'hcp_document_id' => $documentHcp->id,
                    'hcp_document_ids' => $documentHcpIds ? implode(',', array_unique($documentHcpIds)) : $documentHcp->id,
                    'type' => 'document-expired',
                    'status' => 'pending',
                    'created_by' => $user->user_id,
                ];

                $professionRequest = $this->hcp_profession_request->update_or_create($hcpProfessionRequestWhere, $hcpProfessionRequestArray);

                // Update Hcp Profession
                $this->hcp_profession->update(['status' => 'pending-approval'], ['hcp_id' => $user->hcp->id, 'category_id' => $documentHcp['profession']['category_id'], 'profession_id' => $documentHcp['profession']['id']]);
                //----------------------
            }
            //-----------------------------

            // Update Hcp
            if (isset($request['app_registration_screen']) && $request['app_registration_screen'] != '') {
                if ($request['app_registration_screen'] == 0) {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                } else if($request['app_registration_screen'] > $user->hcp->app_registration_screen) {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                } else if($user->hcp->app_registration_screen == null || $user->hcp->app_registration_screen == '') {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                }
            }
            //-----------

            // Hcp Activity
            $professionRecord = $this->profession->find_profession($request['profession_id']);
            create_hcp_activity($user->hcp->id, 'profession-document-updated', __('hcp_activities.profession_document_uploaded', ['documentName' => $request['document_name'], 'professionName' => $professionRecord['name']]), $user->id);
            //-------------

            // Send Notification To Admin
            send_admin_notification('hcp-update-document', 'admin', null, $user->hcp->id);
            //---------------------------

            DB::commit();
        } catch (\Exception $e) {
            $hcpDocumentUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcpDocumentUpdate)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_saved', ['records' => 'Profession document uploaded']),
                'data' => [],
            ];

        } else if(is_null($hcpDocumentUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_saving_failed', ['records' => 'Profession document upload']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Update Hcp Profession Status.
     *
     * @param  ValidateUpdateHcpProfessionStatus  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function updateHcpProfessionStatus(ValidateUpdateHcpProfessionStatus $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $hcpProfessionUpdate = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Update hcp professions status
            $hcpProfessionUpdate =  $this->hcp_profession->update(['status' => $request['status']], ['id' => $request['hcp_profession_id']]);
            //------------------------------

            // Hcp Activity
            $hcpProfessionRecord = $this->hcp_profession->api_find_hcp_professions($request['hcp_profession_id']);
            create_hcp_activity($user->hcp->id, 'professions-status-updated', __('hcp_activities.profession_status_updated', ['categoryName' => $hcpProfessionRecord['category']['name'], 'professionName' => $hcpProfessionRecord['profession']['name'], 'levelName' => isset($hcpProfessionRecord['level']) ? $hcpProfessionRecord['level']['name'] : '', 'status' => $request['status']]), $user->id);
            //-------------

            DB::commit();
        } catch (\Exception $e) {
            $hcpProfessionUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcpProfessionUpdate)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_updated', ['record' => 'Profession status']),
                'data' => [],
            ];

        } else if(is_null($hcpProfessionUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_updation_failed', ['record' => 'Profession status']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Update Account Details.
     *
     * @param  ValidateAccountDetails  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function updateAccountDetails(ValidateAccountDetails $request)
    {
        $responseCode = 200;
        $hcp = null;
        $errorMessage = null;
        $userUpdate = null;
        $hcpUpdate = null;
        $cvName = null;
        $hcpData = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Update Hcp
            $hcpData = [
                'bank_name'  => $request['bank_name'],
                'branch_name_and_address'  => $request['branch_name_and_address'],
                'account_name'  => $request['account_name'],
                'account_number'  => $request['account_number'],
                'bsb_number'  => $request['bsb_number'],
                'superannuation_detail_fund_name'  => $request['superannuation_detail_fund_name'],
                'superannuation_detail_membership_no'  => $request['superannuation_detail_membership_no'],
            ];
            $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
            //-----------

            // Hcp Activity
            create_hcp_activity($user->hcp->id, 'account-details-updated', __('hcp_activities.account_details_updated'), $user->id);
            //-------------

            DB::commit();
        } catch (\Exception $e) {
            $hcpUpdate = null;
            $userUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcpUpdate)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_saved', ['records' => 'Hcp']),
                'data' => [],
            ];

        } else if(is_null($hcpUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_saving_failed', ['records' => 'Hcp']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Delete Hcp Profession.
     *
     * @param  ValidateDeleteHcpProfession  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function deleteHcpProfession(ValidateDeleteHcpProfession $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $hcpProfessionUpdate = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Get hcp professions
            $hcpProfession = $this->hcp_profession->api_find_hcp_professions($request['hcp_profession_id']);
            //---------------------

            // Delete Document
            $hcpProfessionDocument = $this->hcp_profession->api_find_hcp_professions($request['hcp_profession_id']);
            if(!empty($hcpProfessionDocument)) {
                $this->document_hcp->api_delete_hcp_profession_document($hcpProfessionDocument['hcp_id'], $hcpProfessionDocument['profession_id'], $hcpProfessionDocument['level_id']);
            }
            //----------------

            // Hcp Activity
            $hcpProfessionRecord = $this->hcp_profession->api_find_hcp_professions($request['hcp_profession_id']);
            create_hcp_activity($user->hcp->id, 'profession-deleted', __('hcp_activities.profession_deleted', ['categoryName' => $hcpProfessionRecord['category']['name'], 'professionName' => $hcpProfessionRecord['profession']['name'], 'levelName' => isset($hcpProfessionRecord['level']) ? $hcpProfessionRecord['level']['name'] : '']), $user->id);
            //-------------

            // Delete hcp professions
            $hcpProfessionUpdate = $this->hcp_profession->delete($request['hcp_profession_id']);
            //------------------------

            DB::commit();
        } catch (\Exception $e) {
            $hcpProfessionUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcpProfessionUpdate)) {

            $response = [
                'status' => true,
                'message' => __('messages.record_deleted', ['records' => 'Profession']),
                'data' => [],
            ];

        } else if(is_null($hcpProfessionUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.record_deleted_failed', ['records' => 'Profession']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Update App Registration Screen.
     *
     * @param  ValidateUpdateAppRegistrationScreen  $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function updateAppRegistrationScreen(ValidateUpdateAppRegistrationScreen $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $hcpUpdate = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // General Documents
            $generalDocuments = Document::where('required_for', 'hcp')
            ->where('mandatory_status', 'yes')
            ->where('doc_type', 'general-statement')
            ->count();
            //-----------------

            // Hcp Document
            $hcpDoucments = DocumentHcp::where('hcp_id', $user->hcp->id)
            ->whereHas('document', function ($query) {
                $query->where('mandatory_status', 'yes');
                $query->where('doc_type', 'general-statement');
            })
            ->with(['document' => function ($query)  {
                $query->where('mandatory_status', 'yes');
                $query->where('doc_type', 'general-statement');
            }])->count();
            //-------------

            // Update Hcp
            if (isset($request['app_registration_screen']) && $request['app_registration_screen'] != '') {
                if ($request['app_registration_screen'] == 0) {

                    if($hcpDoucments >= $generalDocuments) {
                        $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                        $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                    }

                } else if($request['app_registration_screen'] >= $user->hcp->app_registration_screen) {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                } else if($user->hcp->app_registration_screen == null || $user->hcp->app_registration_screen == '') {
                    $hcpData = ['app_registration_screen' => $request['app_registration_screen']];
                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                }
            }
            //-----------

            DB::commit();
        } catch (\Exception $e) {
            $hcpUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($hcpUpdate)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_updated', ['record' => 'App registration screen']),
                'data' => [],
            ];

        } else if(is_null($hcpUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_updation_failed', ['record' => 'App registration screen']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Send Email Verification
     *
     * @param ValidateSendEmailVerification $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     *
     */
    public function sendEmailVerification(ValidateSendEmailVerification $request)
    {
        $errorMessage = null;
        $responseCode = 200;
        $oneTimePassword = null;
        $otp = rand(100000, 999999);

         // Store one time password
        $oneTimePassword = OneTimePassword::create([
            'one_time_password' => $otp,
            'type'              => config('constants.one_time_password_types.VERIFICATION.value'),
            'request_token'     => (string) Str::uuid(),
            'expires_at'        => Carbon::now()->addHours(24)
        ]);
        //------------------------

        $emailData['message'] = __('messages.verification_mail_subject');
        $emailData['subject'] = __('messages.verification_mail_subject');
        $emailData['sender']  = __('messages.verification_mail_subject');
        $emailData['email']   = $request['email'];
        $emailData['otp'] = $otp;


        // Send email
        Mail::send('emails.verify-email-app', $emailData, function ($message) use ($emailData) {
            $message->to($emailData['email'])->subject($emailData['subject']);
        });
        //-----------

        // Set response
        if (!empty($oneTimePassword)) {

            $response = [
                'status' => true,
                'message' => __('messages.email_verification'),
                'data' => [
                    'request_token' => $oneTimePassword->request_token,
                    'otp' => $oneTimePassword->one_time_password,
                ],
            ];

        } else if (is_null($oneTimePassword) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Verify Email
     *
     * @param ValidateVerify $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     *
     */
    public function verifyEmail(ValidateVerify $request)
    {
        $errorMessage = null;
        $responseCode = 200;

        $oneTimePassword = OneTimePassword::where('type', config('constants.one_time_password_types.VERIFICATION.value'))
        ->where('request_token', $request['request_token'])->where('one_time_password', $request['otp'])->first();

        // Set response
        if (!empty($oneTimePassword)) {

            $response = [
                'status' => true,
                'message' => __('messages.verify_email'),
                'data' => [],
            ];

        } else if (is_null($oneTimePassword) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.verify_email_failed'),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.email_something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Send Mobile Verification
     *
     * @param ValidateSendMobileVerification $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     *
     */
    public function sendMobileVerification(ValidateSendMobileVerification $request)
    {
        $errorMessage = null;
        $responseCode = 200;
        $oneTimePassword = null;
        $otp = rand(100000, 999999);

        // Store one time password
        $oneTimePassword = OneTimePassword::create([
            'one_time_password' => $otp,
            'type'              => config('constants.one_time_password_types.MOBILE_VERIFICATION.value'),
            'request_token'     => (string) Str::uuid(),
            'expires_at'        => Carbon::now()->addHours(24)
        ]);
        //------------------------

        // Send SMS
        try {
            $smsMessage = __('messages.otp_message', ['otp' => $otp]);
            $accountSid = env("TWILIO_SID");
            $authToken = env("TWILIO_AUTH_TOKEN");
            $twilioNumber = env("TWILIO_NAME");
            $client = new Client($accountSid, $authToken);

            $message = $client->messages->create($request['contact_number'], [
                'from' => $twilioNumber,
                'body' => $smsMessage
            ]);

        } catch (\Exception $e) {
            // will return user to with twilio error
            $response = [
                'status' => false,
                'message' => $e->getMessage(),
                'data' => null,
            ];
            // return response()->json($response, $responseCode);
        }


        //---------

        // Set response
        if (!empty($oneTimePassword)) {

            $response = [
                'status' => true,
                'message' => __('messages.mobile_verification'),
                'data' => [
                    'request_token' => $oneTimePassword->request_token,
                    'otp' => $oneTimePassword->one_time_password,
                ],
            ];

        } else if (is_null($oneTimePassword) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Verify Mobile
     *
     * @param ValidateVerifyEmail $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     *
     */
    public function verifyMobile(ValidateVerify $request)
    {
        $errorMessage = null;
        $responseCode = 200;

        $oneTimePassword = OneTimePassword::where('type', config('constants.one_time_password_types.MOBILE_VERIFICATION.value'))
        ->where('request_token', $request['request_token'])->where('one_time_password', $request['otp'])->first();

        $oneTimePasswordTemp = OneTimePassword::where('type', config('constants.one_time_password_types.MOBILE_VERIFICATION.value'))
        ->where('request_token', $request['request_token'])->first();

        // Set response
        if (!empty($oneTimePassword)) {

            $response = [
                'status' => true,
                'message' => __('messages.verify_mobile'),
                'data' => [],
            ];

        } else if (is_null($oneTimePassword) && is_null($errorMessage)) {

            $correctOtp = '';
            if(!empty($oneTimePasswordTemp)) {
                $correctOtp = $oneTimePasswordTemp['one_time_password'];
            }

            $response = [
                'status' => false,
                'message' => __('messages.verify_mobile_failed'),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.sms_something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get hcp rating.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getHcpRating(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $ratings = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            $request['hcp'] = $user->hcp->id;
            $request['status'] = 'publish';
            $ratings =  $this->ratingReview->api_get_hcp_ratings($request);

            DB::commit();
        } catch (\Exception $e) {
            $ratings = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (count($ratings) > 0) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Ratings']),
                'data' => [
                    'ratings'  => $ratings,
                ],
            ];

        } else if(count($ratings) <= 0 && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Ratings']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }


    /**
     * Nurse notifications
     *
     * @param ValidateChangePassword $request
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function changePassword(ValidateChangePassword $request)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        $errorMessage = null;
        $responseCode = 200;
        $currentPasswordMismatch = false;
        $passwordSaved = false;
        $newPasswordIsSame = false;

        // Update user password
        DB::beginTransaction();
        try {

            if (Hash::check($request['current_password'], $authUser->password)) {
                if (!Hash::check($request['new_password'], $authUser->password)) {
                    $authUser->password = Hash::make($request['new_password']);
                    $passwordSaved = $authUser->save();
                } else {
                    $newPasswordIsSame = true;
                }
            } else {
                $currentPasswordMismatch = true;
            }

            DB::commit();
        } catch (\Exception $e) {
            $errorMessage = $e->getMessage();
            $passwordSaved = false;
            DB::rollback();
        }
        //---------------------

        // Set response
        if ($currentPasswordMismatch) {

            // Set response code
            $responseCode = 422;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.current_password_mismatch'),
                'data' => null,
                'errors' => [
                    __('messages.current_password_mismatch')
                ],
            ];

        } else if ($newPasswordIsSame) {

            // Set response code
            $responseCode = 422;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.new_password_is_same'),
                'data' => null,
                'errors' => [
                    'current_password' => [
                        __('messages.new_password_is_same'),
                    ],
                ],
            ];

        } else if ($passwordSaved) {

            $response = [
                'status' => true,
                'message' => __('messages.user_password_updated'),
                'data' => null,
            ];

        } else if ($errorMessage){

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];

        } else {

            $response = [
                'status' => false,
                'message' => __('messages.password_reset_failed'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }
}
