<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Interfaces\ShiftInterface;
use App\Interfaces\ShiftActivityInterface;
use App\Interfaces\RejectedShiftInterface;
use App\Interfaces\ShiftTransferInterface;
use App\Interfaces\ShiftSwapRequestInterface;
use App\Interfaces\RatingReviewInterface;
use App\Interfaces\BuddyShiftRequestInterface;
use App\Interfaces\CancelledShiftInterface;
use App\Interfaces\ShiftTrackingInterface;
use App\Interfaces\HcpInterface;
use App\Interfaces\ShiftSwapRejectedRequestInterface;
use App\Interfaces\TransactionInterface;
use App\Interfaces\HcpBlockTimeInterface;
use App\Interfaces\DocumentHcpInterface;
use App\Interfaces\HcpProfessionInterface;
use App\Http\Requests\API\Shift\ValidateShifts;
use App\Http\Requests\API\Shift\ValidateAcceptShift;
use App\Http\Requests\API\Shift\ValidateRejectShift;
use App\Http\Requests\API\Shift\ValidateShiftDetails;
use App\Http\Requests\API\Shift\ValidateCreateTransferShift;
use App\Http\Requests\API\Shift\ValidateCreateSwapRequest;
use App\Http\Requests\API\Shift\ValidateCreateBuddyShiftRequest;
use App\Http\Requests\API\Shift\ValidateGiveRating;
use App\Http\Requests\API\Shift\ValidateCancelShift;
use App\Http\Requests\API\Shift\ValidateAcceptSwapRequest;
use App\Http\Requests\API\Shift\ValidateShiftTracking;
use App\Http\Requests\API\Shift\ValidateAcceptTransferShift;
use App\Http\Requests\API\Shift\ValidateRejectTransferShift;
use App\Http\Requests\API\Shift\ValidateSwapRequestDetails;
use App\Http\Requests\API\Shift\ValidateRejectSwapRequest;
use App\Http\Requests\API\Shift\ValidateStartShift;
use App\Http\Requests\API\Shift\ValidateShiftPayment;
use App\Http\Requests\API\Shift\ValidateShiftPaymentDetails;
use App\Models\ShiftTransfer;
use Illuminate\Support\Facades\Hash;
use Carbon\Carbon;
use App\Jobs\AdminPanel\SendEmailJob;
use App\Jobs\AdminPanel\ShiftBroadcastJob;
use Illuminate\Support\Facades\Mail;
use Twilio\Rest\Client;

class ShiftController extends Controller
{
    private $shift, $shiftActivity;

    /**
     * Create an controller instance.
     */
    public function __construct(ShiftInterface $shift, ShiftActivityInterface $shiftActivity, RejectedShiftInterface $rejectedShift, ShiftTransferInterface $shiftTransfer, ShiftSwapRequestInterface $shiftSwapRequest, RatingReviewInterface $ratingReview, BuddyShiftRequestInterface $buddyShiftRequest, CancelledShiftInterface $cancelledShift, HcpInterface $hcp, ShiftTrackingInterface $shiftTracking, ShiftSwapRejectedRequestInterface $shiftSwapRejectedRequest, TransactionInterface $transaction, HcpBlockTimeInterface $hcpBlockTime, DocumentHcpInterface $documentHcp, HcpProfessionInterface $hcpProfession){
        $this->shift = $shift;
        $this->shift_activity = $shiftActivity;
        $this->rejected_shift = $rejectedShift;
        $this->shift_transfer = $shiftTransfer;
        $this->shift_swap_request = $shiftSwapRequest;
        $this->ratingReview = $ratingReview;
        $this->buddy_shift_request = $buddyShiftRequest;
        $this->cancelled_shift = $cancelledShift;
        $this->hcp = $hcp;
        $this->shift_tracking = $shiftTracking;
        $this->shift_swap_rejected_request = $shiftSwapRejectedRequest;
        $this->transaction = $transaction;
        $this->hcp_block_time = $hcpBlockTime;
        $this->document_hcp = $documentHcp;
        $this->hcp_profession = $hcpProfession;
        $this->middleware('auth:api');
    }

    /**
     * Get shifts.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getShifts(ValidateShifts $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $shifts = [];
        $where = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            $shifts = $this->shift->api_get_shifts($request, $user);

            DB::commit();
        } catch (\Exception $e) {
            $shifts = [];
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (count($shifts) > 0) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Shifts']),
                'data' => [
                    'shifts'  => $shifts,
                ],
            ];

        } else if(count($shifts) <= 0 && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Shifts']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get shift details.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getShiftDetails(ValidateShiftDetails $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $shift = null;
        $message = __('messages.records_not_found', ['records' => 'Shift']);
        $cancelledShift = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            $shift = $this->shift->api_get_shift_details($request, $user);

            // Check Cancelled shift
            if(!empty($shift) && $shift['shift_status'] != '0' && $shift['shift_transfer_swap_status'] == '') {
                $cancelledShiftsWhere = ['shift_id' => $request['shift_id'], 'hcp_id' => $user->hcp->id];
                $cancelledShifts = $this->cancelled_shift->find_cancelled_shift($cancelledShiftsWhere);

                if(empty($cancelledShifts) && !empty($shift->hcp_id) && $shift->hcp_id != $user->hcp->id) {
                    $shift = null;
                    $message = __('messages.shift_details_failed');
                    $cancelledShift = true;
                }
            }
            //----------------------

            // Check Transfer Request
            if(!empty($shift)  && $shift['shift_status'] != '0' && empty($cancelledShift) && $shift['shift_transfer_swap_status'] == 'transferred') {
                $checkshiftTransferWhere = ['shift_id' => $request['shift_id']];
                $checkShiftTransfer = $this->shift_transfer->find_shift_transfer($checkshiftTransferWhere);
                if(!empty($checkShiftTransfer)) {
                    if($checkShiftTransfer['status'] == 'cancelled' || $checkShiftTransfer['status'] == 'reject') {
                        if($shift->hcp_id != $user->hcp->id && $shift->shift_status == '1') {
                            $shift = null;
                            $message = __('messages.shift_details_failed');
                        }
                    }
                }
            }
            //-----------------------

            // Check Swap Request
            if(!empty($shift) && $shift['shift_status'] != '0' && empty($cancelledShift) && $shift['shift_transfer_swap_status'] == 'swapped') {
                $checkshiftSwaprRequestWhere = ['shift_id' => $request['shift_id'], 'status' => 'pending'];
                $checkshiftSwaprRequest = $this->shift_swap_request->find_shift_swap_request($checkshiftSwaprRequestWhere);
                $checkshiftSwaprCancelledRequestWhere = ['shift_id' => $request['shift_id'], 'hcp_id' => $user->hcp->id];
                $checkshiftSwaprCancelledRequest = $this->shift_swap_rejected_request->find_shift_swap_rejected_request($checkshiftSwaprCancelledRequestWhere);

                $checkshiftSwaprRequestAcceptWhere = ['shift_id' => $request['shift_id'], 'status' => 'accept'];
                $checkshiftSwaprRequestAccept = $this->shift_swap_request->find_shift_swap_request($checkshiftSwaprRequestAcceptWhere);

                if(!empty($checkshiftSwaprRequest) && !empty($checkshiftSwaprCancelledRequest)) {
                    $shift = null;
                    $message = __('messages.shift_swap_details_failed');
                } else if (empty($checkshiftSwaprRequest) && !empty($checkshiftSwaprRequestAccept) && $user->hcp->id != $shift['hcp_id']) {
                    $shift = null;
                    $message = __('messages.shift_details_failed');
                }
            }
            //-------------------

            DB::commit();
        } catch (\Exception $e) {
            $shift = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($shift)) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Shift']),
                'data' => [
                    'shift'  => $shift,
                ],
            ];

        } else if(is_null($shift) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Accept shifts.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function acceptShift(ValidateAcceptShift $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $shiftUpdate = null;
        $shiftActivity = null;
        $dayHoursReached = false;
        $fortnightHoursReached = false;
        $hcpHistoryCheck = false;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {
            // General Configuration
            $generalConfiguration = get_general_configuration();
            //----------------------

            // Get Shift
            $shift = $this->shift->find_shift($request['shift_id']);
            //----------

            // Set time zone
            date_default_timezone_set($shift->state->time_zone_name);
            $currentTime = Carbon::now($shift->state->time_zone_name);
            //--------------

            // Get Hcp Shift
            $hcpShifts = $this->shift->find_shift_with_date($shift['start_time'], $shift['end_time'], ['hcp_id' => $user->hcp->id]);
            //--------------

            // fortnight
            $startDate = Carbon::parse($shift['start_time']);
            if($startDate->format('d') <= '15') {
                $firstDay = $startDate->firstOfMonth()->format('d-m-Y');
                $lastDay = Carbon::parse($firstDay)->addWeeks(2)->format('d-m-Y');
            } else {
                $firstDay = Carbon::parse('16-'.$startDate->format('m').'-'.$startDate->format('Y'))->format('d-m-Y');
                $lastDay = Carbon::parse($shift['start_time'])->lastOfMonth()->format('d-m-Y');
            }
            //----------

            // Get Shift Hours
            $hcpHoursFilter = [
                'hcp_id' => $user->hcp->id,
                'current_day' => $shift['start_time'],
                'fortnight_start' => $firstDay,
                'fortnight_end' => $lastDay,
            ];
            $hcpHours = $this->hcp->get_hcp_shift_hours($hcpHoursFilter);

            if(!empty($hcpHours) && count($hcpHours->current_date_shifts) && $hcpHours->current_date_shifts->sum('working_hours') >= $generalConfiguration['no_of_hrs_allowed_perday']) {
                $dayHoursReached = true;
            } else if(!empty($hcpHours) && count($hcpHours->current_date_shifts) && $hcpHours->fortnight_shifts->sum('working_hours') >= $generalConfiguration['no_of_hrs_allowed_fortnight']) {
                $fortnightHoursReached = true;
            } else {
                $dayHoursReached = false;
                $fortnightHoursReached = false;
            }
            //----------------

            // Find General Expire Documents
            $generalDocumentWhereArr = ['expiry_date' => Carbon::parse($shift['start_time'])->format('Y-m-d'), 'hcp_id' => $user->hcp->id];
            $generalExpireDocuments = $this->document_hcp->api_get_hcp_expire_general_document($generalDocumentWhereArr);
            //-------------------------------

            // Find Profession Expire Documents
            $professionDocumentWhereArr = ['expiry_date' => Carbon::parse($shift['start_time'])->format('Y-m-d'), 'profession_id' => $shift['profession_id'], 'hcp_id' => $user->hcp->id];
            $professionGeneralExpireDocuments = $this->document_hcp->api_get_hcp_expire_profession_document($professionDocumentWhereArr);
            //--------------------------------

            // Find General Pending Approval Documents
            $generalDocumentPendingApprovalWhereArr = ['hcp_id' => $user->hcp->id];
            $generalDocumentPendingApprovalDocuments = $this->document_hcp->api_get_hcp_pending_approval_general_document($generalDocumentPendingApprovalWhereArr);
            //----------------------------------------

            // Find Profession Pending Approval Documents
            $professionDocumentPendingApprovalWhereArr = ['profession_id' => $shift['profession_id'], 'hcp_id' => $user->hcp->id];
            $professionGeneralPendingApprovalDocuments = $this->document_hcp->api_get_hcp_pending_approval_profession_document($professionDocumentPendingApprovalWhereArr);
            //-------------------------------------------
	
            // Check Hcp Profession Status
            $hcpProfession = $this->hcp_profession->find_last_hcp_profession(['profession_id' => $shift['profession_id'],'level_id' => $shift['hcp_level_id'], 'hcp_id' => $user->hcp->id]);
	//p($hcpProfession);
            //----------------------------
	//Check Budy Shift Status
	$hcp_buddy_shift_status = '';
	if(!empty($hcpProfession)){
		
            		$hcp_buddy_shift_status = check_buddy_shift_request($hcpProfession->id);
	}
 
            // Check Hcp Shift History
            $shiftHistoryWehere = ['hcp_id' => $user->hcp->id, 'shift_id' => $request['shift_id']];
            $hcpHistory = $this->hcp->get_hcp_shift_history($shiftHistoryWehere);
            //------------------------

            if(!empty($hcpHistory) && ($hcpHistory['hcp_shift_transfers_count'] != '0' || $hcpHistory['hcp_transfer_shift_transfers_count'] != '0' || $hcpHistory['shift_hcp_swap_count'] != '0' || $hcpHistory['swap_hcp_shift_count'] != '0' || $hcpHistory['hcp_swap_rejected_request_count'] != '0')) {
                $hcpHistoryCheck = true;
            }

            if($user->hcp->status == 'blocked') {
                $shiftUpdate = null;
                $message = __('messages.shift_applied_hcp_blocked');
            }else if (empty($hcpProfession)) {
                $shiftUpdate = null;
                $message = "Profession Not Added";
            } else if($user->hcp->status == 'pending-approval') {
                $shiftUpdate = null;
                $message = __('messages.shift_applied_hcp_failed_pending_approval');
            } else if (count($generalExpireDocuments)) {
                $shiftUpdate = null;
                $message = __('messages.shift_applied_hcp_failed_general_document', ['doucmentName' => $generalExpireDocuments[0]['document']['name']]);
            } else if (count($professionGeneralExpireDocuments)) {
                $shiftUpdate = null;
                $message = __('messages.shift_applied_hcp_failed_profession_document', ['doucmentName' => $professionGeneralExpireDocuments[0]['document']['name']]);
            } else if (count($generalDocumentPendingApprovalDocuments)) {
                $shiftUpdate = null;
                $message = __('messages.shift_applied_hcp_failed_document_pending_approval', ['doucmentName' => $generalDocumentPendingApprovalDocuments[0]['document']['name']]);
            } else if (count($professionGeneralPendingApprovalDocuments)) {
                $shiftUpdate = null;
                $message = __('messages.shift_applied_hcp_failed_document_pending_approval', ['doucmentName' => $professionGeneralPendingApprovalDocuments[0]['document']['name']]);
            } else if (!empty($hcpProfession) && $hcpProfession['status'] == 'inactive') {
                $shiftUpdate = null;
                $message = __('messages.shift_applied_hcp_failed_profession_status', ['professionName' => $hcpProfession['profession']['name'], 'status' => 'inactive']);
            } else if (!empty($hcpProfession) && $hcpProfession['status'] == 'pending-approval') {
                $shiftUpdate = null;
                $message = __('messages.shift_applied_hcp_failed_profession_status', ['professionName' => $hcpProfession['profession']['name'], 'status' => 'pending approval']);
            } else if (!empty($hcpProfession) && $hcpProfession['status'] == 'expired') {
                $shiftUpdate = null;
                $message = __('messages.shift_applied_hcp_failed_profession_status', ['professionName' => $hcpProfession['profession']['name'], 'status' => 'expired']);
            } else if (!empty($hcpProfession) && $hcp_buddy_shift_status != 'approved') {
                $shiftUpdate = null;
                $message = "Buddy Shift is not approved";
            } else if (strtotime($currentTime) >= strtotime(Carbon::parse($shift['end_time']))) {
                $shiftUpdate = null;
                $message = __('messages.shift_applied_hcp_failed_end_time_over');
            } else if ($hcpHistoryCheck == true) {
                $shiftUpdate = null;
                $message = __('messages.shift_history_found');
            } else if($user->hcp->status == 'active' && $user->hcp->points > 0) {
                if(!empty($shift)) {
                    if (empty($hcpShifts)) {
                        if(is_null($shift->hcp_id)) {
                            if($fortnightHoursReached != true) {
                                if($dayHoursReached != true) {

                                    // Update Shift
                                    $shiftData = ['hcp_id' => $user->hcp->id, 'shift_status' => '1'];
                                    $shiftUpdate = $this->shift->update($shiftData, ['id' => $request['shift_id']]);
                                    //-------------

                                    // Send Notification
                                    $shiftStartTime = Carbon::parse($shift->start_time);
                                    if (strtotime($currentTime) >= strtotime($shiftStartTime)) {
                                        send_shift_push_notification($shift->id, 'hcp-shift-start-time-over', $user->hcp->id);
                                    }
                                    //------------------


                                    // Shift Activity
                                    $shiftActivityData = [
                                        'hcp_id'        => $user->hcp->id,
                                        'shift_id'      => $shift->id,
                                        'activity'      => 'shift-accepted',
                                        'created_by'    => $user->id,
                                        'updated_by'    => $user->id,
                                    ];
                                    $shiftActivity = $this->shift_activity->store($shiftActivityData);
                                    //---------------

                                    $message = __('messages.shift_applied_successfully');
                                } else {
                                    $shiftUpdate = null;
                                    $message = __('messages.shift_applied__failed_day_hours_reached');
                                }

                            }  else {
                                $shiftUpdate = null;
                                $message = __('messages.shift_applied__failed_fortnight_hours_reached');
                            }

                        } else {
                            $shiftUpdate = null;
                            $message = __('messages.shift_applied_failed');
                        }
                    } else {
                        $shiftUpdate = null;
                        $message = __('messages.shift_applied_time_failed');
                    }
                } else {
                    $shiftUpdate = null;
                    $message = __('messages.record_not_found', ['record' => 'Shift']);
                }
            } else {
                $shiftUpdate = null;
                $message = __('messages.shift_applied_hcp_failed');
            }

            DB::commit();
        } catch (\Exception $e) {
            $shiftUpdate = null;
            $errorMessage = $e->getMessage();
	p($errorMessage);
            DB::rollback();
        }

        // Set response
        if (!empty($shiftUpdate)) {
            $shift = $shift->fresh();
            if($generalConfiguration->send_email_on_shift_accepted==1){
                $hcp_name = $user->hcp->first_name.' '.$user->hcp->last_name;
                $facility_name = $shift->facility->name;
                $wing = $shift->word_wing;
                $prof = $shift->profession->name;
                $start = $shift->start_time;
                // Set email data
                // $emailData['message'] = __('messages.shift_accepted_mail_subject');
                $emailData['message'] =  __('messages.shift_accept_text_message', ['facility' => $facility_name,'SHIFT' => $shift->shift_prefix." ".$shift->id,'HCP' => $hcp_name,'wing' => $wing,'prof' => $prof,'start' => $start]);
                $emailData['subject'] = __('messages.shift_accepted_mail_subject');
                $emailData['sender'] = __('messages.shift_accepted_mail_subject');
                $emailData['email'] = env('SHIFT_ACTIVITY_EMAIL');
                $emailData['receiver'] = env('SHIFT_ACTIVITY_EMAIL');
                $emailData['shift'] = $shift;

                Mail::send('emails.admin-panel.shift-activity-email', $emailData, function ($message) use ($emailData, $shift) {
                    $message->to($emailData['email'], $emailData['receiver'])->subject($emailData['subject']);
                    if(!empty($shift->facility->additional_nurse_email)){
                        $cc_emails=explode(",",$shift->facility->additional_nurse_email);
                        foreach($cc_emails as $cc){
                            $message->cc($cc);
                        }
                    }
                });


            }

            // Send SMS
                try {
                    $hcp_name = $user->hcp->first_name.' '.$user->hcp->last_name;
                    $facility_name = $shift->facility->name;
                    $wing = $shift->word_wing;
                    $prof = $shift->profession->name;
                    $start = $shift->start_time;
                    $msg =  __('messages.shift_accept_text_message', ['facility' => $facility_name,'SHIFT' => $shift->shift_prefix." ".$shift->id,'HCP' => $hcp_name,'wing' => $wing,'prof' => $prof,'start' => $start]);
                    // $msg="Hi $facility_name shift $shift->shift_prefix $shift->id  has been accepted by $hcp_name. Please find shift details below:<br/>
                    //                 Wing/Ward: $wing,<br/>
                    //                 Profession: $prof,<br/>
                    //                 Start Time: $start<br/>
                    // Thank you<br/>
                    // Smart Health Team
                    //             ";
                    // $smsMessage = __('messages.hcp_account_approve', ['hcp' => $user->hcp->first_name.' '.$user->hcp->last_name]);
                    $accountSid = env("TWILIO_SID");
                    $authToken = env("TWILIO_AUTH_TOKEN");
                    $twilioNumber = env("TWILIO_NAME");
                    $client = new Client($accountSid, $authToken);

                    if(!empty($shift->facility->mobile_number)) {
                        $message = $client->messages->create($shift->facility->mobile_number, [
                            'from' => $twilioNumber,
                            'body' => $msg
                        ]);
                    }

                } catch (\Exception $e) {
                    $response = [
                        'status' => false,
                        'message' => $e->getMessage(),
                        'data' => null,
                    ];
                }
                //---------

            $response = [
                'status' => true,
                'message' => $message,
                'data' => [],
            ];

        } else if(is_null($shiftUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Reject shifts.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function rejectShift(ValidateRejectShift $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $rejectedShift = null;
        $shiftActivity = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Get Shift
            $rejectedShift = $this->rejected_shift->find_rejected_shift(['shift_id' => $request['shift_id'], 'hcp_id' => $user->hcp->id]);
            //----------

            if(empty($rejectedShift)) {
                $rejectedShiftData = [
                    'shift_id'      => $request['shift_id'],
                    'hcp_id'        => $user->hcp->id,
                    'created_by'    => $user->id,
                    'updated_by'    => $user->id,
                ];
                $rejectedShift = $this->rejected_shift->store($rejectedShiftData);

                // Shift Activity
                $shiftActivityData = [
                    'hcp_id'        => $user->hcp->id,
                    'shift_id'      => $request['shift_id'],
                    'activity'      => 'shift-rejected',
                    'created_by'    => $user->id,
                    'updated_by'    => $user->id,
                ];
                $shiftActivity = $this->shift_activity->store($shiftActivityData, ['id' => $request['shift_id']]);
                //---------------

                $message = __('messages.shift_reject_successfully');
            } else {
                $rejectedShift = null;
                $message = __('messages.shift_reject_failed');
            }

            DB::commit();
        } catch (\Exception $e) {
            $rejectedShift = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($rejectedShift)) {

            $response = [
                'status' => true,
                'message' => $message,
                'data' => [],
            ];

        } else if(is_null($rejectedShift) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Create Transfer Request.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function createTransferRequest(ValidateCreateTransferShift $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $shiftTransfer = null;
        $shiftUpdate = null;
        $shiftActivity = null;
        $penaltyPoint = 0;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // General Configuration
            $generalConfiguration = get_general_configuration();
            //----------------------

            // Get Shift
            $shiftWhere = ['id' => $request['shift_id'], 'hcp_id' => $user->hcp->id];
            $shift = $this->shift->find_shift_with_where($shiftWhere);
            //-----------

            // Set time zone
            date_default_timezone_set($shift->state->time_zone_name);
            //--------------

            $currentTime = Carbon::now($shift->state->time_zone_name);

            // Check Time Diff
            $to = Carbon::parse($shift['start_time']);
            $from = Carbon::parse($currentTime);
            $timeDiff = $to->diffInHours($from);
            //----------------

            if(!empty($shift)) {
                if ($timeDiff <= $generalConfiguration['hcp_cannot_cancel_transfer_swap_with_hours']) {
                    $shiftTransfer = null;
                    $message = __('messages.shift_transfer_failed');
                } else {

                    if($shift['shift_type'] != 'buddy') {

                        // Get Transfer Shift
                        $checkshiftTransferWhere = ['shift_id' => $request['shift_id'], 'hcp_id' => $user->hcp->id];
                        $checkShiftTransfer = $this->shift_transfer->find_shift_transfer($checkshiftTransferWhere);
                        //-------------------

                        if(empty($checkShiftTransfer)) {

                            // Check Transfer Hcp Shift
                            $checkHcpShift = $this->shift->find_transfer_hcp_shift($shift['start_time'], $shift['end_time'], $shift, null, $request['transfer_hcp_id']);
                            //-------------------------

                            if (empty($checkHcpShift)) {

                                if ($timeDiff <= $generalConfiguration['free_shift_cancel_period_for_hcp']) {

                                    // Check Penalty
                                    $penaltyPoint = $generalConfiguration['penalty_points_for_shift_cancellation'];
                                    $hcpPoint = $user->hcp->points-$penaltyPoint;

                                    // Hcp Activity
                                    create_hcp_activity($user->hcp->id, 'points-deduction', __('hcp_activities.points_deduction_transfer', ['point' => $penaltyPoint, 'shiftId' => $shift->shift_prefix.$shift->id]), $user->id);
                                    //-------------

                                    if($hcpPoint <= 0) {
                                        $hcpData = ['points'  => $hcpPoint, 'status' => 'blocked'];
                                        $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);

                                        // Store Hcp Block
                                        $dataBlock = [
                                            'hcp_id' => $user->hcp->id,
                                            'created_by' => $user->id,
                                            'updated_by' => $user->id,
                                        ];

                                        $this->hcp_block_time->store($dataBlock);
                                        //----------------

                                        // Hcp Activity
                                        create_hcp_activity($user->hcp->id, 'account-blocked', __('hcp_activities.account_blocked'), $user->id);
                                        //-------------

                                    } else {
                                        $hcpData = ['points'  => $hcpPoint];
                                        $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                                    }
                                    //--------------
                                }


                                // Check Shift Swap Request
                                $shiftSwap = $this->shift_swap_request->find_shift_swap_request(['shift_id' => $shift->id, 'status' => 'pending']);
                                if(!empty($shiftSwap)){
                                    $this->shift_swap_request->update(['status' => 'cancelled'], ['id' => $shiftSwap->id]);

                                    if (!empty($shift['hcp_id'])) {
                                        $hcpId = $shift['hcp_id'];
                                        send_shift_push_notification($shift->id, 'shift-swap-cancelled', $hcpId);
                                    }
                                }
                                //-------------------------

                                // Shift Transfer
                                $shiftTransferData = [
                                    'shift_id' => $request['shift_id'],
                                    'hcp_id' => $shift['hcp_id'],
                                    'transfer_hcp_id' => $request['transfer_hcp_id'],
                                    'status' => 'pending',
                                    'created_by' => $user->id,
                                    'updated_by' => $user->id,
                                    'created_at' => now(),
                                    'updated_at' => now(),
                                ];
                                $shiftTransfer = $this->shift_transfer->store($shiftTransferData);
                                //---------------

                                // Update Shift
                                $shiftData = ['shift_transfer_swap_status' => 'transferred'];
                                $shiftUpdate = $this->shift->update($shiftData, ['id' => $request['shift_id']]);
                                //-------------

                                if (!empty($request['transfer_hcp_id']) && !empty($request['shift_id'])) {
                                    $hcpIds = $request['transfer_hcp_id'];
                                    send_shift_push_notification($request['shift_id'], 'shift-transfer', $hcpIds);
                                }

                                // Shift Activity
                                $shiftActivityData = [
                                    'hcp_id'        => $user->hcp->id,
                                    'shift_id'      => $request['shift_id'],
                                    'activity'      => 'shift-transfer-request-create',
                                    'created_by'    => $user->id,
                                    'updated_by'    => $user->id,
                                ];
                                $shiftActivity = $this->shift_activity->store($shiftActivityData);
                                //---------------

                                $message = __('messages.shift_transfer_successfully');

                            }

                        } else if($checkShiftTransfer->transfer_hcp_id == $request['transfer_hcp_id']) {

                            $shiftTransfer = null;
                            $message = __('messages.shift_already_transfer_to_this_hcp');

                        } else if($checkShiftTransfer->status != 'pending' && $checkShiftTransfer->status != 'accept') {

                            // Get Shift
                            $shiftWhere = ['id' => $request['shift_id']];
                            $shift = $this->shift->find_shift_with_where($shiftWhere);
                            //-----------

                            // Check Transfer Hcp Shift
                            $whereArry = ['hcp_id' => $request['transfer_hcp_id']];
                            $checkHcpShift = $this->shift->find_shift_with_date($shift['start_time'], $shift['end_time'], $whereArry);
                            //-------------------------

                            if (empty($checkHcpShift)) {

                                // Check Penalty
                                if($timeDiff >= $generalConfiguration['free_shift_cancel_period_for_hcp']){
                                    $penaltyPoint = $generalConfiguration['penalty_points_for_shift_cancellation'];

                                    // Update Hcp
                                    $hcpData = ['points'  => $user->hcp->points-$penaltyPoint];
                                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                                    //-----------
                                }
                                //--------------

                                // Shift Transfer
                                $shiftTransferData = [
                                    'shift_id' => $request['shift_id'],
                                    'hcp_id' => $shift['hcp_id'],
                                    'transfer_hcp_id' => $request['transfer_hcp_id'],
                                    'status' => 'pending',
                                    'created_by' => $user->id,
                                    'updated_by' => $user->id,
                                    'created_at' => now(),
                                    'updated_at' => now(),
                                ];
                                $shiftTransfer = $this->shift_transfer->store($shiftTransferData);
                                //---------------

                                // Update Shift
                                $shiftData = ['shift_transfer_swap_status' => 'transferred'];
                                $shiftUpdate = $this->shift->update($shiftData, ['id' => $request['shift_id']]);
                                //-------------

                                if (!empty($request['transfer_hcp_id']) && !empty($request['shift_id'])) {
                                    $hcpIds = $request['transfer_hcp_id'];
                                    send_shift_push_notification($request['shift_id'], 'shift-transfer', $hcpIds);
                                }

                                // Shift Activity
                                $shiftActivityData = [
                                    'hcp_id'        => $user->hcp->id,
                                    'shift_id'      => $request['shift_id'],
                                    'activity'      => 'shift-transfer-request-create',
                                    'created_by'    => $user->id,
                                    'updated_by'    => $user->id,
                                ];
                                $shiftActivity = $this->shift_activity->store($shiftActivityData);
                                //---------------

                                $message = __('messages.shift_transfer_successfully');

                            } else {
                                $shiftTransfer = null;
                                $message = __('messages.shift_cannot_transfer');
                            }

                        } else {
                            $shiftTransfer = null;
                            $message = __('messages.shift_transfer_action_pending');
                        }

                    } else {
                        $shiftTransfer = null;
                        $message = __('messages.shift_transfer_request_not_create_buddy');
                    }
                }
            } else {
                $shiftUpdate = null;
                $message = __('messages.shift_transfer_failed');
            }

            DB::commit();
        } catch (\Exception $e) {
            $shiftTransfer = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($shiftTransfer)) {



            $response = [
                'status' => true,
                'message' => $message,
                'data' => [],
            ];

        } else if(is_null($shiftTransfer) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Reject Transfer Request.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function rejectTransferRequest(ValidateRejectTransferShift $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $shiftTransfer = null;
        $shiftUpdate = null;
        $shiftActivity = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Get Transfer Shift
            $checkshiftTransferWhere = ['shift_id' => $request['shift_id'], 'transfer_hcp_id' => $user->hcp->id];
            $checkShiftTransfer = $this->shift_transfer->find_shift_transfer($checkshiftTransferWhere);
            //-------------------

            if(!empty($checkShiftTransfer)) {

                if (!empty($checkShiftTransfer['hcp_id'])) {
                    $hcpIds = $checkShiftTransfer['hcp_id'];
                    send_shift_push_notification($request['shift_id'], 'shift-transfer-rejected', $hcpIds);
                }

                // Shift Activity
                $shiftActivityData = [
                    'hcp_id'        => $user->hcp->id,
                    'shift_id'      => $request['shift_id'],
                    'activity'      => 'shift-transfer-request-rejected',
                    'created_by'    => $user->id,
                    'updated_by'    => $user->id,
                ];
                $shiftActivity = $this->shift_activity->store($shiftActivityData);
                //---------------

                // Update Shift Transfer
                $shiftTransferData = ['status' => 'reject'];
                $shiftTransfer = $this->shift_transfer->update($shiftTransferData, ['shift_id' => $request['shift_id']]);
                //----------------------

                $message = __('messages.shift_transfer_reject_successfully');

            } else {
                $shiftTransfer = null;
                $message = __('messages.shift_transfer_reject_hcp_error');
            }

            DB::commit();
        } catch (\Exception $e) {
            $shiftTransfer = null;
            $errorMessage = $e->getMessage();

            DB::rollback();
        }

        // Set response
        if (!empty($shiftTransfer)) {

            $response = [
                'status' => true,
                'message' => $message,
                'data' => [],
            ];

        } else if(is_null($shiftTransfer) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Accept Transfer Request.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function acceptTransferRequest(ValidateAcceptTransferShift $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $shiftTransfer = null;
        $shiftUpdate = null;
        $shiftActivity = null;
        $hcpHistoryCheck = false;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Get Transfer Shift
            $checkshiftTransferWhere = ['shift_id' => $request['shift_id'], 'transfer_hcp_id' => $user->hcp->id];
            $checkShiftTransfer = $this->shift_transfer->find_shift_transfer($checkshiftTransferWhere);
            //-------------------

            if($user->hcp->status == 'blocked') {
                $shiftTransfer = null;
                $message = __('messages.shift_applied_hcp_blocked');
            } else if($user->hcp->status == 'pending-approval') {
                $shiftTransfer = null;
                $message = __('messages.shift_applied_hcp_failed_pending_approval');
            } else if(!empty($checkShiftTransfer)) {

                // Get Shift
                $shiftWhere = ['id' => $request['shift_id']];
                $shift = $this->shift->find_shift_with_where($shiftWhere);
                //-----------

                // Find General Expire Documents
                $generalDocumentWhereArr = ['expiry_date' => Carbon::parse($shift['start_time'])->format('Y-m-d'), 'hcp_id' => $user->hcp->id];
                $generalExpireDocuments = $this->document_hcp->api_get_hcp_expire_general_document($generalDocumentWhereArr);
                //-------------------------------

                // Find Profession Expire Documents
                $professionDocumentWhereArr = ['expiry_date' => Carbon::parse($shift['start_time'])->format('Y-m-d'), 'profession_id' => $shift['profession_id'], 'hcp_id' => $user->hcp->id];
                $professionGeneralExpireDocuments = $this->document_hcp->api_get_hcp_expire_profession_document($professionDocumentWhereArr);
                //--------------------------------

                // Find General Pending Approval Documents
                $generalDocumentPendingApprovalWhereArr = ['hcp_id' => $user->hcp->id];
                $generalDocumentPendingApprovalDocuments = $this->document_hcp->api_get_hcp_pending_approval_general_document($generalDocumentPendingApprovalWhereArr);
                //----------------------------------------

                // Find Profession Pending Approval Documents
                $professionDocumentPendingApprovalWhereArr = ['profession_id' => $shift['profession_id'], 'hcp_id' => $user->hcp->id];
                $professionGeneralPendingApprovalDocuments = $this->document_hcp->api_get_hcp_pending_approval_profession_document($professionDocumentPendingApprovalWhereArr);
                //-------------------------------------------

                // Check Hcp Profession Status
                $hcpProfession = $this->hcp_profession->find_last_hcp_profession(['profession_id' => $shift['profession_id'], 'hcp_id' => $user->hcp->id]);
                //----------------------------

                // Check Hcp Shift History
                $shiftHistoryWehere = ['hcp_id' => $user->hcp->id, 'shift_id' => $request['shift_id']];
                $hcpHistory = $this->hcp->get_hcp_shift_history($shiftHistoryWehere);
                //------------------------

                if(!empty($hcpHistory) && ($hcpHistory['hcp_shift_transfers_count'] != '0' || $hcpHistory['hcp_transfer_shift_transfers_count'] != '0' || $hcpHistory['shift_hcp_swap_count'] != '0' || $hcpHistory['swap_hcp_shift_count'] != '0' || $hcpHistory['hcp_swap_rejected_request_count'] != '0')) {
                    $hcpHistoryCheck = true;
                }

                if (count($generalExpireDocuments)) {
                    $shiftTransfer = null;
                    $message = __('messages.shift_applied_hcp_failed_general_document', ['doucmentName' => $generalExpireDocuments[0]['document']['name']]);
                } else if (count($professionGeneralExpireDocuments)) {
                    $shiftTransfer = null;
                    $message = __('messages.shift_applied_hcp_failed_profession_document', ['doucmentName' => $professionGeneralExpireDocuments[0]['document']['name']]);
                } else if (count($generalDocumentPendingApprovalDocuments)) {
                    $shiftTransfer = null;
                    $message = __('messages.shift_applied_hcp_failed_document_pending_approval', ['doucmentName' => $generalDocumentPendingApprovalDocuments[0]['document']['name']]);
                } else if (count($professionGeneralPendingApprovalDocuments)) {
                    $shiftTransfer = null;
                    $message = __('messages.shift_applied_hcp_failed_document_pending_approval', ['doucmentName' => $professionGeneralPendingApprovalDocuments[0]['document']['name']]);
                } else if (!empty($hcpProfession) && $hcpProfession['status'] == 'inactive') {
                    $shiftTransfer = null;
                    $message = __('messages.shift_applied_hcp_failed_profession_status', ['professionName' => $hcpProfession['profession']['name'], 'status' => 'inactive']);
                } else if (!empty($hcpProfession) && $hcpProfession['status'] == 'pending-approval') {
                    $shiftTransfer = null;
                    $message = __('messages.shift_applied_hcp_failed_profession_status', ['professionName' => $hcpProfession['profession']['name'], 'status' => 'pending approval']);
                } else if (!empty($hcpProfession) && $hcpProfession['status'] == 'expired') {
                    $shiftTransfer = null;
                    $message = __('messages.shift_applied_hcp_failed_profession_status', ['professionName' => $hcpProfession['profession']['name'], 'status' => 'expired']);
                } else if($hcpHistoryCheck == false) {

                    // Set time zone
                    date_default_timezone_set($shift->state->time_zone_name);
                    //--------------

                    $currentTime = Carbon::now($shift->state->time_zone_name);

                    if (strtotime($currentTime) >= strtotime(Carbon::parse($shift['end_time']))) {
                        $shiftTransfer = null;
                        $message = __('messages.shift_transfer_end_time_error');
                    } else {

                        // Check Transfer Hcp Shift
                        $whereArry = ['hcp_id' => $user->hcp->id];
                        $checkHcpShift = $this->shift->find_shift_with_date($shift['start_time'], $shift['end_time'], $whereArry);
                        //-------------------------

                        if (empty($checkHcpShift)) {

                            if (!empty($shift['hcp_id'])) {
                                $hcpIds = $shift['hcp_id'];
                                send_shift_push_notification($request['shift_id'], 'shift-transfer-accepted', $hcpIds);
                            }

                            // Update Shift
                            $shiftData = ['hcp_id' => $user->hcp->id];
                            $shiftUpdate = $this->shift->update($shiftData, ['id' => $request['shift_id']]);
                            //-------------

                            // Shift Activity
                            $shiftActivityData = [
                                'hcp_id'        => $user->hcp->id,
                                'shift_id'      => $request['shift_id'],
                                'activity'      => 'shift-transfer-request-accepted',
                                'created_by'    => $user->id,
                                'updated_by'    => $user->id,
                            ];
                            $shiftActivity = $this->shift_activity->store($shiftActivityData);
                            //---------------

                            // Update Shift Transfer
                            $shiftTransferData = ['status' => 'accept'];
                            $shiftTransfer = $this->shift_transfer->update($shiftTransferData, ['id' => $checkShiftTransfer['id']]);
                            //----------------------

                            $message = __('messages.shift_transfer_accept_successfully');

                        } else {
                            $shiftTransfer = null;
                            $message = __('messages.shift_transfer_accept_hcp_error2');
                        }
                    }

                } else {
                    $shiftTransfer = null;
                    $message = __('messages.shift_history_found');
                }

            } else {
                $shiftTransfer = null;
                $message = __('messages.shift_transfer_accept_hcp_error');
            }

            DB::commit();
        } catch (\Exception $e) {
            $shiftTransfer = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($shiftTransfer)) {
            $shiftTransfer= $this->shift_transfer->find_shift_transfer($checkshiftTransferWhere);
            $generalConfiguration = get_general_configuration();

            if($generalConfiguration->send_email_on_shift_accepted=='1'){
                // Set email data
                $emailData['message'] = __('messages.shift_transfer_mail_subject');
                $emailData['subject'] = __('messages.shift_transfer_mail_subject');
                $emailData['sender'] = __('messages.shift_transfer_mail_subject');
                $emailData['email'] = env('SHIFT_ACTIVITY_EMAIL');
                $emailData['receiver'] = env('SHIFT_ACTIVITY_EMAIL');
                $emailData['shift'] = $shift;
                $emailData['shiftTransfer'] = $shiftTransfer;

                // Send email
                Mail::send('emails.admin-panel.shift-transfer-email', $emailData, function ($message) use ($emailData, $shift) {
                    $message->to($emailData['email'], $emailData['receiver'])->subject($emailData['subject']);
                    if(!empty($shift->facility->additional_nurse_email)){
                        $cc_emails=explode(",",$shift->facility->additional_nurse_email);
                        foreach($cc_emails as $cc){
                            $message->cc($cc);
                        }
                    }
                });
            }

            $response = [
                'status' => true,
                'message' => $message,
                'data' => [],
            ];

        } else if(is_null($shiftTransfer) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Create Swap Request.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function createSwapRequest(ValidateCreateSwapRequest $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $swapRequest = null;
        $message = null;
        $swapHcpIds = [];
        $penaltyPoint = 0;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // General Configuration
            $generalConfiguration = get_general_configuration();
            //----------------------

            $swapStartTime = $request['swap_date'].' '.$request['swap_start_time'];
            if($request['swap_end_time']<$request['swap_start_time'])
            {
                $swapEndTime = date('Y-m-d', strtotime($request['swap_date']. ' + 1 days')).' '.$request['swap_end_time'];
            }
            else{

                $swapEndTime = $request['swap_date'].' '.$request['swap_end_time'];
            }

            // // Check Hcp Shift
            // $whereArry = ['hcp_id' => $user->hcp->id];
            // $checkHcpShift = $this->shift->find_shift_with_date($swapStartTime, $swapEndTime, $whereArry);
            // //----------------

            // Check Swap Hcp Shift
            $swapHcpWhere = ['hcp_id' => $user->hcp->id, 'shift_id' => $request['shift_id'], 'status' => 'pending'];
            $swapRequestRecord = $this->shift_swap_request->find_shift_swap_request($swapHcpWhere);
            //---------------------

            // Get Shift
            $shift = $this->shift->find_shift($request['shift_id']);
            //----------

            // Set time zone
            date_default_timezone_set($shift->state->time_zone_name);
            //--------------

            $currentTime = Carbon::now($shift->state->time_zone_name);

            // Check Time Diff
            $to = Carbon::parse($shift['start_time']);
            $from = Carbon::parse($currentTime);
            $timeDiff = $to->diffInHours($from);
            //----------------

            if(!empty($shift) && $shift->hcp_id == $user->hcp->id) {
                if ($timeDiff <= $generalConfiguration['hcp_cannot_cancel_transfer_swap_with_hours']) {
                    $swapRequest = null;
                    $message = __('messages.shift_swap_request_failed');
                } else {
                    if($shift['shift_type'] != 'buddy') {

                        // Update Swap Request
                        if(!empty($swapRequestRecord)){
                            $this->shift_swap_request->update(['status' => 'cancelled'], ['id' => $swapRequestRecord['id']]);
                        }
                        //--------------------

                        // Find swap date hcp
                        $swapHcpShifts = $this->shift->find_swap_date_shift($swapStartTime, $swapEndTime, $shift, $request['range_of_job']);
                        //-------------------

                        $swapHcpIds = [];
                        if(!empty($swapHcpShifts)){
                            foreach($swapHcpShifts as $swapHcpShift) {

                                // Find swap shift date hcp
                                $hcpShift = $this->shift->find_swap_hcp_shift($shift['start_time'], $shift['end_time'], $shift, $request['range_of_job'], $swapHcpShift->hcp_id);
                                //-------------------------

                                if(empty($hcpShift)) {
                                    $swapHcpIds[] = $swapHcpShift['hcp_id'];
                                } else if(!empty($hcpShift) && $swapHcpShift['hcp_id'] != $hcpShift['hcp_id']) {
                                    $swapHcpIds[] = $swapHcpShift['hcp_id'];
                                }
                            }
                        }

                        if(count($swapHcpIds)) {

                            // Check Shift Transfer Request
                            $shiftTransfer = $this->shift_transfer->find_shift_transfer(['shift_id' => $shift->id, 'status' => 'pending']);
                            if(!empty($shiftTransfer)){

                                $shiftTransferUpdate = $this->shift_transfer->update(['status' => 'cancelled'], ['id' => $shiftTransfer['id']]);

                                if (!empty($shift['hcp_id'])) {
                                    $hcpId = $shift['hcp_id'];
                                    send_shift_push_notification($shift->id, 'shift-transfer-cancelled', $hcpId);
                                }
                            }
                            //-----------------------------

                            if ($timeDiff <= $generalConfiguration['free_shift_cancel_period_for_hcp']) {
                                // Update Hcp Point
                                $penaltyPoint = $generalConfiguration['penalty_points_for_shift_cancellation'];
                                $hcpPoint = $user->hcp->points-$penaltyPoint;

                                // Hcp Activity
                                create_hcp_activity($user->hcp->id, 'points-deduction', __('hcp_activities.points_deduction_swap', ['point' => $penaltyPoint, 'shiftId' => $shift->shift_prefix.$shift->id]), $user->id);
                                //-------------

                                if($hcpPoint <= 0) {
                                    $hcpData = ['points'  => $hcpPoint, 'status' => 'blocked'];
                                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);

                                    // Store Hcp Block
                                    $dataBlock = [
                                        'hcp_id' => $user->hcp->id,
                                        'created_by' => $user->id,
                                        'updated_by' => $user->id,
                                    ];

                                    $this->hcp_block_time->store($dataBlock);
                                    //----------------

                                    // Hcp Activity
                                    create_hcp_activity($user->hcp->id, 'account-blocked', __('hcp_activities.account_blocked'), $user->id);
                                    //-------------

                                } else {
                                    $hcpData = ['points'  => $hcpPoint];
                                    $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                                }
                                //-----------------
                            }

                            $data = [
                                'shift_id' => $request['shift_id'],
                                'hcp_id' => $user->hcp->id,
                                'swap_date' => $request['swap_date'],
                                'swap_start_time' => $request['swap_start_time'],
                                'swap_end_time' => $request['swap_end_time'],
                                'range_of_job' => $request['range_of_job'],
                                'request_sent_hcps' => $swapHcpIds ? implode(',', $swapHcpIds) : null,
                                'status' => 'pending',
                                'created_by' => $user->id,
                                'updated_by' => $user->id,
                            ];

                            $swapRequest = $this->shift_swap_request->store($data);

                            // Get Shift & Update Shift
                            $shiftData = ['shift_transfer_swap_status' => 'swapped'];
                            $shiftUpdate = $this->shift->update($shiftData, ['id' => $request['shift_id']]);
                            //-------------------------

                            // Send Notification
                            if (!empty($swapHcpIds)) {
                                send_shift_push_notification($request['shift_id'], 'shift-swap-request', $swapHcpIds, $swapRequest['id']);
                            }
                            //------------------

                            // Shift Activity
                            $shiftActivityData = [
                                'hcp_id'        => $user->hcp->id,
                                'shift_id'      => $request['shift_id'],
                                'activity'      => 'shift-swap-request-created',
                                'created_by'    => $user->id,
                                'updated_by'    => $user->id,
                            ];
                            $shiftActivity = $this->shift_activity->store($shiftActivityData);
                            //---------------

                            $message = __('messages.shift_swap_request_create_successfully', ['hcpCount' => count($swapHcpIds)]);

                        } else {
                            $swapRequest = null;
                            $message = __('messages.shift_swap_request_failed_no_hcp');
                        }

                    } else {
                        $swapRequest = null;
                        $message = __('messages.shift_swap_request_not_create_buddy');
                    }
                }
            } else {
                $swapRequest = null;
                $message = __('messages.shift_swap_request_failed');
            }

            DB::commit();
        } catch (\Exception $e) {
            $swapRequest = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($swapRequest)) {

            if($generalConfiguration->send_email_on_shift_swap==1){
                $swapRequest = $this->shift_swap_request->find_shift_swap_request($swapHcpWhere);
                // Set email data
                $emailData['message'] = __('messages.shift_swap_mail_subject');
                $emailData['subject'] = __('messages.shift_swap_mail_subject');
                $emailData['sender'] = __('messages.shift_swap_mail_subject');
                $emailData['email'] = env('SHIFT_ACTIVITY_EMAIL');
                $emailData['receiver'] = env('SHIFT_ACTIVITY_EMAIL');
                $emailData['shift'] = $shift;
                $emailData['swapRequest'] = $swapRequest;

                // Send email
                Mail::send('emails.admin-panel.shift-activity-email', $emailData, function ($message) use ($emailData) {
                    $message->to($emailData['email'], $emailData['receiver'])->subject($emailData['subject']);
                });
            }

            $response = [
                'status' => true,
                'message' => $message,
                'data' => [],
            ];

        } else if(is_null($swapRequest) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get Swap Requests.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getSwapRequests(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $swapRequests = [];
        $where = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            $swapRequests = $this->shift_swap_request->api_get_swap_requests($request, $user);

            DB::commit();
        } catch (\Exception $e) {
            $swapRequests = [];
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (count($swapRequests) > 0) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Swap Requests']),
                'data' => [
                    'swap_requests'  => $swapRequests,
                ],
            ];

        } else if(count($swapRequests) <= 0 && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Swap Requests']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Accept Swap Request.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function acceptSwapRequest(ValidateAcceptSwapRequest $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $swapRequest = [];
        $where = [];
        $hcpHistoryCheck = false;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Get Swap Shift
            $getSwapShift = $this->shift_swap_request->find_shift_swap_request(['shift_id' => $request['shift_id'], 'status' => 'pending']);
            //---------------

            if($user->hcp->status == 'blocked') {
                $swapRequest = null;
                $message = __('messages.shift_applied_hcp_blocked');
            } else if($user->hcp->status == 'pending-approval') {
                $swapRequest = null;
                $message = __('messages.shift_applied_hcp_failed_pending_approval');
            } else if(!empty($getSwapShift)) {

                // Get Shift (old shift)
                $shiftWhere = ['id' => $request['shift_id']];
                $shift = $this->shift->find_shift_with_where($shiftWhere);
                //----------------------

                // Get Swap Shift (new shift)
                $swapShiftWhere = ['id' => $request['swap_shift_id']];
                $swapShift = $this->shift->find_shift_with_where($swapShiftWhere);
                //--------------------------

                // Find General Expire Documents
                $generalDocumentWhereArr = ['expiry_date' => Carbon::parse($shift['start_time'])->format('Y-m-d'), 'hcp_id' => $user->hcp->id];
                $generalExpireDocuments = $this->document_hcp->api_get_hcp_expire_general_document($generalDocumentWhereArr);
                //-------------------------------

                // Find Profession Expire Documents
                $professionDocumentWhereArr = ['expiry_date' => Carbon::parse($shift['start_time'])->format('Y-m-d'), 'profession_id' => $shift['profession_id'], 'hcp_id' => $user->hcp->id];
                $professionGeneralExpireDocuments = $this->document_hcp->api_get_hcp_expire_profession_document($professionDocumentWhereArr);
                //--------------------------------

                // Find General Pending Approval Documents
                $generalDocumentPendingApprovalWhereArr = ['hcp_id' => $user->hcp->id];
                $generalDocumentPendingApprovalDocuments = $this->document_hcp->api_get_hcp_pending_approval_general_document($generalDocumentPendingApprovalWhereArr);
                //----------------------------------------

                // Find Profession Pending Approval Documents
                $professionDocumentPendingApprovalWhereArr = ['profession_id' => $shift['profession_id'], 'hcp_id' => $user->hcp->id];
                $professionGeneralPendingApprovalDocuments = $this->document_hcp->api_get_hcp_pending_approval_profession_document($professionDocumentPendingApprovalWhereArr);
                //-------------------------------------------

                // Check Hcp Profession Status
                $hcpProfession = $this->hcp_profession->find_last_hcp_profession(['profession_id' => $shift['profession_id'], 'hcp_id' => $user->hcp->id]);
                //----------------------------

                // Check Hcp Shift History
                $shiftHistoryWehere = ['hcp_id' => $user->hcp->id, 'shift_id' => $request['shift_id']];
                $hcpHistory = $this->hcp->get_hcp_shift_history($shiftHistoryWehere);
                //------------------------

                if(!empty($hcpHistory) && ($hcpHistory['hcp_shift_transfers_count'] != '0' || $hcpHistory['hcp_transfer_shift_transfers_count'] != '0' || $hcpHistory['shift_hcp_swap_count'] != '0' || $hcpHistory['swap_hcp_shift_count'] != '0' || $hcpHistory['hcp_swap_rejected_request_count'] != '0')) {
                    $hcpHistoryCheck = true;
                }

                if (count($generalExpireDocuments)) {
                    $swapRequest = null;
                    $message = __('messages.shift_applied_hcp_failed_general_document', ['doucmentName' => $generalExpireDocuments[0]['document']['name']]);
                } else if (count($professionGeneralExpireDocuments)) {
                    $swapRequest = null;
                    $message = __('messages.shift_applied_hcp_failed_profession_document', ['doucmentName' => $professionGeneralExpireDocuments[0]['document']['name']]);
                } else if (count($generalDocumentPendingApprovalDocuments)) {
                    $swapRequest = null;
                    $message = __('messages.shift_applied_hcp_failed_document_pending_approval', ['doucmentName' => $generalDocumentPendingApprovalDocuments[0]['document']['name']]);
                } else if (count($professionGeneralPendingApprovalDocuments)) {
                    $swapRequest = null;
                    $message = __('messages.shift_applied_hcp_failed_document_pending_approval', ['doucmentName' => $professionGeneralPendingApprovalDocuments[0]['document']['name']]);
                } else if (!empty($hcpProfession) && $hcpProfession['status'] == 'inactive') {
                    $swapRequest = null;
                    $message = __('messages.shift_applied_hcp_failed_profession_status', ['professionName' => $hcpProfession['profession']['name'], 'status' => 'inactive']);
                } else if (!empty($hcpProfession) && $hcpProfession['status'] == 'pending-approval') {
                    $swapRequest = null;
                    $message = __('messages.shift_applied_hcp_failed_profession_status', ['professionName' => $hcpProfession['profession']['name'], 'status' => 'pending approval']);
                } else if (!empty($hcpProfession) && $hcpProfession['status'] == 'expired') {
                    $swapRequest = null;
                    $message = __('messages.shift_applied_hcp_failed_profession_status', ['professionName' => $hcpProfession['profession']['name'], 'status' => 'expired']);
                } else if($hcpHistoryCheck == false) {

                    // Set time zone
                    date_default_timezone_set($shift->state->time_zone_name);
                    //--------------

                    $currentTime = Carbon::now($shift->state->time_zone_name);

                    if (strtotime($currentTime) >= strtotime(Carbon::parse($shift['end_time']))) {
                        $swapRequest = null;
                        $message = __('messages.shift_swap_request_end_time_error');
                    } else {

                        // Check Accept Hcp Shift
                        $checkHcpShift = $this->shift->find_swap_hcp_shift($shift['start_time'], $shift['end_time'], $shift, $getSwapShift['range_of_job'], $user->hcp->id);
                        //-----------------------


                        // Check Swap Hcp Shift
                        $checkSwapHcpShift = $this->shift->find_swap_hcp_shift($swapShift['start_time'], $swapShift['end_time'], $swapShift, $getSwapShift['range_of_job'], $getSwapShift->hcp_id);
                        //--------------------

                        if(empty($checkHcpShift) && empty($checkSwapHcpShift)) {

                            $swapStartTime = $getSwapShift['swap_date'].' '.$getSwapShift['swap_start_time'];
                            $swapEndTime = $getSwapShift['swap_date'].' '.$getSwapShift['swap_end_time'];

                            if(strtotime($swapShift['start_time']) == strtotime($swapStartTime) && strtotime($swapShift['end_time']) == strtotime($swapEndTime)) {

                                // Shift Activity
                                $shiftActivityData = [
                                    'hcp_id'        => $swapShift->hcp_id,
                                    'shift_id'      => $shift['id'],
                                    'activity'      => 'shift-swapped',
                                    'created_by'    => $swapShift['hcp']->user_id,
                                    'updated_by'    => $swapShift['hcp']->user_id,
                                ];
                                $shiftActivity = $this->shift_activity->store($shiftActivityData, ['id' => $shift['id']]);
                                //---------------

                                // Swap Shift Activity
                                $swapShiftActivityData = [
                                    'hcp_id'        => $shift['hcp_id'],
                                    'shift_id'      => $swapShift['id'],
                                    'activity'      => 'shift-swapped',
                                    'created_by'    => $shift['hcp']->user_id,
                                    'updated_by'    => $shift['hcp']->user_id,
                                ];
                                $swapShiftActivity = $this->shift_activity->store($swapShiftActivityData, ['id' => $swapShift['id']]);
                                //--------------------

                                if (!empty($getSwapShift['hcp_id'])) {
                                    $hcpIds = $getSwapShift['hcp_id'];
                                    send_shift_push_notification($getSwapShift['shift_id'], 'shift-swap-accepted', $hcpIds);
                                }

                                // Update Swap Request
                                $swapRequestData = ['status' => 'accept', 'swap_shift_id' => $request['swap_shift_id'] , 'swap_hcp_id' => $user->hcp->id];
                                $swapRequestWhere = ['shift_id' => $request['shift_id']];
                                $swapRequest = $this->shift_swap_request->update($swapRequestData, $swapRequestWhere);
                                //--------------------

                                // Update Shift
                                $shiftData = ['hcp_id' => $user->hcp->id, 'shift_transfer_swap_status' => 'swapped'];
                                $shiftUpdate = $this->shift->update($shiftData, ['id' => $shift['id']]);
                                //-------------

                                // Update Swap Shift
                                $swapShiftData = ['hcp_id' => $shift['hcp_id'], 'shift_transfer_swap_status' => 'swapped'];
                                $swapShiftUpdate = $this->shift->update($swapShiftData, ['id' => $swapShift['id']]);
                                //------------------

                                $message = __('messages.shift_swap_successfully');

                            } else {
                                $swapRequest = null;
                                $message = __('messages.shift_swap_date_time_not_match');
                            }

                        } else {
                            $swapRequest = null;
                            $message = __('messages.shift_cannot_swap_same_date_shift');
                        }

                    }
                } else {
                    $swapRequest = null;
                    $message = __('messages.shift_history_found');
                }

            } else {
                $swapRequest = null;
                $message = __('messages.shift_already_swap');
            }

            DB::commit();
        } catch (\Exception $e) {
            $swapRequest = [];
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($swapRequest)) {
            $swapRequest = $this->shift_swap_request->find_shift_swap_request(['shift_id' => $request['shift_id']]);

            $generalConfiguration = get_general_configuration();

            if($generalConfiguration->send_email_on_shift_accepted==1){
                // Set email data
                $emailData['message'] = __('messages.shift_accepted_mail_subject');
                $emailData['subject'] = __('messages.shift_accepted_mail_subject');
                $emailData['sender'] = __('messages.shift_accepted_mail_subject');
                $emailData['email'] = env('SHIFT_ACTIVITY_EMAIL');
                $emailData['receiver'] = env('SHIFT_ACTIVITY_EMAIL');
                $emailData['shift'] = $shift;
                $emailData['swapRequest'] = $swapRequest;

                // Send email
                Mail::send('emails.admin-panel.shift-activity-email', $emailData, function ($message) use ($emailData) {
                    $message->to($emailData['email'], $emailData['receiver'])->subject($emailData['subject']);
                });
            }

            $response = [
                'status' => true,
                'message' => $message,
                'data' => null,
            ];

        } else if(is_null($swapRequest) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get Swap Requests.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getSwapRequestDetails(ValidateSwapRequestDetails $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $swapRequest = null;
        $shift = null;
        $where = [];
        $shiftRequestDetails = false;
        $message = null;
        $checkshiftSwaprCancelledRequest = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Get Swapp Request
            $swapRequest = $this->shift_swap_request->api_find_shift_swap_request($request, $user);
            //------------------

            if(!empty($swapRequest) && $swapRequest['hcp_id'] != $user->hcp->id){

                // Get Shift Details
                $request['start_time'] = $swapRequest['swap_date'].' '.$swapRequest['swap_start_time'];
                $request['end_time'] = $swapRequest['swap_date'].' '.$swapRequest['swap_end_time'];
                $shift = $this->shift->api_get_hcp_shift_detail($request, $user);
                //------------------

            }

            if(!empty($swapRequest)) {
                $checkshiftSwaprCancelledRequestWhere = ['shift_id' => $swapRequest['shift_id'], 'hcp_id' => $user->hcp->id];
                $checkshiftSwaprCancelledRequest = $this->shift_swap_rejected_request->find_shift_swap_rejected_request($checkshiftSwaprCancelledRequestWhere);
            }

            if(empty($swapRequest)) {
                $shiftRequestDetails = false;
                $message = __('messages.shift_swap_request_details_failed');
            } else if(!empty($checkshiftSwaprCancelledRequest)) {
                $shiftRequestDetails = false;
                $message = __('messages.shift_swap_request_details_failed');
            } else if(!empty($swapRequest) && empty($shift)) {
                $shiftRequestDetails = false;
                $message = __('messages.shift_swap_request_details_current_shift_failed');
            } else {
                $shiftRequestDetails = true;
                $message = __('messages.records_found', ['records' => 'Swap Requests']);
            }

            DB::commit();
        } catch (\Exception $e) {
            $shiftRequestDetails = false;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if ($shiftRequestDetails == true) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Swap Requests']),
                'data' => [
                    'my_current_shift'  => $shift,
                    'swap_request'  => $swapRequest,
                ],
            ];

        } else if($shiftRequestDetails == false && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Reject Swap Request.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function rejectSwapRequest(ValidateRejectSwapRequest $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $rejectedSwapShiftRequest = null;
        $shiftActivity = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Get Swap Shift Request
            $rejectedSwapShiftRequest = $this->shift_swap_rejected_request->find_shift_swap_rejected_request(['shift_id' => $request['shift_id'], 'swap_request_id' => $request['swap_request_id'], 'hcp_id' => $user->hcp->id]);
            //-----------------------

            if(empty($rejectedSwapShiftRequest)) {
                $rejectedShiftData = [
                    'shift_id'      => $request['shift_id'],
                    'swap_request_id'      => $request['swap_request_id'],
                    'hcp_id'        => $user->hcp->id,
                    'created_by'    => $user->id,
                    'updated_by'    => $user->id,
                ];
                $rejectedSwapShiftRequest = $this->shift_swap_rejected_request->store($rejectedShiftData);

                // Shift Activity
                $shiftActivityData = [
                    'hcp_id'        => $user->hcp->id,
                    'shift_id'      => $request['shift_id'],
                    'activity'      => 'shift-swap-request-rejected',
                    'created_by'    => $user->id,
                    'updated_by'    => $user->id,
                ];
                $shiftActivity = $this->shift_activity->store($shiftActivityData, ['id' => $request['shift_id']]);
                //---------------

                $message = __('messages.shift_swap_reject_successfully');
            } else {
                $rejectedSwapShiftRequest = null;
                $message = __('messages.shift_swap_reject_failed');
            }

            DB::commit();
        } catch (\Exception $e) {
            $rejectedSwapShiftRequest = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($rejectedSwapShiftRequest)) {

            $response = [
                'status' => true,
                'message' => $message,
                'data' => [],
            ];

        } else if(is_null($rejectedSwapShiftRequest) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Give Rating.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function giveRating(ValidateGiveRating $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $ratingReview = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            $data = [
                'shift_id' => $request['shift_id'],
                'hcp_id' => $user->hcp->id,
                'date_time' => now(),
                'facility_id' => $request['facility_id'],
                'facility_rating' => $request['rating'],
                'category_id' => $request['category_id'],
                'profession_id' => $request['profession_id'],
                'comment' => $request['comment'],
                'created_by' => $user->id,
                'updated_by' => $user->id,
            ];

            $ratingReview = $this->ratingReview->store($data);

            DB::commit();
        } catch (\Exception $e) {
            $ratingReview = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($ratingReview)) {

            $response = [
                'status' => true,
                'message' => __('messages.record_created', ['record' => 'Rating']),
                'data' => [],
            ];

        } else if(is_null($ratingReview) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.record_creation_failed', ['record' => 'Rating']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Create Buddy Shift Request.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function createBuddyShiftRequest(ValidateCreateBuddyShiftRequest $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $buddyShiftRequest = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Check Profession
            $hcpProfession = $this->hcp_profession->find_last_hcp_profession(['id' => $request['hcp_profession_id']]);
            //-----------------

            if ($user->hcp->status == 'active') {
                if (!empty($hcpProfession)) {
                    if ($hcpProfession['status'] != 'pending-approval') {
                        $buddyWhere = [
                            ['hcp_id', '=', $user->hcp->id],
                            ['hcp_profession_id', '=', $request['hcp_profession_id']],
                            ['status', '!=', 'cancelled']
                        ];
                        $buddyRequest = $this->buddy_shift_request->find_buddy_shift_request($buddyWhere);

                        if(empty($buddyRequest)) {
                            $data = [
                                'hcp_id'        => $user->hcp->id,
                                'category_id'   => $request['category_id'],
                                'profession_id' => $request['profession_id'],
                                'level_id'      => $request['level_id'] ?? null,
                                'hcp_profession_id'   => $request['hcp_profession_id'],
                                'status'        => 'requested',
                                'created_by'    => $user->id,
                                'updated_by'    => $user->id,
                            ];

                            $buddyShiftRequest = $this->buddy_shift_request->store($data);
                            $message = __('messages.buddy_shift_request_create_successfully');
                        } else {
                            $buddyShiftRequest = null;
                            $message = __('messages.buddy_shift_request_create_failed');
                        }
                    } else {
                        $buddyShiftRequest = null;
                        $message = __('messages.buddy_shift_request_profession_pending_approval');
                    }
                } else {
                    $buddyShiftRequest = null;
                    $message = __('messages.buddy_shift_request_create_failed_new');
                }
            } else {
                $buddyShiftRequest = null;
                $message = __('messages.buddy_shift_request_create_hcp_failed');
            }

            DB::commit();
        } catch (\Exception $e) {
            $buddyShiftRequest = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($buddyShiftRequest)) {

            $response = [
                'status' => true,
                'message' => $message,
                'data' => [],
            ];

        } else if(is_null($buddyShiftRequest) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Cancel Shifts.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function cancelShift(ValidateCancelShift $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $cancelledShift = null;
        $shiftActivity = null;
        $shift = null;
        $penaltyPoint = 0;

        // Get User
        $user = auth()->user();
        //---------

        // General Configuration
        $generalConfiguration = get_general_configuration();
        //----------------------

        DB::beginTransaction();
        try {

            // Get Shift
            $shift = $this->shift->find_shift($request['shift_id']);
            //----------

            if(!empty($shift) && $shift->hcp_id == $user->hcp->id) {

                // Set time zone
                date_default_timezone_set($shift->state->time_zone_name);
                //--------------

                $cancelTime = Carbon::now($shift->state->time_zone_name);
                $currentTime = Carbon::now($shift->state->time_zone_name);

                // Get Cancelled Shift
                $cancelledShift = $this->cancelled_shift->find_cancelled_shift(['shift_id' => $request['shift_id'], 'hcp_id' => $user->hcp->id]);
                //--------------------

                // Check Time Diff
                $to = Carbon::parse($shift['start_time']);
                $from = Carbon::parse($currentTime);
                $timeDiff = $to->diffInHours($from);
                //----------------
//

                if ($shift->shift_type != 'buddy') {

                    if(empty($cancelledShift) && $timeDiff >= $generalConfiguration['hcp_cannot_cancel_transfer_swap_with_hours']) {

                        // Check Penalty
                        if($timeDiff <= $generalConfiguration['free_shift_cancel_period_for_hcp']){
                            $penaltyPoint = $generalConfiguration['penalty_points_for_shift_cancellation'];
                            $hcpPoint = $user->hcp->points-$penaltyPoint;

                            // Hcp Activity
                            create_hcp_activity($user->hcp->id, 'points-deduction', __('hcp_activities.points_deduction_cancelled', ['point' => $penaltyPoint, 'shiftId' => $shift->shift_prefix.$shift->id]), $user->id);
                            //-------------

                            // Update Hcp
                            if($hcpPoint <= 0) {
                                $hcpData = ['points'  => $hcpPoint, 'status' => 'blocked'];
                                $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);

                                // Store Hcp Block
                                $dataBlock = [
                                    'hcp_id' => $user->hcp->id,
                                    'created_by' => $user->id,
                                    'updated_by' => $user->id,
                                ];

                                $this->hcp_block_time->store($dataBlock);
                                //----------------

                                // Hcp Activity
                                create_hcp_activity($user->hcp->id, 'account-blocked', __('hcp_activities.account_blocked'), $user->id);
                                //-------------

                            } else {
                                $hcpData = ['points'  => $hcpPoint];
                                $hcpUpdate = $this->hcp->update($hcpData, ['id' => $user->hcp->id]);
                            }
                            //-----------
                        }
                        //--------------

                        // Check Shift Transfer Request
                        $shiftTransfer = $this->shift_transfer->find_shift_transfer(['shift_id' => $shift->id, 'status' => 'pending']);
                        if(!empty($shiftTransfer)){
                            $shiftTransferUpdate = $this->shift_transfer->update(['status' => 'cancelled'], ['id' => $shiftTransfer['id']]);
                            if (!empty($shift['hcp_id'])) {
                                $hcpId = $shift['hcp_id'];
                                send_shift_push_notification($shift->id, 'shift-transfer-cancelled', $hcpId);
                            }
                        }
                        //-----------------------------

                        // Check Shift Swap Request
                        $shiftSwap = $this->shift_swap_request->find_shift_swap_request(['shift_id' => $shift->id, 'status' => 'pending']);
                        if(!empty($shiftSwap)){
                            $this->shift_swap_request->update(['status' => 'cancelled'], ['id' => $shiftSwap->id]);
                            if (!empty($shift['hcp_id'])) {
                                $hcpId = $shift['hcp_id'];
                                send_shift_push_notification($shift->id, 'shift-swap-cancelled', $hcpId);
                            }
                        }
                        //-------------------------

                        // Check Buddy Shift Request
                        if ($shift->shift_type == 'buddy') {
                            $buddyshiftRequest = $this->buddy_shift_request->find_buddy_shift_request(['shift_id' => $shift->id]);
                            if(!empty($buddyshiftRequest)){
                                $this->buddy_shift_request->update(['status' => 'cancelled'], ['id' => $buddyshiftRequest->id]);
                            }
                        }
                        //--------------------------

                        // Cancel Trainee Shift
                        if (!empty($shift->trainee_hcp_id)) {
                            $buddyshiftRequest = $this->buddy_shift_request->find_buddy_shift_request(['shift_id' => $shift->trainee_buddy_shift_id]);
                            if(!empty($buddyshiftRequest)){
                                $this->buddy_shift_request->update(['status' => 'cancelled'], ['id' => $buddyshiftRequest->id]);
                            }

                            $shiftTraineeData = ['shift_status' => '3'];
                            $this->shift->update($shiftTraineeData, ['id' => $shift->trainee_buddy_shift_id]);

                            // Send Cancel Shift Notification To Trainee Hcp,Admin,Facility
                            if (!empty($shift['trainee_hcp_id'])) {
                                send_shift_push_notification($shift->trainee_buddy_shift_id, 'trainee-buddy-shift-cancelled', $shift['trainee_hcp_id']);
                                send_admin_notification('hcp-shift-cancelled-admin', 'admin', $shift->trainee_buddy_shift_id, $shift['trainee_hcp_id']);
                                send_admin_notification('hcp-shift-cancelled-facility', 'facility', $shift->trainee_buddy_shift_id, $shift['trainee_hcp_id']);
                            }
                            //----------------------------------------------
                        }
                        //---------------------

                        // Update Shift Broadcast
                        $shiftBroadcastWhere = ['id' => $shift->id];
                        $this->shift->update(['shift_broadcast' => 'favourite', 'shift_rebroadcast_time' => null], $shiftBroadcastWhere);
                        remove_new_shift_notification($shift->id);
                        // ShiftBroadcastJob::dispatch()->delay(now()->addSeconds(1));
                        //-----------------------

                        // Send Cancel Shift Notification To Hcp,Admin,Facility
                        if (!empty($shift['hcp_id'])) {
                            send_shift_push_notification($shift->id, 'shift-cancelled', $shift['hcp_id']);
                            send_admin_notification('hcp-shift-cancelled-admin', 'admin', $shift->id, $shift['hcp_id']);
                            send_admin_notification('hcp-shift-cancelled-facility', 'facility', $shift->id, $shift['hcp_id']);
                        }
                        //-----------------------------------------------------

                        // Update Shift
                        if ($shift->shift_type == 'buddy') {
                            $mentorShiftData = ['trainee_hcp_name' => null, 'trainee_hcp_email' => null, 'trainee_hcp_id' => null, 'trainee_buddy_shift_id' => null];
                            $this->shift->update($mentorShiftData, ['trainee_buddy_shift_id' => $shift->id]);

                            $shiftData = ['shift_status' => '3'];
                            $shiftUpdate = $this->shift->update($shiftData, ['id' => $request['shift_id']]);
                        } else {
                            $shiftData = ['hcp_id' => null, 'shift_status' => '0'];
                            $shiftUpdate = $this->shift->update($shiftData, ['id' => $request['shift_id']]);
                        }
                        //-------------

                        $cancelledShiftData = [
                            'shift_id'      => $request['shift_id'],
                            'hcp_id'        => $user->hcp->id,
                            'cancel_by'     => 'hcp',
                            'cancel_time'   => now(),
                            'cancel_reason' => $request['cancel_reason'],
                            'negative_points' => $penaltyPoint,
                            'created_by'    => $user->id,
                            'updated_by'    => $user->id,
                        ];
                        $cancelledShift = $this->cancelled_shift->store($cancelledShiftData);

                        // Shift Activity
                        $shiftActivityData = [
                            'hcp_id'        => $user->hcp->id,
                            'shift_id'      => $request['shift_id'],
                            'activity'      => 'shift-cancelled',
                            'created_by'    => $user->id,
                            'updated_by'    => $user->id,
                        ];
                        $shiftActivity = $this->shift_activity->store($shiftActivityData, ['id' => $request['shift_id']]);
                        //---------------

                        $message = __('messages.shift_cancel_successfully');
                    } else {
                        $cancelledShift = null;
                        $message = __('messages.shift_cancel_failed');
                    }
                } else {
                    $cancelledShift = null;
                    $message = __('messages.shift_buddy_cancel_failed');
                }

            } else {
                $cancelledShift = null;
                $message = __('messages.shift_cancel_failed');
            }

            DB::commit();
        } catch (\Exception $e) {
            $cancelledShift = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($cancelledShift)) {

            if($generalConfiguration->send_email_on_shift_cancelled==1){
                // Set email data
                $emailData['message'] = __('messages.shift_cancel_mail_subject');
                $emailData['subject'] = __('messages.shift_cancel_mail_subject');
                $emailData['sender'] = __('messages.shift_cancel_mail_subject');
                $emailData['email'] = env('SHIFT_ACTIVITY_EMAIL');
                $emailData['receiver'] = env('SHIFT_ACTIVITY_EMAIL');
                $emailData['shift'] = $shift;
                $emailData['cancelledShift'] = $cancelledShift;

                // Send email
                Mail::send('emails.admin-panel.shift-activity-email', $emailData, function ($message) use ($emailData, $shift) {
                    $message->to($emailData['email'], $emailData['receiver'])->subject($emailData['subject']);
                    if(!empty($shift->facility->additional_nurse_email)){
                        $cc_emails=explode(",",$shift->facility->additional_nurse_email);
                        foreach($cc_emails as $cc){
                            $message->cc($cc);
                        }
                    }
                });
            }

            $response = [
                'status' => true,
                'message' => $message,
                'data' => [],
            ];

        } else if(is_null($cancelledShift) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Get Buddy Shifts.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function getBuddyShifts(Request $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $shifts = [];
        $where = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            $shifts = $this->shift->api_get_buddy_shifts($request, $user);

            DB::commit();
        } catch (\Exception $e) {
            $shifts = [];
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (count($shifts) > 0) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Shifts']),
                'data' => [
                    'shifts'  => $shifts,
                ],
            ];

        } else if(count($shifts) <= 0 && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Shifts']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Shift Tracking.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function shiftTracking(ValidateShiftTracking $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $shiftTracking = null;
        $shiftActivity = null;
        $hcpCoordinates =  [];
        $hcpReached = false;
        $distance = '';
        $hcpReachedArr = [];

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Get Shift
            $shift = $this->shift->find_shift($request['shift_id']);
            //----------

            // Set time zone
            date_default_timezone_set($shift->state->time_zone_name);
            //--------------

            $currentTime = Carbon::now($shift->state->time_zone_name);

            // General Configuration
            $generalConfiguration = get_general_configuration();
            $startMint = $generalConfiguration['shift_start_before_mint'];
            $radius = $generalConfiguration['radius_km_for_shift_start'];
            //----------------------

            // Get Shift Tracking
            $where = ['shift_id' => $request['shift_id'], 'hcp_id' => $user->hcp->id];
            $shiftTracking = $this->shift_tracking->find_shift_tracking($where);
            //-------------------

            // Check Hcp Reached On Time
            if (!empty($shiftTracking) && $shiftTracking['shift']['shift_status'] == '1') {
                $shiftStartTime = Carbon::parse($shiftTracking['shift']['start_time'])->subMinutes($startMint);
                if (!empty($request['latitude']) && !empty($request['longitude'])) {
                    $lat1 = $shiftTracking['shift']['facility']['latitude'];
                    $lon1 = $shiftTracking['shift']['facility']['longitude'];
                    $lat2 = $request['latitude'];
                    $lon2 = $request['longitude'];
                    $unit = 'M';
                    $distance = get_facility_and_hcp_distance($lat1, $lon1, $lat2, $lon2, $unit);
                }

                if (strtotime($currentTime) >= strtotime($shiftStartTime) && strtotime($currentTime) <= strtotime(Carbon::parse($shiftTracking['shift']['end_time'])) && $distance <= $radius) {
                    $hcpReached = true;

                    $hcpReachedArr = [
                        'hcp_reached' => $hcpReached,
                        'shift_id' => $shiftTracking['shift_id'],
                        'distance' => $distance,
                        'time' => $currentTime,
                    ];
                }
            }
            //--------------------------

            if (empty($shiftTracking)) {

                $hcpCoordinates[] = [
                    'latitude'  => $request['latitude'],
                    'longitude'  => $request['longitude'],
                    'time'  => date('d-m-Y H:i'),
                ];

                $data = [
                    'hcp_id'        => $user->hcp->id,
                    'shift_id'      => $request['shift_id'],
                    'facility_id'   => $request['facility_id'],
                    'latitude'      => $request['latitude'],
                    'longitude'     => $request['longitude'],
                    'hcp_coordinates'     => json_encode($hcpCoordinates, true),
                    'created_by'    => $user->id,
                    'updated_by'    => $user->id,
                ];

                $shiftTracking = $this->shift_tracking->store($data);
            } else {

                $hcpCoordinatesNew[] = [
                    'latitude'  => $request['latitude'],
                    'longitude'  => $request['longitude'],
                    'time'  => date('d-m-Y H:i'),
                ];

                $hcpCoordinates = json_decode($shiftTracking->hcp_coordinates, true);
                $newHcpCoordinates = array_merge($hcpCoordinates,$hcpCoordinatesNew);


                $data = [
                    'hcp_id'        => $user->hcp->id,
                    'shift_id'      => $request['shift_id'],
                    'facility_id'   => $request['facility_id'],
                    'latitude'      => $request['latitude'],
                    'longitude'     => $request['longitude'],
                    'hcp_coordinates' => json_encode($newHcpCoordinates, true),
                    'updated_by'    => $user->id,
                ];

                $where = ['id' => $shiftTracking->id];
                $shiftTracking = $this->shift_tracking->update($data, $where);
            }

            DB::commit();
        } catch (\Exception $e) {
            $shiftTracking = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($shiftTracking)) {
            $response = [
                'status' => true,
                'message' => __('messages.record_created', ['record' => 'Shift Tracking']),
                'data' => $hcpReachedArr,
            ];

        } else if(is_null($shiftTracking) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.record_creation_failed', ['record' => 'Shift Tracking']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Start shifts.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function startShift(ValidateStartShift $request)
    {
        // Get User
        $user = auth()->user();
        //---------

        $responseCode = 200;
        $errorMessage = null;
        $shiftUpdate = null;
        $shiftActivity = null;
        $distance = null;

        // General Configuration
        $generalConfiguration = get_general_configuration();
        $startMint = $generalConfiguration['shift_start_before_mint'];
        $radius = $generalConfiguration['radius_km_for_shift_start'];
        //----------------------

        DB::beginTransaction();
        try {

            // Get Shift
            $shiftWhere = ['id' => $request['shift_id'], 'hcp_id' => $user->hcp->id];
            $shift = $this->shift->api_shift_where($shiftWhere);
            //----------

            if(!empty($shift)) {

                // Set time zone
                date_default_timezone_set($shift->state->time_zone_name);
                //--------------

                $currentTime = Carbon::now($shift->state->time_zone_name);

                if(strtotime($currentTime) >= strtotime(Carbon::parse($shift['end_time']))) {
                    $shiftUpdate = null;
                    $message = __('messages.shift_start_end_time_error');
                } else {

                    if (!empty($shift['shift_hcp_loaction'])) {
                        $lat1 = $shift['facility']['latitude'];
                        $lon1 = $shift['facility']['longitude'];
                        $lat2 = $shift['shift_hcp_loaction']['latitude'];
                        $lon2 = $shift['shift_hcp_loaction']['longitude'];
                        $unit = 'M';
                        $distance = get_facility_and_hcp_distance($lat1, $lon1, $lat2, $lon2, $unit);
                    }

                    if($shift->shift_status == '1') {
                        $shiftStartTime = Carbon::parse($shift->start_time)->subMinutes($startMint);
                        if (strtotime($currentTime) >= strtotime($shiftStartTime)) {
                            if(!empty($shift['shift_hcp_loaction'])) {
                                if ($distance <= $radius) {

                                    // Check Shift Transfer Request
                                    $shiftTransfer = $this->shift_transfer->find_shift_transfer(['shift_id' => $shift->id, 'status' => 'pending']);
                                    if(!empty($shiftTransfer)){
                                        $shiftTransferUpdate = $this->shift_transfer->update(['status' => 'cancelled'], ['id' => $shiftTransfer['id']]);
                                        if (!empty($shift['hcp_id'])) {
                                            $hcpId = $shift['hcp_id'];
                                            send_shift_push_notification($shift->id, 'shift-transfer-cancelled', $hcpId);
                                        }
                                    }
                                    //-----------------------------

                                    // Check Shift Swap Request
                                    $shiftSwap = $this->shift_swap_request->find_shift_swap_request(['shift_id' => $shift->id, 'status' => 'pending']);
                                    if(!empty($shiftSwap)){
                                        $this->shift_swap_request->update(['status' => 'cancelled'], ['id' => $shiftSwap->id]);
                                        if (!empty($shift['hcp_id'])) {
                                            $hcpId = $shift['hcp_id'];
                                            send_shift_push_notification($shift->id, 'shift-swap-cancelled', $hcpId);
                                        }
                                    }
                                    //-------------------------

                                    if (strtotime($currentTime) >= strtotime($shift->start_time)) {
                                        $startTime = $currentTime;
                                    } else {
                                        $startTime = $shift->start_time;
                                    }

                                    // Update Shift
                                    $shiftData = ['shift_status' => '2', 'hcp_shift_start_time' => $startTime];
                                    $shiftUpdate = $this->shift->update($shiftData, ['id' => $request['shift_id']]);
                                    //-------------

                                    // Update buddy shift request
                                    if(!empty($shift) &&  $shift['shift_type'] == 'buddy'){
                                        $this->buddy_shift_request->update(['status' => 'running'], ['shift_id' => $shift->id]);
                                    }
                                    //---------------------------

                                    // Shift Activity
                                    $shiftActivityData = [
                                        'hcp_id'        => $user->hcp->id,
                                        'shift_id'      => $request['shift_id'],
                                        'activity'      => 'shift-started',
                                        'created_by'    => $user->id,
                                        'updated_by'    => $user->id,
                                    ];
                                    $shiftActivity = $this->shift_activity->store($shiftActivityData);
                                    //---------------

                                    if (!empty($shift['hcp_id'])) {
                                        $hcpId = $shift['hcp_id'];
                                        send_shift_push_notification($shift->id, 'shift-start', $hcpId);
                                    }

                                    $message = __('messages.shift_start_successfully');
                                } else {
                                    $shiftUpdate = null;

                                    $km = $distance / 1000;
                                    if($km < 1) {
                                        $distanceMsg = round($km).' M';
                                    } else {
                                        $distanceMsg = round($km).' K';
                                    }

                                    $message = __('messages.shift_start_on_location', ['distance' => $distanceMsg]);
                                }
                            } else {
                                $shiftUpdate = null;
                                $message = __('messages.shift_start_no_location_found');
                            }
                        } else {
                            $shiftUpdate = null;
                            $message = __('messages.shift_cannot_start_before_time');
                        }
                    } else {
                        $shiftUpdate = null;
                        $message = __('messages.shift_already_start');
                    }
                }

            } else {
                $shiftUpdate = null;
                $message = __('messages.shift_start_failed');
            }

            DB::commit();
        } catch (\Exception $e) {
            $shiftUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($shiftUpdate)) {

            $response = [
                'status' => true,
                'message' => $message,
                'data' => [],
            ];

        } else if(is_null($shiftUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => [],
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => [],
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Shift payment.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function shiftPayment(ValidateShiftPayment $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $shiftPayment = null;
        $paymentRecords = [];

        // Get User
        $user = auth()->user();
        //---------

        // Get Current Date
        $firstOfMonth = Carbon::now()->firstOfMonth();
        $lastOfMonth = Carbon::now()->lastOfMonth();
        //-----------------

        DB::beginTransaction();
        try {
            $monthDates = [];
            if(isset($request['filter_start_date']) && isset($request['filter_end_date']) && !empty($request['filter_start_date']) && !empty($request['filter_end_date'])) {
                $filterStartDate = Carbon::parse($request['filter_start_date']);
                $filterEndDate = Carbon::parse($request['filter_end_date']);
                $year = $filterStartDate->format('Y');
                $weekCount = $filterStartDate->diffInWeeks($filterEndDate);
                $monthDates = [
                    'start_date' => $filterStartDate,
                    'end_date' => $filterEndDate,
                ];
            } else {
                $weekCount = $firstOfMonth->diffInWeeks($lastOfMonth);
                $year = $firstOfMonth->format('Y');
                $filterStartDate = $firstOfMonth;
                $filterEndDate = $lastOfMonth;
            }



            $transactions = $this->transaction->api_hcp_tranactions_list($request, $user, $monthDates);

            if(count($transactions)) {
                $statementDateRange = '';
                foreach($transactions as $transaction) {
                    $statementDateRange = explode(' - ', $transaction[0]->invoice_generated_date_range);
                    $paymentRecords[] = [
                        'statement_prefix' => $transaction[0]->statement_prefix,
                        'statement_id' => $transaction[0]->statement_id,
                        'start_date' => isset($statementDateRange[0])?$statementDateRange[0]:'',
                        'end_date' => isset($statementDateRange[1])?$statementDateRange[1]:'',
                        'total_payable' => round($transaction->sum('hcp_amount'), 2),
                        'reffer_amount' => round($transaction->where('amount_type', 'refer-and-earn')->sum('hcp_amount'), 2),
                        'shift_earning' => round($transaction->where('amount_type', 'shift')->sum('shift_earning'), 2),
                        'total_working_hours' => round($transaction->where('amount_type', 'shift')->sum('working_hours'), 2),
                        'total_shift' => $transaction->where('amount_type', 'shift')->whereNotNull('shift_count')->count(),
                    ];
                }
            }

            DB::commit();
        } catch (\Exception $e) {
            $paymentRecords = [];
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (count($paymentRecords) > 0) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Payment']),
                'data' => [
                    'date_range' => $filterStartDate->format('d M Y').' - '.$filterEndDate->format('d M Y'),
                    'payments' => $paymentRecords,
                ],
            ];

        } else if(count($paymentRecords) <= 0 && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Payment']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * Shift payment details.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function shiftPaymentDetails(ValidateShiftPaymentDetails $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $shiftPayment = null;
        $paymentRecords = [];
        $weekDates = null;

        // Get User
        $user = auth()->user();
        //---------

        $startDate = Carbon::parse($request['start_date']);
        $endDate = Carbon::parse($request['end_date']);

        DB::beginTransaction();
        try {

            $transactions = $this->transaction->api_hcp_tranactions_details_list($request, $user, $weekDates);
            if(count($transactions)) {
                if(!empty($transactions[0]) && !empty($transactions[0]['invoice_generated_date_range'])) {
                    $weekDates = explode(' - ', $transactions[0]['invoice_generated_date_range']);
                    $startDate =  Carbon::parse($weekDates[0]);
                    $endDate = Carbon::parse($weekDates[1]);
                }
                $paymentRecords = $transactions;
            } else {
                $paymentRecords = [];
            }

            DB::commit();
        } catch (\Exception $e) {
            $paymentRecords = [];
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (count($paymentRecords) > 0) {

            $response = [
                'status' => true,
                'message' => __('messages.records_found', ['records' => 'Payment']),
                'data' => [
                    'date_range' => $startDate->format('d M Y').' - '.$endDate->format('d M Y'),
                    'total_payable' => round($paymentRecords->sum('hcp_amount'), 2),
                    'payments' => $paymentRecords,
                ],
            ];

        } else if(count($paymentRecords) <= 0 && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => __('messages.records_not_found', ['records' => 'Payment']),
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

    /**
     * End shifts.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function endShift(ValidateStartShift $request)
    {
        // Get User
        $user = auth()->user();
        //---------

        $responseCode = 200;
        $errorMessage = null;
        $shiftUpdate = null;
        $shiftActivity = null;

        DB::beginTransaction();
        try {

            // Get Shift
            $shiftWhere = ['id' => $request['shift_id'], 'hcp_id' => $user->hcp->id];
            $shift = $this->shift->api_shift_where($shiftWhere);
            //----------

            $currentTime = Carbon::now($shift->state->time_zone_name);

            if(!empty($shift)) {

                if($shift->shift_status == '2') {

                    // Set time zone
                    date_default_timezone_set($shift->state->time_zone_name);
                    //--------------

                    $currentTime = Carbon::now($shift->state->time_zone_name);

                    // Update Shift
                    $shiftData = ['shift_status' => '4', 'hcp_shift_end_time' => $currentTime];
                    $shiftUpdate = $this->shift->update($shiftData, ['id' => $request['shift_id']]);
                    //-------------

                    // Update buddy shift request
                    if(!empty($shift) &&  $shift->shift_type == 'buddy'){
                        $this->buddy_shift_request->update(['status' => 'completed'], ['shift_id' => $shift->id]);
                    }
                    //---------------------------

                    // Shift Activity
                    $getShiftActivity = $this->shift_activity->find_shift_activity(['shift_id' => $request['shift_id'], 'activity' => 'shift-end']);

                    if(empty($getShiftActivity)) {
                        $shiftActivityData = [
                            'hcp_id'        => $user->hcp->id,
                            'shift_id'      => $request['shift_id'],
                            'activity'      => 'shift-end',
                            'created_by'    => $user->id,
                            'updated_by'    => $user->id,
                        ];
                        $shiftActivity = $this->shift_activity->store($shiftActivityData);
                    }
                    //---------------

                    if (!empty($shift['hcp_id'])) {
                        $hcpId = $shift['hcp_id'];
                        send_shift_push_notification($shift->id, 'shift-completed', $hcpId);
                    }

                    // Send Admin Notification
                    send_admin_notification('admin-hcp-shift-completed', 'admin', $shift->id);
                    //------------------------

                    $message = __('messages.shift_end_successfully');

                } else {
                    $shiftUpdate = null;
                    $message = __('messages.shift_already_end');
                }

            } else {
                $shiftUpdate = null;
                $message = __('messages.shift_end_failed');
            }

            DB::commit();
        } catch (\Exception $e) {
            $shiftUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($shiftUpdate)) {

            $response = [
                'status' => true,
                'message' => $message,
                'data' => [],
            ];

        } else if(is_null($shiftUpdate) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }


    /**
     * Cancel Transfer Request.
     *
     * @return \Illuminate\Http\Response|\Illuminate\Contracts\Routing\ResponseFactory
     */
    public function cancelTransferRequest(ValidateRejectTransferShift $request)
    {
        $responseCode = 200;
        $errorMessage = null;
        $shiftTransfer = null;
        $shiftUpdate = null;
        $shiftActivity = null;

        // Get User
        $user = auth()->user();
        //---------

        DB::beginTransaction();
        try {

            // Get Transfer Shift
            $checkshiftTransferWhere = ['shift_id' => $request['shift_id'], 'hcp_id' => $user->hcp->id];
            $checkShiftTransfer = $this->shift_transfer->find_shift_transfer($checkshiftTransferWhere);
            //-------------------

            if(!empty($checkShiftTransfer)) {

                // Shift Activity
                $shiftActivityData = [
                    'hcp_id'        => $user->hcp->id,
                    'shift_id'      => $request['shift_id'],
                    'activity'      => 'shift-transfer-request-cancelled',
                    'created_by'    => $user->id,
                    'updated_by'    => $user->id,
                ];
                $shiftActivity = $this->shift_activity->store($shiftActivityData);
                //---------------

                // Update Shift Transfer
                $shiftTransferData = ['status' => 'cancelled'];
                $shiftTransfer = $this->shift_transfer->update($shiftTransferData, ['shift_id' => $request['shift_id']]);
                //----------------------

                $message = __('messages.shift_transfer_cancel_successfully');

            } else {
                $shiftTransfer = null;
                $message = __('messages.shift_transfer_cancel_hcp_error');
            }

            DB::commit();
        } catch (\Exception $e) {
            $shiftTransfer = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }

        // Set response
        if (!empty($shiftTransfer)) {

            $response = [
                'status' => true,
                'message' => $message,
                'data' => [],
            ];

        } else if(is_null($shiftTransfer) && is_null($errorMessage)) {

            $response = [
                'status' => false,
                'message' => $message,
                'data' => null,
            ];

        } else {

            // Set response code
            $responseCode = 500;
            //------------------

            $response = [
                'status' => false,
                'message' => __('messages.something_went_wrong'),
                'data' => null,
            ];
        }
        //-------------

        return response()->json($response, $responseCode);
    }

}
