<?php

namespace App\Http\Controllers\AdminPanel;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Http\Requests\AdminPanel\ValidateCategory;
use DataTables;
use App\Models\Category;
use App\Models\Profession;
use App\Models\Facility;
use Carbon\Carbon;

class CategoryController extends Controller
{
    /**
     * @var array
     */
    public $viewData = [];

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth.admin');
    }

    /**
     * View categories.
     *
     * @return \Illuminate\View\View|\Illuminate\Contracts\View\Factory
     *
     * @author Sumit
     * @created 1 Aug 2020
     */
    public function index()
    {
        // Get users
        $authUser = auth()->user();
        //----------

        $this->viewData['authUser'] = $authUser;

        return view('admin-panel.categories.index')->with($this->viewData);
    }

    /**
     * Get categories data.
     *
     * @return datatable
     *
     * @author Sumit
     * @created 1 Aug 2020
     */
    public function getCategories(Request $request)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        $categories = Category::orderBy('id', 'DESC')->get();

        return DataTables::of($categories)
            ->addColumn('name', function ($category) {
                $name = '<div>' . ucwords($category->name) . '</div> <small>' . $category->category_prefix . $category->id . '</small>';
                return $name;
            })
            ->addColumn('status', function ($category) {
                $status = '';
                if ($category->status == 'inactive') {
                    $status .= '<label class="badge badge-danger mb0">Inactive</label> &nbsp;';
                } else {
                    $status .= '<label class="badge badge-success mb0">Active</label> &nbsp;';
                }
                return $status;
            })
            ->addColumn('action', function ($category) use ($authUser) {
                $action = '
                    <ul class="datatable-action">
                        <li>
                            <div class="dropdown custom-dropdown-icon">
                                <a class="dropdown-toggle" href="#" role="button" id="dropdownMenuLink-4" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                                    <i class="fa fa-ellipsis-v" aria-hidden="true"></i>
                                </a>
                                <div class="dropdown-menu dropdown-menu-right" aria-labelledby="dropdownMenuLink-4">
                                    <a class="dropdown-item" href="' . route('adminPanel.categories.edit', ['id' => ev($category->id)]) . '"> ' . __("language.edit") . ' </a>
                                </div>
                            </div>
                        </li>
                    </ul>
                ';

                return $action;
            })
            ->rawColumns([
                'name' => 'name',
                'action' => 'action',
                'status' => 'status',
            ])
            ->addIndexColumn()->make(true);
    }

    /**
     * Change status.
     *
     * @return boolean
     *
     * @author Sumit
     * @created 1 Aug 2020
     */
    public function changeStatus(Request $request)
    {
        $category = Category::toggleStatus($request['ids']);

        // Set response
        if (!is_null($category)) {
            $response = [
                '_status' => true,
                '_message' => __('messages.status_changed'),
                '_type' => 'success',
            ];
        } else {
            $response = [
                '_status' => false,
                '_message' => __('messages.status_change_failed'),
                '_type' => 'error',
            ];
        }
        //-------------

        return response()->json($response, 200);
    }

    /**
     * View create categories.
     *
     * @return \Illuminate\View\View|\Illuminate\Contracts\View\Factory
     *
     * @author Sumit
     * @created 1 Aug 2020
     */
    public function create()
    {
        return view('admin-panel.categories.create');
    }

    /**
     * Store categories.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 1 Aug 2020
     */
    public function store(ValidateCategory $request)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        $category = null;
        $errorMessage = null;

        DB::beginTransaction();

        // Create Category
        try {

            // Set data
            $data = [
                'name' => $request['name'],
                'status' => $request['status'],
                'comments' => $request['comments'],
                'created_by' => $authUser->id,
                'updated_by' => $authUser->id,
            ];
            //---------

            $category = Category::create($data);

            DB::commit();
        } catch (\Exception $e) {
            $category = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }
        //----------------

        if (!is_null($category)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.record_created', ['record' => 'Category']),
                '_type' => 'success',
            ];
            //-----------------

            return redirect()->route('adminPanel.hcp-configuration.index', ['#category'])->with(['notification' => $notification]);
        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.record_creation_failed', ['record' => 'Category']),
                '_type' => 'error',
            ];
            //-----------------

            return redirect()->route('adminPanel.categories.create')->withInput()->with(['notification' => $notification]);
        }
    }

    /**
     * Edit categories.
     *
     * @return \Illuminate\View\View|\Illuminate\Contracts\View\Factory
     *
     * @author Sumit
     * @created 4 Aug 2020
     */
    public function edit(Request $request, $id)
    {
        // Get categories
        $category = Category::where('id', dv($id))->first();
        //---------------

        $this->viewData['category'] = $category;

        return view('admin-panel.categories.edit')->with($this->viewData);
    }

    /**
     * Update categories.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 1 Aug 2020
     */
    public function update(ValidateCategory $request, $id)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        $categoryUpdate = false;
        $errorMessage = null;

        // Get categories
        $category = Category::where('id', dv($id))->first();
        //---------------

        // Update category
        DB::beginTransaction();
        try {

            $category->name = $request['name'];
            $category->status = $request['status'];
            $category->comments = $request['comments'];
            $category->updated_by = $authUser->id;

            $categoryUpdate = $category->save();

            DB::commit();
        } catch (\Exception $e) {
            $categoryUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }
        //----------------

        if (!is_null($categoryUpdate)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.records_updated', ['record' => 'Category']),
                '_type' => 'success',
            ];
            //-----------------

            return redirect()->route('adminPanel.hcp-configuration.index', ['#category'])->with(['notification' => $notification]);
        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.records_updation_failed', ['record' => 'Category']),
                '_type' => 'error',
            ];
            //-----------------

            return redirect()->route('adminPanel.categories.edit', ['id' => ev($category->id)])->withInput()->with(['notification' => $notification]);
        }
    }

    /**
     * Get category.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 18 Jan 2021
     */
    public function getCategory(Request $request)
    {
        // Get category
        $categories = Category::active()->get();
        //------------

        // Get Facility
        $facility = Facility::where('id', $request['facility_id'])
        ->with(['state' => function ($query) {
            $query->select('id', 'time_zone_name');
        }])->select('id', 'state_id')->first();
        //-------------

        if(!empty($facility)) {
            $facilityTime = Carbon::now($facility->state->time_zone_name)->format('Y-m-d h:i a');
        } else {
            $facilityTime = null;
        }

        // Set response
        if (count($categories)) {
            $response = [
                '_status' => true,
                '_message' => __('messages.record_found', ['record' => 'Category']),
                '_data' => $categories,
                '_facility_data' => $facility,
                '_facility_current_time' => $facilityTime,
            ];
        } else {
            $response = [
                '_status' => false,
                '_message' => __('messages.record_not_found', ['record' => 'Category']),
                '_data' => [],
                '_facility_data' => [],
            ];
        }
        //-------------

        return response()->json($response, 200);
    }

    /**
     * Get professions.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 20 Aug 2020
     */
    public function getProfessions(Request $request)
    {
        // Get profession
        $professions = Profession::where('category_id', $request['category_id'])->active()->orderBy('name')->get();
        //---------------

        // Set response
        if (count($professions)) {
            $response = [
                '_status' => true,
                '_message' => __('messages.record_found', ['record' => 'Profession']),
                '_data' => $professions,
            ];
        } else {
            $response = [
                '_status' => false,
                '_message' => __('messages.record_not_found', ['record' => 'Profession']),
                '_data' => [],
            ];
        }
        //-------------

        return response()->json($response, 200);
    }
}
