<?php

namespace App\Http\Controllers\AdminPanel;
use App\Http\Controllers\Controller;
use App\Interfaces\SubAdminInterface;
use App\Interfaces\FacilityInterface;
use App\Interfaces\FacilityGroupInterface;
use App\Interfaces\PermissionInterface;
use App\Interfaces\UserInterface;
use App\Http\Requests\AdminPanel\ValidateSubAdmin;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use App\Http\Traits\UploadImage;
use Illuminate\Http\Request;
use App\User;
use Maatwebsite\Excel\Facades\Excel;
use App\Exports\SubAdminExport;
use Spatie\Permission\Models\Permission;

class FacilityGroupSubAdminController extends Controller
{
    use UploadImage;

    private $sub_admin, $facilities;
    public function __construct(SubAdminInterface $sub_admin, FacilityInterface $facilities, UserInterface $user, FacilityGroupInterface $facility_group){
        $this->sub_admin = $sub_admin;
        $this->facilities = $facilities;
        $this->user = $user;
        $this->facility_group = $facility_group;
    }

    /**
     * View Sub Admin.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 25 Sep 2020
    */
    public function index(Request $request)
    {
        // Get User
        $authUser = auth()->user();
        //---------

        try {

            // Get Sub Admin
            $where = [['facility_group_id', '!=', NULL]];
            $subAdmins = $this->sub_admin->getAll($where, true, $request);
            //--------------

            // Request
            $request = $request->all();
            //--------

            return view('admin-panel.facility-group-sub-admin.index', compact('subAdmins', 'request'));

        } catch (\Exception $exception) {
            p($exception->getMessage());
            return dashboard_redirect($exception->getMessage());
        }
    }


    /**
     * View create sub admin.
     *
     * @return \Illuminate\Contracts\Foundation\Application|\Illuminate\Contracts\View\Factory|\Illuminate\Http\RedirectResponse|\Illuminate\Routing\Redirector|\Illuminate\View\View
     *
     * @author Sumit
     * @created 24 Sep 2020
    */
    public function create()
    {
        // Get User
        $authUser = auth()->user();
        //---------

        $where = [];

        try {
            $facilityGroups = $this->facility_group->get_list(['status'=>'active']);
            $where = ['status' => 'active'];
            // Get Permissions
//            $facilities = $this->facilities->getAll($where, true);
            // p($facilities);
            //----------------

            return view('admin-panel.facility-group-sub-admin.create', compact( 'facilityGroups'));
        } catch (\Exception $exception) {
            p($exception->getMessage());
            return dashboard_redirect($exception->getMessage());
        }
    }

    /**
     * Store sub admin.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 28 Sep 2020
    */
    public function store(ValidateSubAdmin $request)
    {
        // Get User
        $authUser = auth()->user();
        //---------

        $subAdmin = null;
        $randomPassword = str_random(8);
        $imageName = null;
        $subAdminData = [];
        $userData = [];

        DB::beginTransaction();
        try {

            $userData = [
                'name' => $request['name'],
                'email' => $request['email'],
                'password' => bcrypt($randomPassword),
                'status' => $request['status'],
                'created_by' => $authUser->id,
                'updated_by' => $authUser->id,
            ];


            $userData['role_name'] = 'facility-group-sub-admin';


            $user = $this->user->store($userData);

            // Assign Permissions
            $permissions = Permission::where('panel_permissions', 'facility-group-sub-admin')->get()->toArray();
            if (!is_null($user) && !is_null($permissions)) {
                foreach($permissions as $permission){
                    $user->givePermissionTo($permission['name']);
                }
            }
            //-------------------

            // Assign role
            $user->assignRole('Facility Group Sub Admin');

            //------------

            $subAdminData = [
                'name' => $request['name'],
                'surname' => $request['surname'],
                'username' => $request['username'],
                'contact_number' => $request['contact_number'],
                'status' => $request['status'],
                'user_id' => $user->id,
                'created_by' => $authUser->id,
                'updated_by' => $authUser->id,
            ];

            $subAdminData['facility_group_id'] = $request->facility_group;
            $subAdminData['assigned_facilities'] = json_encode($request['facility']);

            $subAdmin = $this->sub_admin->store($subAdminData);

            // Upload image
            if ($request->hasFile('image')) {
                $path  = config('constants.sub_admins.sub_admins_path') . '/' . $subAdmin->sub_admin_prefix . $subAdmin->id . '/image/';
                $image = $this->uploadImage($request->file('image'), $path, 75, strtolower($subAdmin->name));
                if ($image['_status']) {
                    $imageName = $image['_data'];
                }

                $subAdmin = $this->sub_admin->update(['image' => $imageName], ['id' => $subAdmin->id]);
            }
            //-------------

            // Send nurse email with password
            if(!empty($subAdmin)){
                $emailTemplate = config('constants.admin_email_path') . 'user-welcome-email';
                $emailData = [
                    'template' => $emailTemplate,
                    'data' => [
                        'name' => ucwords($request['name']),
                        'email' => $request['email'],
                        'subject' => __('subjects.user_welcome', ['app_name' => config('app.name')]),
                        'password' => $randomPassword,
                    ],
                ];


               if(config('app.server') == 'AWS' ) {
                   SendEmailJob::dispatch($emailData)->delay(now()->addSeconds(5));
               } else {
                   $data = $emailData['data'];
                   Mail::send($emailTemplate, $data, function ($message) use ($data) {
                       $message->to($data['email']);
                       $message->subject($data['subject']);
                   });
               }
            }
            //-------------------------------

            DB::commit();
        } catch (\Exception $exception) {
            p($exception->getMessage());
            $subAdmin = null;
            DB::rollback();
            return dashboard_redirect($exception->getMessage());
        }

        if(!is_null($subAdmin)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.record_created', ['record' => 'Sub Admin']),
                '_type' => 'success',
            ];
            //-----------------

            return redirect()->route('adminPanel.facility-group-sub-admins.index')->with(['notification' => $notification]);

        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.record_creation_failed', ['record' => 'Sub Admin']),
                '_type' => 'error',
            ];
            //-----------------

            return redirect()->route('adminPanel.facility-group-sub-admins.create')->withInput()->with(['notification' => $notification]);
        }

    }

    /**
     * View edit sub admin.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 28 Sep 2020
    */
    public function edit(Request $request, $id)
    {
        // Get User
        $authUser = auth()->user();
        //---------

        $where = [];
        try {
            $facilityGroups = $this->facility_group->get_list(['status'=>'active']);


            // Get sub admin
            $subAdmin = $this->sub_admin->find_sub_admin(dv($id));
            //--------------
            $where = ['status' => 'active', 'facility_group_id' =>  $subAdmin->facility_group_id];
            // Get Facilities
            $facilities = $this->facilities->getAll($where, true);
            // Get user
            $user = $this->user->find_user($subAdmin->user_id);
            //---------

            // User Permissions
            $userPermissions = $user->getDirectPermissions();
            //-----------------

            return view('admin-panel.facility-group-sub-admin.edit', compact( 'subAdmin', 'facilityGroups','facilities'));
        } catch (\Exception $exception) {
            p($exception->getMessage());
            return dashboard_redirect($exception->getMessage());
        }
    }

    /**
     * Update sub admin.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 28 Sep 2020
    */
    public function update(ValidateSubAdmin $request, $id)
    {
        // Get User
        $authUser = auth()->user();
        //---------

        $subAdmin = null;
        $subAdminUpdate = null;
        $userUpdate =  null;
        $randomPassword = str_random(8);
        $imageName = null;
        $subAdminData = [];
        $userData = [];

        DB::beginTransaction();
        try {
            // Get sub admin
            $subAdmin = $this->sub_admin->find_sub_admin(dv($id));
            //--------------

            // Get user
            $user = $this->user->find_user($subAdmin->user_id);
            //---------

            $subAdminData = [
                'name' => $request['name'],
                'surname' => $request['surname'],
                'contact_number' => $request['contact_number'],
                'username' => $request['username'],
                'status' => $request['status'],
                'user_id' => $user->id,
                'updated_by' => $authUser->id,
            ];
            $subAdminData['facility_group_id'] = $request['facility_group'];
            $subAdminData['assigned_facilities'] = json_encode($request['facility']);

            $subAdminUpdate = $this->sub_admin->update($subAdminData, ['id' => $subAdmin->id]);

            // Upload image
            if ($request->hasFile('image')) {
                $path  = config('constants.sub_admins.sub_admins_path') . '/' . $subAdmin->sub_admin_prefix . $subAdmin->id . '/image/';
                $image = $this->uploadImage($request->file('image'), $path, 75, strtolower($subAdmin->name));
                if ($image['_status']) {
                    $imageName = $image['_data'];
                }

                $subAdminUpdate = $this->sub_admin->update(['image' => $imageName], ['id' => $subAdmin->id]);
            }
            //-------------

            // Send nurse email with password
            if(($user->email != $request['email']) && empty($request['password'])) {
                $emailTemplate = config('constants.admin_email_path') . 'user-welcome-email';
                $emailData = [
                    'template' => $emailTemplate,
                    'data' => [
                        'name' => ucwords($request['name']),
                        'email' => $request['email'],
                        'subject' => __('subjects.user_welcome', ['app_name' => config('app.name')]),
                        'password' => $randomPassword,
                    ],
                ];


                if(config('app.server') == 'AWS' ) {
                    SendEmailJob::dispatch($emailData)->delay(now()->addSeconds(5));
                } else {
                    $data = $emailData['data'];
                    Mail::send($emailTemplate, $data, function ($message) use ($data) {
                        $message->to($data['email']);
                        $message->subject($data['subject']);
                    });
                }

                $userData = [
                    'password' => bcrypt($randomPassword),
                ];
            }
            //-------------------------------

            $userData = [
                'name' => $request['name'] . ' ' . $request['surname'],
                'status' => $request['status'],
                'email' => $request['email'],
                'updated_by' => $authUser->id,
            ];
            if(!empty($request['password'])){
                $userData['password'] = bcrypt($request['password']);
            }
            $userUpdate = $this->user->update($userData, ['id' => $subAdmin->user_id]);

            // Assign Permissions
//            $user->syncPermissions($request['permission']);
            //-------------------

            DB::commit();
        } catch (\Exception $exception) {
            $subAdmin = null;
            DB::rollback();
            return dashboard_redirect($exception->getMessage());
        }


        if (!is_null($subAdminUpdate)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.records_updated', ['record' => 'Sub Admin']),
                '_type' => 'success',
            ];
            //-----------------

            return redirect()->route('adminPanel.facility-group-sub-admins.index')->with(['notification' => $notification]);

        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.records_updation_failed', ['record' => 'Sub Admin']),
                '_type' => 'error',
            ];
            //-----------------

            return redirect()->route('adminPanel.facility-group-sub-admins.edit', ['id' => ev($subAdmin->id)])->withInput()->with(['notification' => $notification]);
        }


    }

    /**
     * Change status.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 28 Sep 2020
    */
    public function changeStatus(Request $request, $id, $status, $page = null)
    {
        // Get User
        $authUser = auth()->user();
        //---------

        $subAdmin = null;
        $data = [];
        $where = [];

        DB::beginTransaction();
        try {

            $where = [
                'id' => dv($id)
            ];

            $data = [
                'status' => $request['status'],
                'updated_by' => $authUser->id,
            ];

            $subAdmin = $this->sub_admin->update($data, $where);

            DB::commit();
        } catch (\Exception $exception) {
            $subAdmin = null;
            DB::rollback();
            return dashboard_redirect($exception->getMessage());
        }

        if (!is_null($subAdmin)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.status_changed'),
                '_type' => 'success',
            ];
            //-----------------

            if (!empty($page)) {
                return response()->json($notification, 200);
            } else {
                return redirect()->route('adminPanel.sub-admins.details', ['id' => $id])->with(['notification' => $notification]);
            }


        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.status_change_failed'),
                '_type' => 'error',
            ];
            //-----------------


            if (!empty($page)) {
                return response()->json($notification, 200);
            } else {
                return redirect()->route('adminPanel.sub-admins.details', ['id' => $id])->with(['notification' => $notification]);
            }
        }
    }

    /**
     * View details.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 28 Sep 2020
    */
    public function details(Request $request, $id)
    {
        try {

            // Get sub admin
            $subAdmin = $this->sub_admin->find_sub_admin(dv($id));
            //--------------

            // Get user
            $user = $this->user->find_user($subAdmin->user_id);
            //---------

            // User Permissions
            $userPermissions = $user->getDirectPermissions();
            //-----------------

            return view('admin-panel.sub-admin.details', compact('subAdmin', 'userPermissions'));
        } catch (\Exception $exception) {
            return dashboard_redirect($exception->getMessage());
        }
    }


    /**
     * Check user email.
     *
     * @return boolean
     *
     * @author Sumit
     * @created 10 Aug 2020
     */
    public function checkUserEmail(Request $request)
    {
        $status = false;

        if (!is_null($request->email)) {
            $user = User::where('email', $request['email'])->first();

            if (!is_null($user)) {
                if ($request->filled('id') && $user->id == $request['id']) {
                    $status = true;
                } else {
                    $status = false;
                }
            } else {
                $status = true;
            }
        }

        return response()->json($status, 200);
    }

    /**
     * Export.
     *
     * @return Excel
     *
     * @author Sumit
     * @created 16 Feb 2021
     */
    public function export(Request $request)
    {
        // Get users
        $authUser = auth()->user();
        //----------
        try {

            $request = $request->all();
            $request['export'] = true;
            unset($request['request']['per_page']);

            return Excel::download(new SubAdminExport($this->sub_admin, $request), 'sub-admins.xls');

        } catch (\Exception $exception) {
            return dashboard_redirect($exception->getMessage());
        }
    }
}
