<?php

namespace App\Http\Controllers\AdminPanel;
use App\Http\Controllers\Controller;
use App\Interfaces\FacilityNameInterface;
use App\Interfaces\CountryInterface;
use App\Interfaces\StateInterface;
use App\Interfaces\FacilityGroupInterface;
use App\Interfaces\ModuleImportFileInterface;
use App\Http\Requests\AdminPanel\ValidateFacilityName;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use App\Http\Traits\UploadFile;
use Excel;
use App\Imports\FacilityNamesImport;
use App\Exports\FacilitiesNameExport;
use App\Models\FacilityName;

class FacilityNameController extends Controller
{
    use UploadFile;

    private $facilityName, $countries, $states, $facilityGroup, $moduleImportFile;
    public function __construct(FacilityNameInterface $facilityName, CountryInterface $countries, StateInterface $states, FacilityGroupInterface $facilityGroup, ModuleImportFileInterface $moduleImportFile){
        $this->facility_name = $facilityName;
        $this->facility_group = $facilityGroup;
        $this->countries = $countries;
        $this->states = $states;
        $this->module_import_file = $moduleImportFile;
        $this->middleware('auth.admin');
    }

    /**
     * View Facility Name.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 31 Dec 2020
     */
    public function index(Request $request)
    {
        try {

            // Request
            $request = $request->all();
            //--------

            // Get Facility Name
            $where = [];
            $facilityNames = $this->facility_name->getAll($where, true, $request);
            //------------------

            // Get States
            $whereState = ['status' => 'active', 'country_id' => 1];
            $states = $this->states->getAll($whereState);
            //-----------

            // Get facility Groups
            $whereFacilityGroup = ['status' => 'active'];
            $facilityGroups = $this->facility_group->get_list($whereFacilityGroup);
            //--------------------


            return view('admin-panel.facility-names.index', compact('facilityNames', 'request', 'states', 'facilityGroups'));

        } catch (\Exception $exception) {
            return dashboard_redirect($exception->getMessage());
        }
    }


    /**
     * View create facility name.
     *
     * @return \Illuminate\View\View|\Illuminate\Contracts\View\Factory
     *
     * @author Sumit
     * @created 31 Dec 2020
     */
    public function create()
    {
        try {

            // Get States
            $whereState = ['status' => 'active', 'country_id' => 1];
            $states = $this->states->getAll($whereState);
            //-----------

            // Get facility Groups
            $whereFacilityGroup = ['status' => 'active'];
            $facilityGroups = $this->facility_group->get_list($whereFacilityGroup);
            //--------------------

            return view('admin-panel.facility-names.create', compact('facilityGroups', 'states'));
        } catch (\Exception $exception) {
            return dashboard_redirect($exception->getMessage());
        }
    }


    /**
     * Store facility name.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 31 Dec 2020
     */
    public function store(ValidateFacilityName $request)
    {
        // Get User
        $authUser = auth()->user();
        //---------

        $facilityName = null;
        $data = [];

        DB::beginTransaction();
        try {

            $data = [
                'name' => $request['name'],
                'facility_group_id' => $request['facility_group'],
                'address' => $request['address'],
                'city_name' => $request['city_name'],
                'suburb_name' => $request['suburb_name'],
                'post_code' => $request['post_code'],
                'country_id' => 1,
                'state_id' => $request['state_id'],
                'latitude' => $request['latitude'],
                'longitude' => $request['longitude'],
                'status' => $request['status'],
                'created_by' => $authUser->id,
                'updated_by' => $authUser->id,
            ];

            $facilityName = $this->facility_name->store($data);

            DB::commit();
        } catch (\Exception $exception) {
            $facilityName = null;
            DB::rollback();
            return dashboard_redirect($exception->getMessage());
        }

        if (!is_null($facilityName)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.record_created', ['record' => 'Facility Name']),
                '_type' => 'success',
            ];
            //-----------------

            return redirect()->route('adminPanel.facility-names.index')->with(['notification' => $notification]);

        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.record_creation_failed', ['record' => 'Facility Name']),
                '_type' => 'error',
            ];
            //-----------------

            return redirect()->route('adminPanel.facility-names.create')->withInput()->with(['notification' => $notification]);
        }

    }

    /**
     * View edit facility name.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 31 Dec 2020
     */
    public function edit(Request $request, $id)
    {
        try {

            // Get Facility Name
            $facilityName = $this->facility_name->find_facility_name(dv($id));
            //------------------

            // Get States
            $whereState = ['status' => 'active', 'country_id' => 1];
            $states = $this->states->getAll($whereState);
            //-----------

            // Get facility Groups
            $whereFacilityGroup = ['status' => 'active'];
            $facilityGroups = $this->facility_group->get_list($whereFacilityGroup);
            //--------------------

            return view('admin-panel.facility-names.edit', compact('facilityName', 'facilityGroups', 'states'));

        } catch (\Exception $exception) {

            return dashboard_redirect($exception->getMessage());
        }
    }

    /**
     * Update facility name.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 31 Dec 2020
     */
    public function update(ValidateFacilityName $request, $id)
    {
        // Get User
        $authUser = auth()->user();
        //---------

        $facilityName = null;
        $data = [];
        $where = [];

        DB::beginTransaction();
        try {

            $where = [
                'id' => dv($id)
            ];

            $data = [
                'name' => $request['name'],
                'facility_group_id' => $request['facility_group'],
                'address' => $request['address'],
                'city_name' => $request['city_name'],
                'suburb_name' => $request['suburb_name'],
                'post_code' => $request['post_code'],
                'country_id' => 1,
                'state_id' => $request['state_id'],
                'latitude' => $request['latitude'],
                'longitude' => $request['longitude'],
                'status' => $request['status'],
                'updated_by' => $authUser->id,
            ];

            $facilityName = $this->facility_name->update($data, $where);

        DB::commit();
        } catch (\Exception $exception) {
            $facilityName = null;
            DB::rollback();
            return dashboard_redirect($exception->getMessage());
        }

        if (!is_null($facilityName)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.records_updated', ['record' => 'Facility Name']),
                '_type' => 'success',
            ];
            //-----------------

            return redirect()->route('adminPanel.facility-names.index')->with(['notification' => $notification]);

        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.records_updation_failed', ['record' => 'Facility Name']),
                '_type' => 'error',
            ];
            //-----------------

            return redirect()->route('adminPanel.facility-names.edit', ['id' => $id])->withInput()->with(['notification' => $notification]);
        }
    }


    /**
     * Change Status.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 31 Dec 2020
     */
    public function changeStatus(Request $request, $id, $status)
    {
        // Get User
        $authUser = auth()->user();
        //---------

        $facilityName = null;
        $data = [];
        $where = [];

        DB::beginTransaction();
        try {

            $where = [
                'id' => dv($id)
            ];

            $data = [
                'status' => $status,
                'updated_by' => $authUser->id,
            ];

            $facilityName = $this->facility_name->update($data, $where);

            DB::commit();
        } catch (\Exception $exception) {
            $facilityName = null;
            DB::rollback();
            return dashboard_redirect($exception->getMessage());
        }

        if (!is_null($facilityName)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.status_changed'),
                '_type' => 'success',
            ];
            //-----------------

            return response()->json($notification, 200);

        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.status_change_failed', ['record' => 'Facility Name']),
                '_type' => 'error',
            ];
            //-----------------

            return response()->json($notification, 200);
        }
    }

    /**
     * View Import Excel Modal.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 31 Dec 2020
     */
    public function importExcelModal(Request $request)
    {
        try {

            return view('admin-panel.includes.facility-names.import-excel');

        } catch (\Exception $exception) {
            return dashboard_redirect($exception->getMessage());
        }
    }

    /**
     * View Import Excel.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 31 Dec 2020
     */
    public function importExcel(Request $request)
    {
        // Get User
        $authUser = auth()->user();
        //---------

        $importModuleFile = null;
        $fileName = null;

        DB::beginTransaction();
        try {

            // Upload file
            if (!empty($request['file']) && file_exists($request['file'])) {
                $path = config('constants.module_import_file.file_path');

                $file = $this->uploadFile($request['file'], $path, null, 'facility-names');

                if ($file['_status']) {
                    $fileName = $file['_data'];
                }
            }
            //------------

            // Delete all records
            if(!empty($request['import_action']) && $request['import_action'] == 2) {
                FacilityName::truncate();
            }
            //-------------------

            $import = new FacilityNamesImport;
            Excel::import($import, $request->file('file'));

            if ($import->getRowCount() > 0) {
                $status = 'uploaded';
            } else {
                $status = 'failed';
            }

            if (!empty($request['file']) && file_exists($request['file'])) {

                $data = [
                    'file_name' => $fileName,
                    'module_name' => 'facility-names',
                    'status' => $status,
                    'created_by' => $authUser->id,
                    'updated_by' => $authUser->id,
                ];

                $importModuleFile = $this->module_import_file->store($data);
            }



            DB::commit();
        } catch (\Exception $exception) {
            $importModuleFile = null;
            DB::rollback();

            return dashboard_redirect($exception->getMessage());
        }

        if (!is_null($importModuleFile)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.file_uploaded'),
                '_type' => 'success',
            ];
            //-----------------

            return redirect()->route('adminPanel.facility-names.index')->with(['notification' => $notification]);

        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.file_uploading_failed'),
                '_type' => 'error',
            ];
            //-----------------

            return redirect()->route('adminPanel.facility-names.index')->withInput()->with(['notification' => $notification]);
        }
    }

    /**
     * Get Facility Names.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 04 Jan 2021
     */
    public function getFacilityNames(Request $request)
    {

        // Get facility name
        $facilityNames = $this->facility_name->get_list(['status' => 'active', 'facility_group_id' => $request['facility_group_id']]);
        //------------------

        // Set response
        if (count($facilityNames)) {
            $response = [
                '_status' => true,
                '_message' => __('messages.record_found', ['record' => 'Facility Names']),
                '_data' => $facilityNames,
            ];
        } else {
            $response = [
                '_status' => false,
                '_message' => __('messages.record_not_found', ['record' => 'Facility Names']),
                '_data' => [],
            ];
        }
        //-------------

        return response()->json($response, 200);
    }


    /**
     * Get Facility Name Details.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 04 Jan 2021
     */
    public function getFacilityNameDetails(Request $request)
    {

        // Get facility name
        $facilityName = $this->facility_name->find_facility_name($request['facility_name_id']);
        //------------------

        // Set response
        if (!empty($facilityName)) {
            $response = [
                '_status' => true,
                '_message' => __('messages.record_found', ['record' => 'Facility Names']),
                '_data' => $facilityName,
            ];
        } else {
            $response = [
                '_status' => false,
                '_message' => __('messages.record_not_found', ['record' => 'Facility Names']),
                '_data' => [],
            ];
        }
        //-------------

        return response()->json($response, 200);
    }

    /**
     * Export Facility.
     *
     * @return Excel
     *
     * @author Sumit
     * @created 06 Jan 2021
     */
    public function export(Request $request)
    {

        // Get users
        $authUser = auth()->user();
        //----------

        try {

            $where = [];
            $request = $request->all();
            $request['export'] = true;
            unset($request['request']['per_page']);

            return Excel::download(new FacilitiesNameExport($this->facility_name,$where,$request), 'facility-name.xls');

        } catch (\Exception $exception) {
            return dashboard_redirect($exception->getMessage());
        }
    }
}
