<?php

namespace App\Http\Controllers\AdminPanel;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Http\Requests\AdminPanel\ValidateLevel;
use Yajra\DataTables\Facades\DataTables;
use App\Models\Profession;
use App\Models\Category;
use App\Models\Level;

class LevelController extends Controller
{
    /**
     * @var array
     */
    public $viewData = [];

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth.admin');
    }

    /**
     * View levels.
     *
     * @return \Illuminate\View\View|\Illuminate\Contracts\View\Factory
     *
     * @author Sumit
     * @created 5 Aug 2020
     */
    public function index()
    {
        // Get users
        $authUser = auth()->user();
        //----------

        $this->viewData['authUser'] = $authUser;

        return view('admin-panel.levels.index')->with($this->viewData);
    }

    /**
     * Get levels data.
     *
     * @return datatable
     *
     * @author Sumit
     * @created 5 Aug 2020
     */
    public function getLevels(Request $request)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        $levels = Level::orderBy('id', 'DESC')->get();

        return DataTables::of($levels)
            ->addColumn('name', function ($level) {
                $name = '<div>' . ucwords($level->name) . '</div> <small>' . $level->level_prefix . $level->id . '</small>';
                return $name;
            })
            ->addColumn('category', function ($level) {
                if (!empty($level->category)) {
                    $category = '<div>' . ucwords($level->category->name) . '</div> <small>' . $level->category->category_prefix . $level->category->id . '</small>';
                    return $category;
                } else {
                    return 'N/A';
                }
            })
            ->addColumn('profession', function ($level) {
                if (!empty($level->category)) {
                    $category = '<div>' . ucwords($level->profession->name) . '</div> <small>' . $level->profession->profession_prefix . $level->profession->id . '</small>';
                    return $category;
                } else {
                    return 'N/A';
                }
            })
            ->addColumn('hcp_rate', function ($level) {
                if (!empty($level->hcp_per_rate)) {
                    return '$' . $level->hcp_per_rate;
                } else {
                    return 'N/A';
                }
            })
            ->addColumn('facility_rate', function ($level) {
                if (!empty($level->facility_per_rate)) {
                    return '$' . $level->facility_per_rate;
                } else {
                    return 'N/A';
                }
            })
            ->addColumn('admin_fees', function ($level) {
                if (!empty($level->admin_fees)) {
                    return '$' . $level->admin_fees;
                } else {
                    return 'N/A';
                }
            })
            ->addColumn('status', function ($level) {
                $status = '';
                if ($level->status == 'inactive') {
                    $status .= '<label class="badge badge-danger mb0">Inactive</label> &nbsp;';
                } else {
                    $status .= '<label class="badge badge-success mb0">Active</label> &nbsp;';
                }
                return $status;
            })
            ->addColumn('action', function ($level) use ($authUser) {
                $action = '
                    <ul class="datatable-action">
                        <li>
                            <div class="dropdown custom-dropdown-icon">
                                <a class="dropdown-toggle" href="#" role="button" id="dropdownMenuLink-4" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                                    <i class="fa fa-ellipsis-v" aria-hidden="true"></i>
                                </a>
                                <div class="dropdown-menu dropdown-menu-right" aria-labelledby="dropdownMenuLink-4">
                                    <a class="dropdown-item" href="' . route('adminPanel.levels.edit', ['id' => ev($level->id)]) . '"> ' . __("language.edit") . ' </a>
                                </div>
                            </div>
                        </li>
                    </ul>
                ';

                return $action;
            })
            ->rawColumns([
                'name' => 'name',
                'category' => 'category',
                'profession' => 'profession',
                'hcp_rate' => 'hcp_rate',
                'facility_rate' => 'facility_rate',
                'admin_fees' => 'admin_fees',
                'action' => 'action',
                'status' => 'status',
            ])
            ->addIndexColumn()->make(true);
    }

    /**
     * Change status.
     *
     * @return boolean
     *
     * @author Sumit
     * @created 5 Aug 2020
     */
    public function changeStatus(Request $request)
    {
        $level = Level::toggleStatus($request['ids']);

        // Set response
        if (!is_null($level)) {
            $response = [
                '_status' => true,
                '_message' => __('messages.status_changed'),
                '_type' => 'success',
            ];
        } else {
            $response = [
                '_status' => false,
                '_message' => __('messages.status_change_failed'),
                '_type' => 'error',
            ];
        }
        //-------------

        return response()->json($response, 200);
    }

    /**
     * View create levels.
     *
     * @return \Illuminate\View\View|\Illuminate\Contracts\View\Factory
     *
     * @author Sumit
     * @created 5 Aug 2020
     */
    public function create()
    {
        // Get categories
        $categories = Category::active()->get();
        //---------------

        // General Configuration
        $generalConfiguration = get_general_configuration();
        //----------------------

        $this->viewData['categories'] = $categories;
        $this->viewData['generalConfiguration'] = $generalConfiguration;

        return view('admin-panel.levels.create')->with($this->viewData);
    }

    /**
     * Store levels.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 5 Aug 2020
     */
    public function store(ValidateLevel $request)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        $level = null;
        $errorMessage = null;

        DB::beginTransaction();

        // Create level
        try {

            // Set data
            $data = [
                'name' => $request['name'],
                'category_id' => $request['category'],
                'profession_id' => $request['profession'],
                'hcp_per_rate' => $request['hcp_rate'],
                'facility_per_rate' => $request['facility_rate'],
                'admin_fees' => $request['admin_fees'],
                'status' => $request['status'],
                'comments' => $request['comments'],
                'created_by' => $authUser->id,
                'updated_by' => $authUser->id,
            ];
            //---------

            $level = Level::create($data);

            DB::commit();
        } catch (\Exception $e) {
            $level = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }
        //-------------

        if (!is_null($level)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.record_created', ['record' => 'Level']),
                '_type' => 'success',
            ];
            //-----------------

            return redirect()->route('adminPanel.hcp-configuration.index', ['#level'])->with(['notification' => $notification]);
        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.record_creation_failed', ['record' => 'Level']),
                '_type' => 'error',
            ];
            //-----------------

            return redirect()->route('adminPanel.levels.create')->withInput()->with(['notification' => $notification]);
        }
    }

    /**
     * Edit levels.
     *
     * @return \Illuminate\View\View|\Illuminate\Contracts\View\Factory
     *
     * @author Sumit
     * @created 5 Aug 2020
     */
    public function edit(Request $request, $id)
    {
        // Get levels
        $level = Level::where('id', dv($id))->first();
        //-----------

        // Get categories
        $categories = Category::active()->get();
        //---------------

        // Get professions
        $professions = Profession::where('category_id', $level['category_id'])->active()->get();
        //----------------

        // General Configuration
        $generalConfiguration = get_general_configuration();
        //----------------------

        $this->viewData['categories'] = $categories;
        $this->viewData['professions'] = $professions;
        $this->viewData['level'] = $level;
        $this->viewData['generalConfiguration'] = $generalConfiguration;

        return view('admin-panel.levels.edit')->with($this->viewData);
    }

    /**
     * Update professions.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 1 Aug 2020
     */
    public function update(ValidateLevel $request, $id)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        $levelUpdate = false;
        $errorMessage = null;

        // Get level
        $level = Level::where('id', dv($id))->first();
        //----------

        // Update level
        DB::beginTransaction();
        try {

            $level->name = $request['name'];
            $level->category_id = $request['category'];
            $level->profession_id = $request['profession'];
            $level->hcp_per_rate = $request->hcp_rate;
            $level->facility_per_rate = $request->facility_rate;
            $level->admin_fees = $request->admin_fees;
            $level->comments = $request->comments;
            $level->status = $request->status;
            $level->updated_by = $authUser->id;
            $levelUpdate = $level->save();

            DB::commit();
        } catch (\Exception $e) {
            $levelUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }
        //-------------

        if (!is_null($levelUpdate)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.records_updated', ['record' => 'Level']),
                '_type' => 'success',
            ];
            //-----------------

            return redirect()->route('adminPanel.hcp-configuration.index', ['#level'])->with(['notification' => $notification]);
        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.records_updation_failed', ['record' => 'Level']),
                '_type' => 'error',
            ];
            //-----------------

            return redirect()->route('adminPanel.levels.edit', ['id' => ev($level->id)])->withInput()->with(['notification' => $notification]);
        }
    }

    /**
     * Get professions.
     *
     * @return \Illuminate\View\View|\Illuminate\Contracts\View\Factory
     *
     * @author Sumit
     * @created 28 May 2020
     */
    public function getProfessions(Request $request)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        // Get Professions
        $professions = Profession::where('category_id', $request['category_id'])->where('has_levels', 'yes')->active()->get();
        //----------------

        // Set response
        if (count($professions)) {
            $response = [
                '_status' => true,
                '_message' => __('messages.record_found', ['record' => 'Professions']),
                '_data' => $professions,
            ];
        } else {
            $response = [
                '_status' => false,
                '_message' => __('messages.record_not_found', ['record' => 'Professions']),
                '_data' => [],
            ];
        }
        //-------------

        return response()->json($response, 200);
    }
}
