<?php

namespace App\Http\Controllers\AdminPanel;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Http\Requests\AdminPanel\ValidateProfession;
use Yajra\DataTables\Facades\DataTables;
use App\Interfaces\DocumentConfigInterface;
use App\Models\Profession;
use App\Models\Category;
use App\Models\Level;

class ProfessionController extends Controller
{
    /**
     * @var array
     */
    public $viewData = [];

    private $documentConfiguration;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(DocumentConfigInterface $documentConfiguration)
    {
        $this->middleware('auth.admin');
        $this->documentConfiguration = $documentConfiguration;
    }

    /**
     * View professions.
     *
     * @return \Illuminate\View\View|\Illuminate\Contracts\View\Factory
     *
     * @author Sumit
     * @created 1 Aug 2020
     */
    public function index()
    {
        // Get users
        $authUser = auth()->user();
        //----------

        $this->viewData['authUser'] = $authUser;

        return view('admin-panel.professions.index')->with($this->viewData);
    }

    /**
     * Get professions data.
     *
     * @return datatable
     *
     * @author Sumit
     * @created 1 Aug 2020
     */
    public function getProfessions(Request $request)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        $professions = Profession::orderBy('id', 'DESC')->get();

        return DataTables::of($professions)
            ->addColumn('name', function ($profession) {
                $name = '<div>' . ucwords($profession->name) . '</div> <small>' . $profession->profession_prefix . $profession->id . '</small>';
                return $name;
            })
            ->addColumn('category', function ($profession) {
                if (!empty($profession->category)) {
                    $category = '<div>' . ucwords($profession->category->name) . '</div> <small>' . $profession->category->category_prefix . $profession->category->id . '</small>';
                    return $category;
                } else {
                    return 'N/A';
                }
            })
            ->addColumn('has_levels', function ($profession) {
                return ucwords($profession->has_levels);
            })
            ->addColumn('hcp_rate', function ($profession) {
                if (!empty($profession->hcp_per_rate)) {
                    return '$' . $profession->hcp_per_rate;
                } else {
                    return 'N/A';
                }
            })
            ->addColumn('facility_rate', function ($profession) {
                if (!empty($profession->facility_per_rate)) {
                    return '$' . $profession->facility_per_rate;
                } else {
                    return 'N/A';
                }
            })
            ->addColumn('admin_fees', function ($profession) {
                if (!empty($profession->admin_fees)) {
                    return '$' . $profession->admin_fees;
                } else {
                    return 'N/A';
                }
            })
            ->addColumn('status', function ($profession) {
                $status = '';
                if ($profession->status == 'inactive') {
                    $status .= '<label class="badge badge-danger mb0">Inactive</label> &nbsp;';
                } else {
                    $status .= '<label class="badge badge-success mb0">Active</label> &nbsp;';
                }
                return $status;
            })
            ->addColumn('action', function ($profession) use ($authUser) {
                $action = '
                    <ul class="datatable-action">
                        <li>
                            <div class="dropdown custom-dropdown-icon">
                                <a class="dropdown-toggle" href="#" role="button" id="dropdownMenuLink-4" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                                    <i class="fa fa-ellipsis-v" aria-hidden="true"></i>
                                </a>
                                <div class="dropdown-menu dropdown-menu-right" aria-labelledby="dropdownMenuLink-4">
                                    <a class="dropdown-item" href="' . route('adminPanel.professions.edit', ['id' => ev($profession->id)]) . '"> ' . __("language.edit") . ' </a>
                                </div>
                            </div>
                        </li>
                    </ul>
                ';

                return $action;
            })
            ->rawColumns([
                'name' => 'name',
                'category' => 'category',
                'has_levels' => 'has_levels',
                'hcp_rate' => 'hcp_rate',
                'facility_rate' => 'facility_rate',
                'admin_fees' => 'admin_fees',
                'action' => 'action',
                'status' => 'status',
            ])
            ->addIndexColumn()->make(true);
    }

    /**
     * Change status.
     *
     * @return boolean
     *
     * @author Sumit
     * @created 1 Aug 2020
     */
    public function changeStatus(Request $request)
    {
        $profession = Profession::toggleStatus($request['ids']);

        // Set response
        if (!is_null($profession)) {
            $response = [
                '_status' => true,
                '_message' => __('messages.status_changed'),
                '_type' => 'success',
            ];
        } else {
            $response = [
                '_status' => false,
                '_message' => __('messages.status_change_failed'),
                '_type' => 'error',
            ];
        }
        //-------------

        return response()->json($response, 200);
    }

    /**
     * View create professions.
     *
     * @return \Illuminate\View\View|\Illuminate\Contracts\View\Factory
     *
     * @author Sumit
     * @created 1 Aug 2020
     */
    public function create()
    {
        // Get categories
        $categories = Category::active()->get();
        //---------------

        // General Configuration
        $generalConfiguration = get_general_configuration();
        //----------------------

        $this->viewData['categories'] = $categories;
        $this->viewData['generalConfiguration'] = $generalConfiguration;

        return view('admin-panel.professions.create')->with($this->viewData);
    }

    /**
     * Store professions.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 1 Aug 2020
     */
    public function store(ValidateProfession $request)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        $profession = null;
        $errorMessage = null;

        DB::beginTransaction();

        // Create Category
        try {

            // Set data
            $data = [
                'name' => $request['name'],
                'category_id' => $request['category'],
                'has_levels' => $request['has_levels'],
                'status' => $request['status'],
                'comments' => $request['comments'],
                'hcp_per_rate' => ($request->has_levels == 'no') ? $request->hcp_rate : null,
                'facility_per_rate' => ($request->has_levels == 'no') ? $request->facility_rate : null,
                'admin_fees' => ($request->has_levels == 'no') ? $request->admin_fees : null,
                'created_by' => $authUser->id,
                'updated_by' => $authUser->id,
            ];
            //---------

            $profession = Profession::create($data);

            DB::commit();
        } catch (\Exception $e) {
            $profession = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }
        //----------------

        if (!is_null($profession)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.record_created', ['record' => 'Profession']),
                '_type' => 'success',
            ];
            //-----------------

            return redirect()->route('adminPanel.hcp-configuration.index', ['#profession'])->with(['notification' => $notification]);
        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.record_creation_failed', ['record' => 'Profession']),
                '_type' => 'error',
            ];
            //-----------------

            return redirect()->route('adminPanel.professions.create')->withInput()->with(['notification' => $notification]);
        }
    }

    /**
     * Edit categories.
     *
     * @return \Illuminate\View\View|\Illuminate\Contracts\View\Factory
     *
     * @author Sumit
     * @created 4 Aug 2020
     */
    public function edit(Request $request, $id)
    {
        // Get professions
        $profession = Profession::where('id', dv($id))->first();
        //----------------

        // Get categories
        $categories = Category::active()->get();
        //---------------

        // General Configuration
        $generalConfiguration = get_general_configuration();
        //----------------------

        $this->viewData['categories'] = $categories;
        $this->viewData['profession'] = $profession;
        $this->viewData['generalConfiguration'] = $generalConfiguration;

        return view('admin-panel.professions.edit')->with($this->viewData);
    }

    /**
     * Update professions.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 1 Aug 2020
     */
    public function update(ValidateProfession $request, $id)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        $professionUpdate = false;
        $errorMessage = null;

        // Get professions
        $profession = Profession::where('id', dv($id))->first();
        //----------------

        // Update band
        DB::beginTransaction();
        try {

            $profession->name = $request['name'];
            $profession->category_id = $request['category'];
            $profession->has_levels = $request['has_levels'];
            $profession->comments = $request['comments'];
            $profession->status = $request['status'];
            $profession->hcp_per_rate = ($request->has_levels == 'no') ? $request->hcp_rate : null;
            $profession->facility_per_rate = ($request->has_levels == 'no') ? $request->facility_rate : null;
            $profession->admin_fees = ($request->has_levels == 'no') ? $request->admin_fees : null;
            $profession->updated_by = $authUser->id;
            $professionUpdate = $profession->save();

            DB::commit();
        } catch (\Exception $e) {
            $professionUpdate = null;
            $errorMessage = $e->getMessage();
            DB::rollback();
        }
        //------------

        if (!is_null($professionUpdate)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.records_updated', ['record' => 'Profession']),
                '_type' => 'success',
            ];
            //-----------------

            return redirect()->route('adminPanel.hcp-configuration.index', ['#profession'])->with(['notification' => $notification]);
        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.records_updation_failed', ['record' => 'Profession']),
                '_type' => 'error',
            ];
            //-----------------

            return redirect()->route('adminPanel.professions.edit', ['id' => ev($profession->id)])->withInput()->with(['notification' => $notification]);
        }
    }


    /**
     * Get levels.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 20 Aug 2020
     */
    public function getLevels(Request $request)
    {
        // Get levels
        // $levels = Level::where('profession_id', $request['profession_id'])->active()->get();
        $levels = Level::where('profession_id', $request['profession_id'])->where('status','active')->get();
        //-----------

        // Set response
        if (count($levels)) {
            $response = [
                '_status' => true,
                '_message' => __('messages.record_found', ['record' => 'Level']),
                '_data' => $levels,
            ];
        } else {
            $response = [
                '_status' => false,
                '_message' => __('messages.record_not_found', ['record' => 'Level']),
                '_data' => [],
            ];
        }
        //-------------

        return response()->json($response, 200);
    }

    /**
     * Get Professions Document.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 20 Aug 2020
     */
    public function getProfessionsDocument(Request $request)
    {
        $content = null;
        // p($request->all());

        // Get Document With Profession
        if(!empty($request['profession_id'])){
            $documentConfigurations =  $this->documentConfiguration->get_document_with_profession($request['profession_id']);

            if(count($documentConfigurations)) {
                $content = view('admin-panel.includes.hcps.profession-document')->with([
                    'documentConfigurations' => $documentConfigurations,
                    'counter' => $request['counter'],
                    'professionsCount' => $request['professions_count'],
                    'levelId' => $request['level_id'],
                    'documentCount' => $request['document_count'],
                ])->render();
            }

        } else {
            $documentConfigurations = [];
        }
        //-----------------------------

        // p($documentConfigurations);

        // Set response
        if (count($documentConfigurations)) {
            $response = [
                '_status' => true,
                '_message' => __('messages.record_found', ['record' => 'Document Configuration']),
                '_data' => [],
                '_professions_count' => $request['professions_count'],
                '_document_count' => count($documentConfigurations),
                '_content' => $content,
            ];
        } else {
            $response = [
                '_status' => false,
                '_message' => __('messages.record_not_found', ['record' => 'Document Configuration']),
                '_data' => [],
            ];
        }
        //-------------

        return response()->json($response, 200);
    }
}
