<?php

namespace App\Http\Controllers\AdminPanel;
use App\Http\Controllers\Controller;
use App\Interfaces\PublicHolidayInterface;
use App\Interfaces\CountryInterface;
use App\Interfaces\StateInterface;
use App\Http\Requests\AdminPanel\ValidatePublicHoliday;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use Maatwebsite\Excel\Facades\Excel;
use App\Exports\PublicHolidayExport;

class PublicHolidayController extends Controller
{
    private $publicHoliday, $countries, $states;
    public function __construct(PublicHolidayInterface $publicHoliday, CountryInterface $countries, StateInterface $states){
        $this->public_holiday = $publicHoliday;
        $this->countries = $countries;
        $this->states = $states;
    }

    /**
     * View public holiday.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 24 Sep 2020
     */
    public function index(Request $request)
    {
        try {

            // Get holidays
            $where = [];
            $publicHolidays = $this->public_holiday->getAll($where, true, $request);
            //-------------

            // Request
            $request = $request->all();
            //--------

            // Get States
            $whereState = ['status' => 'active', 'country_id' => 1];
            $states = $this->states->getAll($whereState);
            //-----------

            return view('admin-panel.public-holidays.index', compact('publicHolidays', 'request', 'states'));

        } catch (\Exception $exception) {
            return dashboard_redirect($exception->getMessage());
        }
    }

    /**
     * View create public holiday.
     *
     * @return \Illuminate\View\View|\Illuminate\Contracts\View\Factory
     *
     * @author Sumit
     * @created 24 Sep 2020
     */
    public function create()
    {
        try {

            // Get Countries
            $where = ['status' => 'active'];
            $countries = $this->countries->getAll($where);
            //--------------

            return view('admin-panel.public-holidays.create', compact('countries'));
        } catch (\Exception $exception) {
            return dashboard_redirect($exception->getMessage());
        }
    }

    /**
     * Store public holiday.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 24 Sep 2020
     */
    public function store(ValidatePublicHoliday $request)
    {

        // Get User
        $authUser = auth()->user();
        //---------

        $publicHoliday = null;
        $data = [];

        DB::beginTransaction();
        try {

            $data = [
                'name' => $request['name'],
                'holiday_date' => $request['holiday_date'],
                'status' => $request['status'],
                'created_by' => $authUser->id,
                'updated_by' => $authUser->id,
            ];

            $publicHoliday = $this->public_holiday->store($data);

            // Create Public Holiday State
            $stateData = [];
            if (count($request['state_id'])) {
                foreach ($request['state_id'] as $state) {
                    $stateData[$state] = [
                        'public_holiday_id' => $publicHoliday->id,
                        'country_id' => $request['country_id'],
                        'created_by' => $authUser->id,
                        'updated_by' => $authUser->id,
                        'created_at' => now(),
                        'updated_at' => now(),
                    ];
                }
                $publicHoliday->states()->attach($stateData);
            }
            //----------------------------

            DB::commit();
        } catch (\Exception $exception) {
            $publicHoliday = null;
            DB::rollback();
            return dashboard_redirect($exception->getMessage());
        }

        if (!is_null($publicHoliday)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.record_created', ['record' => 'Public Holiday']),
                '_type' => 'success',
            ];
            //-----------------

            return redirect()->route('adminPanel.public-holidays.index')->with(['notification' => $notification]);

        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.record_creation_failed', ['record' => 'Public Holiday']),
                '_type' => 'error',
            ];
            //-----------------

            return redirect()->route('adminPanel.public-holidays.create')->withInput()->with(['notification' => $notification]);
        }

    }

    /**
     * View edit public holiday.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 24 Sep 2020
     */
    public function edit(Request $request, $id)
    {
        try {

            // Get holidays
            $publicHolidays = $this->public_holiday->find_public_holiday(dv($id));
            //-------------

            // Get Countries
            $where = ['status' => 'active'];
            $countries = $this->countries->getAll($where);
            //--------------

            // Get States
            if(count($publicHolidays['states'])) {
                $whereState = ['status' => 'active', 'country_id' => $publicHolidays['states'][0]['country_id']];
                $states = $this->states->getAll($whereState);
            } else {
                $states = [];
            }
            //-----------


            return view('admin-panel.public-holidays.edit', compact('publicHolidays', 'countries', 'states'));

        } catch (\Exception $exception) {
            return dashboard_redirect($exception->getMessage());
        }
    }

    /**
     * Update public holiday.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 24 Sep 2020
     */
    public function update(ValidatePublicHoliday $request, $id)
    {
        // Get User
        $authUser = auth()->user();
        //---------

        $publicHoliday = null;
        $data = [];
        $where = [];

        DB::beginTransaction();
        try {

            $where = [
                'id' => dv($id)
            ];

            $data = [
                'name' => $request['name'],
                'holiday_date' => $request['holiday_date'],
                'status' => $request['status'],
                'updated_by' => $authUser->id,
            ];

            $publicHoliday = $this->public_holiday->update($data, $where);

            // Update Public Holiday State
            $stateData = [];
            $publicHolidayRecord = $this->public_holiday->find_public_holiday(dv($id));
            if (count($request['state_id'])) {
                foreach ($request['state_id'] as $state) {
                    $stateData[$state] = [
                        'public_holiday_id' => dv($id),
                        'country_id' => $request['country_id'],
                        'updated_by' => $authUser->id,
                        'updated_at' => now(),
                    ];
                }
                $publicHolidayRecord->states()->sync($stateData);
            }
            //----------------------------

        DB::commit();
        } catch (\Exception $exception) {
            $publicHoliday = null;
            DB::rollback();
            return dashboard_redirect($exception->getMessage());
        }

        if (!is_null($publicHoliday)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.records_updated', ['record' => 'Public Holiday']),
                '_type' => 'success',
            ];
            //-----------------

            return redirect()->route('adminPanel.public-holidays.index')->with(['notification' => $notification]);

        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.records_updation_failed', ['record' => 'Public Holiday']),
                '_type' => 'error',
            ];
            //-----------------

            return redirect()->route('adminPanel.public-holidays.edit', ['id' => $id])->withInput()->with(['notification' => $notification]);
        }
    }

    /**
     * Change Status.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 24 Sep 2020
     */
    public function changeStatus(Request $request, $id, $status)
    {
        // Get User
        $authUser = auth()->user();
        //---------

        $publicHoliday = null;
        $data = [];
        $where = [];

        DB::beginTransaction();
        try {

            $where = [
                'id' => dv($id)
            ];

            $data = [
                'status' => $request['status'],
                'updated_by' => $authUser->id,
            ];

            $publicHoliday = $this->public_holiday->update($data, $where);

            DB::commit();
        } catch (\Exception $exception) {
            $publicHoliday = null;
            DB::rollback();
            return dashboard_redirect($exception->getMessage());
        }

        if (!is_null($publicHoliday)) {

            // Set notification
            $notification = [
                '_status' => true,
                '_message' => __('messages.status_changed'),
                '_type' => 'success',
            ];
            //-----------------

            return response()->json($notification, 200);

        } else {

            // Set notification
            $notification = [
                '_status' => false,
                '_message' => __('messages.status_change_failed', ['record' => 'Public Holiday']),
                '_type' => 'error',
            ];
            //-----------------

            return response()->json($notification, 200);
        }
    }

    /**
     * Get Facility public holiday.
     *
     * @return mixed
     *
     * @author Sumit
     * @created 24 Sep 2020
     */
    public function getFacilityPublicHoliday(Request $request)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        // Get facility
        if (!empty($authUser->facility)) {
            $facility = $authUser->facility;
        } else if(!empty($authUser->sub_admin) && !empty($authUser->sub_admin->facility)) {
            $facility = $authUser->sub_admin->facility;
        }
        //-------------

        try {

            // Request
            $request = $request->all();
            //--------

            // Get State
            $state = $this->states->find_state($facility->state_id);
            //-----------

            // Get holidays
            $where = [];
            $publicHolidays = $this->public_holiday->get_public_holiday_by_state_with_pagination($where, $facility->state_id, $request);
            //-------------

            return view('admin-panel.public-holidays.facility-public-holiday', compact('publicHolidays', 'state', 'request'));

        } catch (\Exception $exception) {
            return dashboard_redirect($exception->getMessage());
        }
    }

    /**
     * Facility Export.
     *
     * @return Excel
     *
     * @author Sumit
     * @created 16 Feb 2021
     */
    public function facilityExport(Request $request)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        try {

            // Get facility
            $facility = $authUser->facility;
            //-------------

            $request = $request->all();
            $request['facility_export'] = true;
            unset($request['request']['per_page']);

            return Excel::download(new PublicHolidayExport($this->public_holiday, $facility->state_id, $request), 'public-holidays.xls');

        } catch (\Exception $exception) {
            return dashboard_redirect($exception->getMessage());
        }
    }


    /**
     * Export Facility.
     *
     * @return Excel
     *
     * @author Sumit
     * @created 06 Jan 2021
     */
    public function export(Request $request)
    {
        // Get users
        $authUser = auth()->user();
        //----------

        try {

            $where = [];
            $request = $request->all();
            $request['export'] = true;
            unset($request['request']['per_page']);

            return Excel::download(new PublicHolidayExport($this->public_holiday,$where,$request), 'public-holidays.xls');

        } catch (\Exception $exception) {

            return dashboard_redirect($exception->getMessage());
        }
    }
}
