<?php

namespace App\Models;
use Illuminate\Database\Eloquent\Model;
use App\Http\Traits\Statusable;
use App\Http\Traits\StatusToggleable;
use Kyslik\ColumnSortable\Sortable;

class Facility extends Model
{
    use Statusable, StatusToggleable, Sortable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name',
        'user_id',
        'contact_number',
        'address',
        'city_id',
        'state_id',
        'country_id',
        'facility_name_id',
        'facility_group_id',
        'contact_person',
        'suburb_id',
        'latitude',
        'longitude',
        'post_code',
        'status',
        'city_name',
        'suburb_name',
        'card_number',
        'card_expiry_date',
        'card_cvv',
        'additional_email',
        'created_by',
        'updated_by'
    ];

    protected $appends = ['average_rating'];

    /**
     * Insert prefix.
     */
    public static function boot()
    {
        parent::boot();
        self::creating(function ($model) {
            $model->facility_prefix = '#FAC';
        });
    }

    /**
     * Get User.
     */
    public function user()
    {
        return $this->belongsTo('App\User');
    }

    /**
     * Get Country.
     */
    public function country()
    {
        return $this->belongsTo('App\Models\Country');
    }


    /**
     * Get State.
     */
    public function state()
    {
        return $this->belongsTo('App\Models\State');
    }

    /**
     * Get city.
     */
    public function city()
    {
        return $this->belongsTo('App\Models\City');
    }

    /**
     * Get suburb.
     */
    public function suburb()
    {
        return $this->belongsTo('App\Models\Suburb');
    }

    /**
     * Get Plans.
     */
    public function plans()
    {
        return $this->belongsToMany('App\Models\Plan')->withPivot([
            'id',
            'start_date',
            'expiry_date',
            'status',
            'plan_cycle',
            'plan_details',
            'shift_posting_limit_in_trial_account',
            'days_of_shift_posting_limit_in_trial_account',
            'payment_amount',
            'last_payment_date',
            'created_by',
            'updated_by'
        ])->orderBy('last_payment_date', 'DESC');
    }

    /**
     * Get hcp ratings.
     */
    public function ratings()
    {
        return $this->hasMany('App\Models\RatingReview')->where('status', 'publish');
    }

    /**
     * Get average ratings.
     */
    public function getAverageRatingAttribute()
    {
        if($this->ratings()->average('facility_rating') > 0) {
            return (float) number_format($this->ratings()->average('facility_rating'), 1);
        } else {
            return null;
        }
    }


    /**
     * Get Facility Group.
     */
    public function facility_group()
    {
        return $this->belongsTo('App\Models\FacilityGroup');
    }

    /**
     * Get Shifts.
     */
    public function shifts()
    {
        return $this->hasMany('App\Models\Shift');
    }


    /**
     * Get Facility Name.
     */
    public function facility_name()
    {
        return $this->belongsTo('App\Models\FacilityName');
    }

    /**
     * Get activities.
     */
    public function activities()
    {
        return $this->hasMany('App\Models\FacilityActivity');
    }

    /**
     * Get Facility Plan.
     */
    public function facility_plan()
    {
        return $this->hasMany('App\Models\FacilityPlan');
    }

    /**
     * Get Facility Document.
     */
    public function facility_document()
    {
        return $this->hasMany('App\Models\FacilityDocument');
    }


    /**
     * Get blocklists.
     */
    public function blocklists()
    {
        return $this->hasMany('App\Models\FavouritesAndBlocklist');
    }
}
