<?php

namespace App\Models;
use Illuminate\Database\Eloquent\Model;
use App\Http\Traits\Statusable;
use App\Http\Traits\StatusToggleable;

class Hcp extends Model
{
    use Statusable, StatusToggleable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name',
        'image',
        'first_name',
        'last_name',
        'contact_number',
        'invitation_code',
        'referral_code',
        'date_of_birth',
        'gender',
        'address',
        'user_id',
        'city_id',
        'state_id',
        'country_id',
        'points',
        'post_code',
        'registration_number',
        'app_registration_screen',
        'accept_lower_level_job',
        'show_approved_screen',
        'abn_number',
        'tfn_number',
        'about_me',
        'latitude',
        'longitude',
        'created_by',
        'updated_by',
        'city_name',
        'suburb_name',
    ];

    protected $appends = ['average_rating'];

    /**
     * Insert prefix.
     */
    public static function boot()
    {
        parent::boot();
        self::creating(function ($model) {
            $model->hcp_prefix = '#HCP';
        });
    }

    /**
     * Get User.
     */
    public function user()
    {
        return $this->belongsTo('App\User');
    }

    /**
     * Get Country.
     */
    public function country()
    {
        return $this->belongsTo('App\Models\Country');
    }


    /**
     * Get State.
     */
    public function state()
    {
        return $this->belongsTo('App\Models\State');
    }

    /**
     * Get city.
     */
    public function city()
    {
        return $this->belongsTo('App\Models\City');
    }

    /**
     * Get suburb.
     */
    public function suburb()
    {
        return $this->belongsTo('App\Models\Suburb');
    }

    /**
     * Get professions.
     */
    public function professions()
    {
        return $this->belongsToMany('App\Models\Profession')->withPivot([
            'id',
            'hcp_id',
            'category_id',
            'profession_id',
            'level_id',
        ]);
    }

    /**
     * Get hcp professions.
     */
    public function hcp_professions()
    {
        return $this->hasMany('App\Models\HcpProfession')->with('profession','category','level');
    }

    /**
     * Get documents.
     */
    public function documents()
    {
        return $this->belongsToMany('App\Models\Document');
    }

    /**
     * Get favourites and blocklists.
     */
    public function favourites_and_blocklists()
    {
        return $this->hasMany('App\Models\FavouritesAndBlocklist');
    }

    /**
     * Get hcp ratings.
     */
    public function ratings()
    {
        return $this->hasMany('App\Models\RatingReview')->where('status', 'publish');
    }

    /**
     * Get average ratings.
     */
    public function getAverageRatingAttribute()
    {
        if($this->ratings()->average('hcp_rating') > 0) {
            return (float) number_format($this->ratings()->average('hcp_rating'), 1);
        } else {
            return null;
        }
    }

    /**
     * Hcp Block Time.
     */
    public function hcp_block_times()
    {
        return $this->hasMany('App\Models\HcpBlockTime', 'hcp_id');
    }

    /**
     * Get one hcp professions.
     */
    public function one_hcp_professions()
    {
        return $this->hasOne('App\Models\HcpProfession')->with('profession','category','level');
    }


    /**
     * Get notifications.
     */
    public function notifications()
    {
        return $this->hasMany('App\Models\Notification');
    }

    /**
     * Get hcp documents.
     */
    public function hcp_documents()
    {
        return $this->hasMany('App\Models\DocumentHcp');
    }

    /**
     * Get blocklists.
     */
    public function blocklists()
    {
        return $this->hasMany('App\Models\FavouritesAndBlocklist');
    }

    /**
     * Get favourites.
     */
    public function favourites()
    {
        return $this->hasMany('App\Models\FavouritesAndBlocklist');
    }

    /**
     * Get shifts.
     */
    public function shifts()
    {
        return $this->hasMany('App\Models\Shift');
    }

    /**
     * Get buddy shift request.
     */
    public function buddy_shift_requests()
    {
        return $this->hasMany('App\Models\BuddyShiftRequest')->where('status', 'approved');
    }

    /**
     * Get Current Date Shifts.
     */
    public function current_date_shifts()
    {
        return $this->hasMany('App\Models\Shift');
    }

    /**
     * Get Fortnight Shifts.
     */
    public function fortnight_shifts()
    {
        return $this->hasMany('App\Models\Shift');
    }

    /**
     * Get referral hcp.
     */
    public function referral_transactions()
    {
        return $this->hasMany('App\Models\Transaction', 'hcp_id');
    }

    /**
     * Get hcp general documents.
     */
    public function hcp_general_documents()
    {
        return $this->hasMany('App\Models\DocumentHcp')->where('profession_id', null);
    }

    /**
     * Get hcp profession documents.
     */
    public function hcp_profession_documents()
    {
        return $this->hasMany('App\Models\DocumentHcp')->where('profession_id', '!=', null);
    }

    /**
     * Get one hcp professions.
     */
    public function hcp_shift_transfers()
    {
        return $this->hasMany('App\Models\ShiftTransfer', 'hcp_id');
    }

    /**
     * Get one hcp professions.
     */
    public function hcp_transfer_shift_transfers()
    {
        return $this->hasMany('App\Models\ShiftTransfer', 'transfer_hcp_id');
    }

    /**
     * Get one hcp professions.
     */
    public function shift_hcp_swap()
    {
        return $this->hasMany('App\Models\ShiftSwapRequest', 'hcp_id');
    }

    /**
     * Get one hcp professions.
     */
    public function swap_hcp_shift()
    {
        return $this->hasMany('App\Models\ShiftSwapRequest', 'swap_hcp_id');
    }

    /**
     * Get one hcp professions.
     */
    public function hcp_swap_rejected_request()
    {
        return $this->hasMany('App\Models\ShiftSwapRequest', 'hcp_id');
    }
}
