<?php

namespace App\Repositories;

use App\Interfaces\HcpInterface;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use App\Models\Hcp;
use App\Models\Shift;

class HcpRepository implements HcpInterface
{
    private $hcp;

    public function __construct(Hcp $hcp)
    {
        $this->hcp = $hcp;
    }

    public function find_hcp_without_details($id)
    {
        return $this->hcp->select('id', 'user_id', 'hcp_prefix', 'first_name', 'last_name')->find($id);
    }

    public function find_hcp($user_id)
    {
        return $this->hcp->with('user', 'country', 'state', 'city', 'suburb', 'documents', 'hcp_block_times.user')
            ->with([
                'hcp_professions' => function ($query) {
                    $query->withCount([
                        'buddy_shift_request' => function ($query) {
                            // $query->where('status', 'approved');
                        },
                    ]);
                    $query->orderBy('buddy_shift_request_count', 'DESC');
                },
            ])
            ->withCount('hcp_block_times')->find($user_id);
    }

    public function getAll($where, $options = false, $request = null)
    {
        if ($options) {
            return $this->hcp->where(function ($query) use ($where, $request) {

                if (!empty($request['state_id'])) {
                    $query->where('state_id', $request['state_id']);
                }

                $query->where($where);

            })->with('hcp_professions')->get();
        } else {
            return $this->hcp->with('hcp_professions')->get();
        }
    }

    public function get_hcp_with_pagination($where, $request = null)
    {
        $paginationLimit = null;

        if (isset($request['per_page']) && !empty($request['per_page'])) {
            $paginationLimit = $request['per_page'];
        } else {
            $paginationLimit = config('constants.pagination_limit');
        }

        $hcpQuery = $this->hcp->where(function ($query) use ($where, $request) {

            if (isset($request['availability']) && $request['availability'] != '') {
                $query->where('available_for_job', $request['availability']);
            }

            if (isset($request['states']) && !empty($request['states'])) {
                $query->whereIn('state_id', $request['states']);
            }

            if (isset($request['state_id']) && !empty($request['state_id'])) {
                $query->where('state_id', $request['state_id']);
            }

            if (!empty($request['category']) || !empty($request['profession'])) {
                $query->whereHas('hcp_professions', function ($query) use ($request) {
                    if (!empty($request['category'])) {
                        $query->where('category_id', $request['category']);
                    }

                    if (!empty($request['profession'])) {
                        $query->where('profession_id', $request['profession']);
                    }

                });
            }

            if (!empty($request['search'])) {
                if (is_numeric($request['search'])) {
                    $query->where('id', 'LIKE', "%{$request['search']}%");
                } else {
                    $query->where(DB::raw('concat(first_name," ",last_name)'), 'like', '%' . strtolower($request['search']) . '%');
                }
            }

            $query->where($where);
        })
            ->with([
                'hcp_professions' => function ($query) use ($where, $request) {
                    if (!empty($request['category'])) {
                        $query->where('category_id', $request['category']);
                    }

                    if (!empty($request['profession'])) {
                        $query->where('profession_id', $request['profession']);
                    }
                },
            ]);

        if (isset($request['shortby_filter']) && !empty($request['shortby_filter'])) {
            if ($request['shortby_filter'] == '1') {
                $hcpQuery->orderBy('id', 'ASC');
            } else if ($request['shortby_filter'] == '2') {
                $hcpQuery->orderBy('id', 'DESC');
            } else {
                $hcpQuery->orderBy('id', 'DESC');
            }
        } else {
            $hcpQuery->orderBy('id', 'DESC');
        }

        if (isset($request['export']) && $request['export'] == true) {
            return $hcpQuery->get();
        } else {
            return $hcpQuery->paginate($paginationLimit)->onEachSide(2);
        }
    }

    public function get_hcp_without_marked_by_facility($facilityId, $stateId = null)
    {
        $paginationLimit = config('constants.pagination_limit');

        return $this->hcp
            ->doesntHave('favourites_and_blocklists', 'or', function ($query) use ($facilityId, $stateId) {
                $query->where('facility_id', $facilityId);
                $query->where('marked_by', 'facility');

            })->where(function ($query) use ($stateId) {
                if (!empty($stateId)) {
                    $query->where('state_id', $stateId);
                }
            })->orderBy('first_name', 'ASC')->paginate($paginationLimit);
    }

    public function store($array)
    {
        return $this->hcp->create($array);
    }

    public function update($array, $where)
    {
        return $this->hcp->where($where)->update($array);
    }

    public function delete($user_id)
    {
        return $this->hcp->destroy($user_id);
    }

    public function api_get_hcps($request, $user, $pagination_limit = "")
    {
        if ($pagination_limit == "") {
            $paginationLimit = config('constants.pagination_limit');
        } else {
            $paginationLimit = $pagination_limit;
        }

        // General Configuration
        $generalConfiguration = get_general_configuration();
        $radius = $generalConfiguration['radius_km_for_shift_broadcast'];
        //----------------------

        $hcpQuery = $this->hcp
            ->where(function ($query) use ($request, $user, $generalConfiguration) {
                $query->where('status', 'active');
                $query->where('available_for_job', 1);
                $query->where('points', '>', '0');

                if (!empty($request['name'])) {
                    $query->where('first_name', 'like', '%' . $request['name'] . '%');
                }

                if (!empty($user)) {
                    $query->where('id', '!=', $user->hcp->id);
                }

            });

        if (isset($request['facility_latitude']) && !empty($request['facility_latitude']) && isset($request['facility_longitude']) && !empty($request['facility_longitude'])) {
            $facilityLatitude = $request['facility_latitude'];
            $facilityLongitude = $request['facility_longitude'];
        }

        if (!empty($facilityLatitude) && !empty($facilityLongitude)) {
            $hcpQuery->select('*', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS(' . $facilityLatitude . ')) * COS(RADIANS(latitude)) * COS( RADIANS(' . $facilityLongitude . ' - longitude)) + SIN(RADIANS(' . $facilityLatitude . ')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<=", $radius);
        }

        if (isset($request['check_transfer_case']) && !empty($request['check_transfer_case']) && isset($request['shift_id']) && !empty($request['shift_id'])) {
            $hcpQuery->doesntHave('hcp_shift_transfers', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
            })->with([
                'hcp_shift_transfers' => function ($query) use ($request) {
                    $query->where('shift_id', $request['shift_id']);
                    $query->where('status', '!=', 'cancelled');
                },
            ]);

            $hcpQuery->doesntHave('hcp_transfer_shift_transfers', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
                $query->where('status', '!=', 'pending');
            })->with([
                'hcp_transfer_shift_transfers' => function ($query) use ($request) {
                    $query->where('shift_id', $request['shift_id']);
                    $query->where('status', '!=', 'cancelled');
                    $query->where('status', '!=', 'pending');
                },
            ]);
        }

        if (isset($request['check_swap_case']) && !empty($request['check_swap_case']) && isset($request['shift_id']) && !empty($request['shift_id'])) {
            $hcpQuery->doesntHave('shift_hcp_swap', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
            })->with([
                'shift_hcp_swap' => function ($query) use ($request) {
                    $query->where('shift_id', $request['shift_id']);
                    $query->where('status', '!=', 'cancelled');
                },
            ]);

            $hcpQuery->doesntHave('swap_hcp_shift', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
            })->with([
                'swap_hcp_shift' => function ($query) use ($request) {
                    $query->where('shift_id', $request['shift_id']);
                    $query->where('status', '!=', 'cancelled');
                },
            ]);

            $hcpQuery->doesntHave('hcp_swap_rejected_request', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
            })->with([
                'hcp_swap_rejected_request' => function ($query) use ($request) {
                    $query->where('shift_id', $request['shift_id']);
                },
            ]);
        }

        $hcpQuery->whereHas('buddy_shift_requests', function ($query) use ($request, $generalConfiguration) {
            if (!empty($request['category_id'])) {
                $query->where('category_id', $request['category_id']);
            }

            if (!empty($request['profession_id'])) {
                $query->where('profession_id', $request['profession_id']);
            }
        })
            ->whereHas('one_hcp_professions', function ($query) use ($request, $generalConfiguration) {
                if (!empty($request['category_id'])) {
                    $query->where('category_id', $request['category_id']);
                }

                if (!empty($request['profession_id'])) {
                    $query->where('profession_id', $request['profession_id']);
                }

                if ($generalConfiguration['show_accept_lower_level_job_checkbox'] == '1' && !empty($request['level_id'])) {
                    $query->whereRaw('FIND_IN_SET(?, accept_level_ids)', [$request['level_id']]);
                } else if ($generalConfiguration['show_accept_lower_level_job_checkbox'] == '0' && !empty($request['level_id'])) {
                    $query->where('level_id', $request['level_id']);
                }

                $query->where('status', 'active');
                $query->orderBy("id", 'DESC');

            })
            ->doesntHave('shifts', 'and', function ($query) use ($request) {
                $query->select('id', 'shift_prefix', 'facility_id', 'hcp_id', 'category_id', 'profession_id', 'hcp_level_id', 'start_time', 'end_time', 'shift_status', 'shift_transfer_swap_status');

                // if(!empty($request['category_id'])) {
                //     $query->where('category_id', $request['category_id']);
                // }

                // if(!empty($request['profession_id'])) {
                //     $query->where('profession_id', $request['profession_id']);
                // }

                // if(!empty($request['level_id'])) {
                //     $query->where('hcp_level_id', $request['level_id']);
                // }

                // if (!empty($request['shift_start_time']) && !empty($request['shift_end_time'])) {
                //     $query->where('start_time', '>=', Carbon::parse($request['shift_start_time']))->where('start_time', '<=', Carbon::parse($request['shift_end_time']))->orWhere('end_time', '>=', Carbon::parse($request['shift_start_time']))->where('end_time', '<=', Carbon::parse($request['shift_end_time']));
                // }

                $startTime = $request['shift_start_time'];
                $endTime = $request['shift_end_time'];

                // $query->where(function ($query) use ($startTime, $endTime) {
                //     if (!empty($startTime) && !empty($endTime)) {
                //         $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                //     }
                // });

                $query->where(function ($query) use ($request) {
                    $query->where('shift_status', '!=', '3');
                    $query->where('shift_status', '!=', '6');
                });

                $query->where(function ($query) use ($startTime, $endTime) {
                    if (!empty($startTime) && !empty($endTime)) {
                        $query->orwhere(function ($query) use ($startTime, $endTime) {
                            $query->where('start_time', '<=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($startTime));
                        });
                        $query->orwhere(function ($query) use ($startTime, $endTime) {
                            $query->where('start_time', '>=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($endTime))->where('start_time', '<=', Carbon::parse($endTime));
                        });
                        $query->orwhere(function ($query) use ($startTime, $endTime) {
                            $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)])
                                ->orWhereBetween('end_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                        });
                    }
                });
            })
            ->doesntHave('blocklists', 'and', function ($query) use ($request) {
                if (!empty($request['facility_id'])) {
                    $query->where('facility_id', $request['facility_id']);
                    $query->where('type', 'blocked');
                    $query->where('marked_by', 'facility');
                }
            })
            ->with([
                'user' => function ($query) use ($request) {
                    $query->select('id', 'name', 'email', 'status');
                },

                'country' => function ($query) use ($request) {
                    $query->select('id', 'sortname', 'name', 'status');
                },

                'state' => function ($query) use ($request) {
                    $query->select('id', 'country_id', 'name', 'status');
                },

                'city' => function ($query) use ($request) {
                    $query->select('id', 'country_id', 'state_id', 'name', 'status');
                },

                'suburb' => function ($query) use ($request) {
                    $query->select('id', 'city_id', 'country_id', 'state_id', 'name', 'status');
                },

                'buddy_shift_requests' => function ($query) use ($request) {
                    // $query->select('id', 'city_id', 'country_id', 'state_id', 'name', 'status');
                },

                'blocklists' => function ($query) use ($request) {
                    if (!empty($request['facility_id'])) {
                        $query->where('facility_id', $request['facility_id']);
                        $query->where('type', 'blocked');
                        $query->where('marked_by', 'facility');
                    }
                },

                'one_hcp_professions' => function ($query) use ($request, $generalConfiguration) {
                    if (!empty($request['category_id'])) {
                        $query->where('category_id', $request['category_id']);
                    }

                    if (!empty($request['profession_id'])) {
                        $query->where('profession_id', $request['profession_id']);
                    }

                    if ($generalConfiguration['show_accept_lower_level_job_checkbox'] == '1' && !empty($request['level_id'])) {
                        $query->whereRaw('FIND_IN_SET(?, accept_level_ids)', [$request['level_id']]);
                    } else if ($generalConfiguration['show_accept_lower_level_job_checkbox'] == '0' && !empty($request['level_id'])) {
                        $query->where('level_id', $request['level_id']);
                    }

                    $query->where('status', 'active');
                    $query->orderBy("id", 'DESC');
                },

                'shifts' => function ($query) use ($request) {
                    $query->select('id', 'shift_prefix', 'facility_id', 'hcp_id', 'category_id', 'profession_id', 'hcp_level_id', 'start_time', 'end_time', 'shift_status', 'shift_transfer_swap_status');

                    // if(!empty($request['category_id'])) {
                    //     $query->where('category_id', $request['category_id']);
                    // }

                    // if(!empty($request['profession_id'])) {
                    //     $query->where('profession_id', $request['profession_id']);
                    // }

                    // if(!empty($request['level_id'])) {
                    //     $query->where('hcp_level_id', $request['level_id']);
                    // }

                    $query->where(function ($query) use ($request) {
                        $query->where('shift_status', '!=', '3');
                        $query->where('shift_status', '!=', '6');
                    });

                    $startTime = $request['shift_start_time'];
                    $endTime = $request['shift_end_time'];

                    // $query->where(function ($query) use ($startTime, $endTime) {
                    //     if (!empty($startTime) && !empty($endTime)) {
                    //         $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                    //     }
                    // });

                    $query->where(function ($query) use ($startTime, $endTime) {
                        if (!empty($startTime) && !empty($endTime)) {
                            $query->orwhere(function ($query) use ($startTime, $endTime) {
                                $query->where('start_time', '<=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($startTime));
                            });
                            $query->orwhere(function ($query) use ($startTime, $endTime) {
                                $query->where('start_time', '>=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($endTime))->where('start_time', '<=', Carbon::parse($endTime));
                            });
                            $query->orwhere(function ($query) use ($startTime, $endTime) {
                                $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)])
                                    ->orWhereBetween('end_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                            });
                        }
                    });
                },
            ]);

        return $hcpQuery->orderBy('first_name', 'ASC')->paginate($paginationLimit);
    }

    public function get_hcp_listing_for_shift($request)
    {

        $facilityLatitude = null;
        $facilityLongitude = null;
        $radius = null;
        $hcpQuery1 = [];
        $hcpQuery2 = [];

        // General Configuration
        $generalConfiguration = get_general_configuration();
        $radius = $generalConfiguration['radius_km_for_shift_broadcast'];
        //----------------------

        // Set time zone
        date_default_timezone_set($request['time_zone_name']);
        //--------------

        if (isset($request['facility_latitude']) && !empty($request['facility_latitude']) && isset($request['facility_longitude']) && !empty($request['facility_longitude'])) {
            $facilityLatitude = $request['facility_latitude'];
            $facilityLongitude = $request['facility_longitude'];
        }

        $hcpQuery = $this->hcp->where(function ($query) use ($request, $generalConfiguration) {
            $query->where('status', 'active');
            $query->where('available_for_job', 1);
            $query->where('points', '>', '0');

            if (!empty($request['buddy_hcp_id'])) {
                $query->where('id', '!=', $request['buddy_hcp_id']);
            }

            if (!empty($request['shift_hcp_id'])) {
                $query->where('id', '!=', $request['shift_hcp_id']);
            }

            if (!empty($request['cancelled_hcps'])) {
                $query->whereNotIn('id', $request['cancelled_hcps']);
            }

            if (!empty($request['search'])) {
                if (is_numeric($request['search'])) {
                    $query->where('id', 'LIKE', "%{$request['search']}%");
                } else {
                    $query->where(DB::raw('concat(first_name," ",last_name)'), 'like', '%' . strtolower($request['search']) . '%');
                }
            }
        });

        if (!empty($facilityLatitude) && !empty($facilityLongitude)) {
            $hcpQuery->select('id', 'hcp_prefix', 'image', 'contact_number', 'first_name', 'last_name', 'latitude', 'longitude', 'user_id', 'accept_lower_level_job', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS(' . $facilityLatitude . ')) * COS(RADIANS(latitude)) * COS( RADIANS(' . $facilityLongitude . ' - longitude)) + SIN(RADIANS(' . $facilityLatitude . ')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<=", $radius);
        }

        $hcpQuery->doesntHave('blocklists', 'and', function ($query) use ($request) {
            if (!empty($request['facility_id'])) {
                $query->where('facility_id', $request['facility_id']);
                $query->where('type', 'blocked');
                $query->where('marked_by', 'facility');
            }
        })->with([
            'blocklists' => function ($query) use ($request) {
                if (!empty($request['facility_id'])) {
                    $query->where('facility_id', $request['facility_id']);
                    $query->where('type', 'blocked');
                    $query->where('marked_by', 'facility');
                }
            },
        ]);

        if (isset($request['check_transfer_case']) && !empty($request['check_transfer_case']) && isset($request['shift_id']) && !empty($request['shift_id'])) {
            $hcpQuery->doesntHave('hcp_shift_transfers', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
            })->with([
                'hcp_shift_transfers' => function ($query) use ($request) {
                    $query->where('shift_id', $request['shift_id']);
                    $query->where('status', '!=', 'cancelled');
                },
            ]);

            $hcpQuery->doesntHave('hcp_transfer_shift_transfers', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
                $query->where('status', '!=', 'pending');
            })->with([
                'hcp_transfer_shift_transfers' => function ($query) use ($request) {
                    $query->where('shift_id', $request['shift_id']);
                    $query->where('status', '!=', 'cancelled');
                    $query->where('status', '!=', 'pending');
                },
            ]);
        }

        if (isset($request['check_swap_case']) && !empty($request['check_swap_case']) && isset($request['shift_id']) && !empty($request['shift_id'])) {
            $hcpQuery->doesntHave('shift_hcp_swap', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
            })->with([
                'shift_hcp_swap' => function ($query) use ($request) {
                    $query->where('shift_id', $request['shift_id']);
                    $query->where('status', '!=', 'cancelled');
                },
            ]);

            $hcpQuery->doesntHave('swap_hcp_shift', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
                $query->where('status', '!=', 'pending');
            })->with([
                'swap_hcp_shift' => function ($query) use ($request) {
                    $query->where('shift_id', $request['shift_id']);
                    $query->where('status', '!=', 'cancelled');
                    $query->where('status', '!=', 'pending');
                },
            ]);

            $hcpQuery->doesntHave('hcp_swap_rejected_request', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
            })->with([
                'hcp_swap_rejected_request' => function ($query) use ($request) {
                    $query->where('shift_id', $request['shift_id']);
                },
            ]);
        }

        if (isset($request['only_favourites']) && !empty($request['only_favourites'])) {

            $hcpQuery->whereHas('favourites', function ($query) use ($request) {
                if (!empty($request['facility_id'])) {
                    $query->where('facility_id', $request['facility_id']);
                    $query->where('type', 'favourite');
                    $query->where('marked_by', 'facility');
                }
            })->withCount([
                'favourites' => function ($query) use ($request) {
                    if (!empty($request['facility_id'])) {
                        $query->where('facility_id', $request['facility_id']);
                        $query->where('type', 'favourite');
                        $query->where('marked_by', 'facility');
                    }
                },
            ]);

        } else {

            $hcpQuery->with([
                'favourites' => function ($query) use ($request) {
                    if (!empty($request['facility_id'])) {
                        $query->where('facility_id', $request['facility_id']);
                        $query->where('type', 'favourite');
                        $query->where('marked_by', 'facility');
                    }
                },
            ])->withCount([
                'favourites' => function ($query) use ($request) {
                    if (!empty($request['facility_id'])) {
                        $query->where('facility_id', $request['facility_id']);
                        $query->where('type', 'favourite');
                        $query->where('marked_by', 'facility');
                    }
                },
            ]);
        }

        $hcpQuery->whereHas('one_hcp_professions', function ($query) use ($request, $generalConfiguration) {

            if (!empty($request['category_id'])) {
                $query->where('category_id', $request['category_id']);
            }

            if (!empty($request['profession_id'])) {
                $query->where('profession_id', $request['profession_id']);
            }

            if ($generalConfiguration['show_accept_lower_level_job_checkbox'] == '1' && !empty($request['level_id'])) {
                $query->whereRaw('FIND_IN_SET(?, accept_level_ids)', [$request['level_id']]);
            } else if ($generalConfiguration['show_accept_lower_level_job_checkbox'] == '0' && !empty($request['level_id'])) {
                $query->where('level_id', $request['level_id']);
            }

            $query->where('status', 'active');
            $query->orderBy("id", 'DESC');

        })->with(['one_hcp_professions' => function ($query) use ($request, $generalConfiguration) {

            if (!empty($request['category_id'])) {
                $query->where('category_id', $request['category_id']);
            }

            if (!empty($request['profession_id'])) {
                $query->where('profession_id', $request['profession_id']);
            }

            if ($generalConfiguration['show_accept_lower_level_job_checkbox'] == '1' && !empty($request['level_id'])) {
                $query->whereRaw('FIND_IN_SET(?, accept_level_ids)', [$request['level_id']]);
            } else if ($generalConfiguration['show_accept_lower_level_job_checkbox'] == '0' && !empty($request['level_id'])) {
                $query->where('level_id', $request['level_id']);
            }

            $query->with([
                'profession' => function ($query) use ($request) {
                    $query->addSelect('id', 'name');
                },
            ]);

            $query->with([
                'level' => function ($query) use ($request) {
                    $query->addSelect('id', 'name');
                },
            ]);

            $query->with([
                'category' => function ($query) use ($request) {
                    $query->addSelect('id', 'name');
                },
            ]);
            $query->where('status', 'active');
            $query->orderBy("id", 'DESC');

        },
        ]);

        if (!empty($request['shift_id'])) {
            if (isset($request['notification_not_sent']) && !empty($request['notification_not_sent'])) {
                $hcpQuery->doesntHave('notifications', 'and', function ($query) use ($request) {

                    if (!empty($request['shift_id'])) {
                        $query->where('shift_id', $request['shift_id']);
                    }

                })->withCount(['notifications' => function ($query) use ($request) {
                    if (!empty($request['shift_id'])) {
                        $query->where('shift_id', $request['shift_id']);
                    }
                }]);

            } else {
                $hcpQuery->withCount(['notifications' => function ($query) use ($request) {
                    if (!empty($request['shift_id'])) {
                        $query->where('shift_id', $request['shift_id']);
                    }
                }]);
            }
        }

        if (isset($request['only_ratings']) && !empty($request['only_ratings'])) {
            $hcpQuery->whereHas('ratings', function ($query) use ($request) {
                $query->where('status', 'publish')->where('hcp_rating', '>=', '4');
            })->withCount(['ratings' => function ($query) use ($request) {
                $query->where('status', 'publish')->where('hcp_rating', '>=', '4');
            }]);
        }

        $hcpQuery->whereHas('buddy_shift_requests', function ($query) use ($request) {
            if (!empty($request['category_id'])) {
                $query->where('category_id', $request['category_id']);
            }

            if (!empty($request['profession_id'])) {
                $query->where('profession_id', $request['profession_id']);
            }
        });

        if (isset($request['shift_start_time']) && isset($request['shift_end_time']) && !empty($request['shift_start_time']) && !empty($request['shift_end_time'])) {

            $hcpQuery->doesntHave('shifts', 'and', function ($query) use ($request) {

                $query->where(function ($query) use ($request) {
                    // $query->where('shift_status', '!=', '3');
                    // $query->where('shift_status', '!=', '6');
                    // $query->where('shift_status', '!=', '4');
                    $query->where('shift_status', '!=', '3');
                    $query->where('shift_status', '!=', '6');
                });

                $startTime = $request['shift_start_time'];
                $endTime = $request['shift_end_time'];

                // $query->where(function ($query) use ($startTime, $endTime) {
                //     if (!empty($startTime) && !empty($endTime)) {
                //         $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                //     }
                // });

                // $query->where(function ($query) use ($startTime, $endTime) {
                //     if (!empty($startTime) && !empty($endTime)) {
                //         $query->orwhere(function ($query) use ($startTime, $endTime) {
                //             $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                //         });
                //         $query->orwhere(function ($query) use ($startTime, $endTime) {
                //             $query->whereBetween('end_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                //         });
                //     }
                // });

                $query->where(function ($query) use ($startTime, $endTime) {
                    if (!empty($startTime) && !empty($endTime)) {
                        $query->orwhere(function ($query) use ($startTime, $endTime) {
                            $query->where('start_time', '<=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($startTime));
                        });
                        $query->orwhere(function ($query) use ($startTime, $endTime) {
                            $query->where('start_time', '>=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($endTime))->where('start_time', '<=', Carbon::parse($endTime));
                        });
                        $query->orwhere(function ($query) use ($startTime, $endTime) {
                            $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)])
                                ->orWhereBetween('end_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                        });
                    }
                });

            })->with(['shifts' => function ($query) use ($request) {

                $query->where(function ($query) use ($request) {
                    $query->where('shift_status', '!=', '3');
                    $query->where('shift_status', '!=', '6');
                });

                $startTime = $request['shift_start_time'];
                $endTime = $request['shift_end_time'];

                // $query->where(function ($query) use ($startTime, $endTime) {
                //     if (!empty($startTime) && !empty($endTime)) {
                //         $query->orwhere(function ($query) use ($startTime, $endTime) {
                //             $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                //         });
                //         $query->orwhere(function ($query) use ($startTime, $endTime) {
                //             $query->whereBetween('end_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                //         });
                //     }
                // });

                $query->where(function ($query) use ($startTime, $endTime) {
                    if (!empty($startTime) && !empty($endTime)) {
                        $query->orwhere(function ($query) use ($startTime, $endTime) {
                            $query->where('start_time', '<=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($startTime));
                        });
                        $query->orwhere(function ($query) use ($startTime, $endTime) {
                            $query->where('start_time', '>=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($endTime))->where('start_time', '<=', Carbon::parse($endTime));
                        });
                        $query->orwhere(function ($query) use ($startTime, $endTime) {
                            $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)])
                                ->orWhereBetween('end_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                        });
                    }
                });

            }]);

            $hcpQuery->doesntHave('hcp_general_documents', 'and', function ($query) use ($request) {

                if (isset($request['shift_start_time']) && !empty($request['shift_start_time'])) {
                    $query->whereDate('expiry_date', '<=', Carbon::parse($request['shift_start_time'])->format('Y-m-d'));
                }

                $query->where('document_approval', '=', 'pending')->orWhere('document_approval', '=', 'pending-reapproval');

                $query->whereHas('document', function ($query) use ($request) {
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'general-statement');
                    $query->where('status', 'active');
                })->with(['document' => function ($query) use ($request) {
                    $query->addSelect('id');
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'general-statement');
                    $query->where('status', 'active');
                }]);

            })->with(['hcp_general_documents' => function ($query) use ($request) {

                if (isset($request['shift_start_time']) && !empty($request['shift_start_time'])) {
                    $query->whereDate('expiry_date', '<=', Carbon::parse($request['shift_start_time'])->format('Y-m-d'));
                }

                $query->whereHas('document', function ($query) use ($request) {
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'general-statement');
                    $query->where('status', 'active');
                })->with(['document' => function ($query) use ($request) {
                    $query->addSelect('id');
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'general-statement');
                    $query->where('status', 'active');
                }]);
            }]);

            $hcpQuery->doesntHave('hcp_profession_documents', 'and', function ($query) use ($request) {

                if (isset($request['shift_start_time']) && !empty($request['shift_start_time'])) {
                    $query->whereDate('expiry_date', '<=', Carbon::parse($request['shift_start_time'])->format('Y-m-d'));
                }

                if (isset($request['profession_id']) && !empty($request['profession_id'])) {
                    $query->where('profession_id', $request['profession_id']);
                }

                $query->where('document_approval', '=', 'pending')->orWhere('document_approval', '=', 'pending-reapproval');

                $query->whereHas('document', function ($query) use ($request) {
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'profession');
                    $query->where('status', 'active');
                })->with(['document' => function ($query) use ($request) {
                    $query->addSelect('id');
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'profession');
                    $query->where('status', 'active');
                }]);

            })->with(['hcp_profession_documents' => function ($query) use ($request) {

                if (isset($request['shift_start_time']) && !empty($request['shift_start_time'])) {
                    $query->whereDate('expiry_date', '<=', Carbon::parse($request['shift_start_time'])->format('Y-m-d'));
                }

                if (isset($request['profession_id']) && !empty($request['profession_id'])) {
                    $query->where('profession_id', $request['profession_id']);
                }

                $query->whereHas('document', function ($query) use ($request) {
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'profession');
                    $query->where('status', 'active');
                })->with(['document' => function ($query) use ($request) {
                    $query->addSelect('id');
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'profession');
                    $query->where('status', 'active');
                }]);
            }]);
        }

        return $hcpQuery->orderBy('favourites_count', 'DESC')->orderBy('distance', 'ASC')->get();
        //--------

    }

    public function get_hcp_listing_for_shift_admin($request)
    {

        $facilityLatitude = null;
        $facilityLongitude = null;
        $radius = null;
        $hcpQuery1 = [];
        $hcpQuery2 = [];

        // General Configuration
        $generalConfiguration = get_general_configuration();
        $radius = $generalConfiguration['radius_km_for_shift_broadcast'];
        //----------------------

        if (isset($request['facility_latitude']) && !empty($request['facility_latitude']) && isset($request['facility_longitude']) && !empty($request['facility_longitude'])) {
            $facilityLatitude = $request['facility_latitude'];
            $facilityLongitude = $request['facility_longitude'];
        }

        $hcpQuery = $this->hcp->where(function ($query) use ($request, $generalConfiguration) {
            $query->where('status', 'active');
            $query->where('points', '>', '0');

            if (!empty($request['buddy_hcp_id'])) {
                $query->where('id', '!=', $request['buddy_hcp_id']);
            }

            if (!empty($request['shift_hcp_id'])) {
                $query->where('id', '!=', $request['shift_hcp_id']);
            }

            if (!empty($request['cancelled_hcps'])) {
                // $query->whereNotIn('id', $request['cancelled_hcps']);
            }

            if (!empty($request['search'])) {
                if (is_numeric($request['search'])) {
                    $query->where('id', 'LIKE', "%{$request['search']}%");
                } else {
                    $query->where(DB::raw('concat(first_name," ",last_name)'), 'like', '%' . strtolower($request['search']) . '%');
                }
            }
        });

        if (!empty($facilityLatitude) && !empty($facilityLongitude)) {
            $hcpQuery->select('id', 'hcp_prefix', 'image', 'contact_number', 'first_name', 'last_name', 'latitude', 'longitude', 'user_id', 'accept_lower_level_job', 'available_for_job', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS(' . $facilityLatitude . ')) * COS(RADIANS(latitude)) * COS( RADIANS(' . $facilityLongitude . ' - longitude)) + SIN(RADIANS(' . $facilityLatitude . ')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<=", $radius);
        }

        $hcpQuery->doesntHave('blocklists', 'and', function ($query) use ($request) {
            if (!empty($request['facility_id'])) {
                $query->where('facility_id', $request['facility_id']);
                $query->where('type', 'blocked');
                $query->where('marked_by', 'facility');
            }
        })->with([
            'blocklists' => function ($query) use ($request) {
                $query->addSelect('id');
                if (!empty($request['facility_id'])) {
                    $query->where('facility_id', $request['facility_id']);
                    $query->where('type', 'blocked');
                    $query->where('marked_by', 'facility');
                }
            },
        ]);

        if (isset($request['check_transfer_case']) && !empty($request['check_transfer_case']) && isset($request['shift_id']) && !empty($request['shift_id'])) {
            $hcpQuery->doesntHave('hcp_shift_transfers', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
            })->with([
                'hcp_shift_transfers' => function ($query) use ($request) {
                    $query->addSelect('id');
                    $query->where('shift_id', $request['shift_id']);
                    $query->where('status', '!=', 'cancelled');
                },
            ]);

            $hcpQuery->doesntHave('hcp_transfer_shift_transfers', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
                $query->where('status', '!=', 'pending');
            })->with([
                'hcp_transfer_shift_transfers' => function ($query) use ($request) {
                    $query->addSelect('id');
                    $query->where('shift_id', $request['shift_id']);
                    $query->where('status', '!=', 'cancelled');
                    $query->where('status', '!=', 'pending');
                },
            ]);
        }

        if (isset($request['check_swap_case']) && !empty($request['check_swap_case']) && isset($request['shift_id']) && !empty($request['shift_id'])) {
            $hcpQuery->doesntHave('shift_hcp_swap', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
            })->with([
                'shift_hcp_swap' => function ($query) use ($request) {
                    $query->addSelect('id');
                    $query->where('shift_id', $request['shift_id']);
                    $query->where('status', '!=', 'cancelled');
                },
            ]);

            $hcpQuery->doesntHave('swap_hcp_shift', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
                $query->where('status', '!=', 'pending');
            })->with([
                'swap_hcp_shift' => function ($query) use ($request) {
                    $query->addSelect('id');
                    $query->where('shift_id', $request['shift_id']);
                    $query->where('status', '!=', 'cancelled');
                    $query->where('status', '!=', 'pending');
                },
            ]);

            $hcpQuery->doesntHave('hcp_swap_rejected_request', 'and', function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
            })->with([
                'hcp_swap_rejected_request' => function ($query) use ($request) {
                    $query->addSelect('id');
                    $query->where('shift_id', $request['shift_id']);
                },
            ]);
        }

        if (isset($request['only_favourites']) && !empty($request['only_favourites'])) {

            $hcpQuery->whereHas('favourites', function ($query) use ($request) {
                if (!empty($request['facility_id'])) {
                    $query->where('facility_id', $request['facility_id']);
                    $query->where('type', 'favourite');
                    $query->where('marked_by', 'facility');
                }
            })->withCount([
                'favourites' => function ($query) use ($request) {
                    if (!empty($request['facility_id'])) {
                        $query->where('facility_id', $request['facility_id']);
                        $query->where('type', 'favourite');
                        $query->where('marked_by', 'facility');
                    }
                },
            ]);

        } else {

            $hcpQuery->with([
                'favourites' => function ($query) use ($request) {
                    $query->addSelect('id');
                    if (!empty($request['facility_id'])) {
                        $query->where('facility_id', $request['facility_id']);
                        $query->where('type', 'favourite');
                        $query->where('marked_by', 'facility');
                    }
                },
            ])->withCount([
                'favourites' => function ($query) use ($request) {
                    if (!empty($request['facility_id'])) {
                        $query->where('facility_id', $request['facility_id']);
                        $query->where('type', 'favourite');
                        $query->where('marked_by', 'facility');
                    }
                },
            ]);
        }

        $hcpQuery->whereHas('one_hcp_professions', function ($query) use ($request, $generalConfiguration) {

            if (!empty($request['category_id'])) {
                $query->where('category_id', $request['category_id']);
            }
            if (!empty($request['profession_id'])) {
                $query->where('profession_id', $request['profession_id']);
            }

            if ($generalConfiguration['show_accept_lower_level_job_checkbox'] == '1' && !empty($request['level_id'])) {
                $query->whereRaw('FIND_IN_SET(?, accept_level_ids)', [$request['level_id']]);
            } else if ($generalConfiguration['show_accept_lower_level_job_checkbox'] == '0' && !empty($request['level_id'])) {
                $query->where('level_id', $request['level_id']);
            }
            $query->orderBy("id", 'DESC');

        })->with(['one_hcp_professions' => function ($query) use ($request, $generalConfiguration) {
            $query->addSelect('id', 'hcp_id', 'category_id', 'profession_id', 'level_id', 'status');
            if (!empty($request['category_id'])) {
                $query->where('category_id', $request['category_id']);
            }

            if (!empty($request['profession_id'])) {
                $query->where('profession_id', $request['profession_id']);
            }
            if ($generalConfiguration['show_accept_lower_level_job_checkbox'] == '1' && !empty($request['level_id'])) {
                $query->whereRaw('FIND_IN_SET(?, accept_level_ids)', [$request['level_id']]);
            } else if ($generalConfiguration['show_accept_lower_level_job_checkbox'] == '0' && !empty($request['level_id'])) {
                $query->where('level_id', $request['level_id']);
            }

            $query->with([
                'profession' => function ($query) use ($request) {
                    $query->addSelect('id', 'name');
                },
            ]);

            $query->with([
                'level' => function ($query) use ($request) {
                    $query->addSelect('id', 'name');
                },
            ]);

            $query->with([
                'category' => function ($query) use ($request) {
                    $query->addSelect('id', 'name');
                },
            ]);

            $query->orderBy("id", 'DESC');

        },
        ]);
//        dd($hcpQuery->get(), "d1");
        if (!empty($request['shift_id'])) {
            if (isset($request['notification_not_sent']) && !empty($request['notification_not_sent'])) {
                $hcpQuery->doesntHave('notifications', 'and', function ($query) use ($request) {

                    if (!empty($request['shift_id'])) {
                        $query->where('shift_id', $request['shift_id']);
                    }

                })->withCount(['notifications' => function ($query) use ($request) {
                    $query->addSelect('id');
                    if (!empty($request['shift_id'])) {
                        $query->where('shift_id', $request['shift_id']);
                    }
                }]);

            } else {
                $hcpQuery->withCount(['notifications' => function ($query) use ($request) {
                    $query->addSelect('id');
                    if (!empty($request['shift_id'])) {
                        $query->where('shift_id', $request['shift_id']);
                    }
                }]);
            }
        }
        if (isset($request['only_ratings']) && !empty($request['only_ratings'])) {
            $hcpQuery->whereHas('ratings', function ($query) use ($request) {
                $query->where('status', 'publish')->where('hcp_rating', '>=', '4');
            })->withCount(['ratings' => function ($query) use ($request) {
                $query->where('status', 'publish')->where('hcp_rating', '>=', '4');
            }]);
        }

        $hcpQuery->whereHas('buddy_shift_requests', function ($query) use ($request) {
            if (!empty($request['category_id'])) {
                $query->where('category_id', $request['category_id']);
            }

            if (!empty($request['profession_id'])) {
                $query->where('profession_id', $request['profession_id']);
            }
        });

        if (isset($request['shift_start_time']) && isset($request['shift_end_time']) && !empty($request['shift_start_time']) && !empty($request['shift_end_time'])) {

            $hcpQuery->withCount(['shifts' => function ($query) use ($request) {
                $query->where(function ($query) use ($request) {
                    // $query->where('shift_status', '!=', '3');
                    // $query->where('shift_status', '!=', '6');
                    $query->where('shift_status', '!=', '4');
                    $query->where('shift_status', '!=', '3');
                    $query->where('shift_status', '!=', '6');
                });

                $startTime = $request['shift_start_time'];
                $endTime = $request['shift_end_time'];

                // $query->where(function ($query) use ($startTime, $endTime) {
                //     if (!empty($startTime) && !empty($endTime)) {
                //         $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                //     }
                // });

                // $query->where(function ($query) use ($startTime, $endTime) {
                //     if (!empty($startTime) && !empty($endTime)) {
                //         $query->orwhere(function ($query) use ($startTime, $endTime) {
                //             $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                //         });
                //         $query->orwhere(function ($query) use ($startTime, $endTime) {
                //             $query->whereBetween('end_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                //         });
                //     }
                // });

                // $query->where(function($query) use ($startTime, $endTime){
                //     if (!empty($startTime) && !empty($endTime)) {
                //         $query->whereBetween('start_time', [Carbon::parse($startTime)->format('Y-m-d H:i'), Carbon::parse($endTime)->format('Y-m-d H:i')])
                //         ->orWhereBetween('end_time', [Carbon::parse($startTime)->format('Y-m-d H:i'), Carbon::parse($endTime)->format('Y-m-d H:i')]);
                //     }
                // });

                $query->where(function ($query) use ($startTime, $endTime) {
                    if (!empty($startTime) && !empty($endTime)) {
                        $query->orwhere(function ($query) use ($startTime, $endTime) {
                            $query->where('start_time', '<=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($startTime));
                        });
                        $query->orwhere(function ($query) use ($startTime, $endTime) {
                            $query->where('start_time', '>=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($endTime))->where('start_time', '<=', Carbon::parse($endTime));
                        });
                        $query->orwhere(function ($query) use ($startTime, $endTime) {
                            $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)])
                                ->orWhereBetween('end_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                        });
                    }
                });
            }]);

            $hcpQuery->with(['current_date_shifts' => function ($query) use ($request) {
                $query->addSelect('id');
                $query->where(function ($query) use ($request) {
                    $query->where('shift_status', '!=', '3');
                });
                $query->where(function ($query) use ($request) {
                    $query->whereDate('start_time', '=', Carbon::parse($request['shift_start_time']));
                });
            }]);

            $hcpQuery->withCount(['hcp_documents as hcp_expire_general_documents' => function ($query) use ($request) {

                if (isset($request['shift_start_time']) && !empty($request['shift_start_time'])) {
                    $query->whereDate('expiry_date', '<=', Carbon::parse($request['shift_start_time'])->format('Y-m-d'));
                }

                $query->whereHas('document', function ($query) use ($request) {
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'general-statement');
                    $query->where('status', 'active');
                })->with(['document' => function ($query) use ($request) {
                    $query->addSelect('id');
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'general-statement');
                    $query->where('status', 'active');
                }]);
            }]);

            $hcpQuery->withCount(['hcp_documents as hcp_expire_profession_document' => function ($query) use ($request) {
                if (isset($request['profession_id']) && !empty($request['profession_id'])) {
                    $query->where('profession_id', $request['profession_id']);
                }

                if (isset($request['shift_start_time']) && !empty($request['shift_start_time'])) {
                    $query->whereDate('expiry_date', '<=', Carbon::parse($request['shift_start_time'])->format('Y-m-d'));
                }

                $query->whereHas('document', function ($query) use ($request) {
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'profession');
                    $query->where('status', 'active');
                })->with(['document' => function ($query) use ($request) {
                    $query->addSelect('id');
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'profession');
                    $query->where('status', 'active');
                }]);
            }]);

            $hcpQuery->withCount(['hcp_documents as hcp_pending_approval_general_documents' => function ($query) use ($request) {
                $query->where('profession_id', '=', NULL);
                $query->where('document_approval', '=', 'pending')->orWhere('document_approval', '=', 'pending-reapproval');

                $query->whereHas('document', function ($query) use ($request) {
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'general-statement');
                    $query->where('status', 'active');
                })->with(['document' => function ($query) use ($request) {
                    $query->addSelect('id');
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'general-statement');
                    $query->where('status', 'active');
                }]);
            }]);

            $hcpQuery->withCount(['hcp_documents as hcp_pending_approval_profession_document' => function ($query) use ($request) {

                if (isset($request['profession_id']) && !empty($request['profession_id'])) {
                    $query->where('profession_id', $request['profession_id']);
                }
                $query->where('document_approval', '=', 'pending')->orWhere('document_approval', '=', 'pending-reapproval');

                $query->whereHas('document', function ($query) use ($request) {
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'profession');
                    $query->where('status', 'active');
                })->with(['document' => function ($query) use ($request) {
                    $query->addSelect('id');
                    $query->where('required_for', 'hcp');
                    $query->where('mandatory_status', 'yes');
                    $query->where('doc_type', 'profession');
                    $query->where('status', 'active');
                }]);
            }]);
        }

        if (isset($request['fortnight_start']) && isset($request['fortnight_end']) && !empty($request['fortnight_start']) && !empty($request['fortnight_end'])) {
            $hcpQuery->with(['fortnight_shifts' => function ($query) use ($request) {
                $query->addSelect('id');
                $query->where(function ($query) use ($request) {
                    $query->where('shift_status', '!=', '3');
                });
                $query->where(function ($query) use ($request) {
                    $query->whereDate('start_time', '>=', Carbon::parse($request['fortnight_start']))
                        ->whereDate('start_time', '<=', Carbon::parse($request['fortnight_end']));
                });
            }]);
        }

        //return $hcpQuery->orderBy('favourites_count', 'DESC')->orderBy('distance', 'ASC')->get();
        //p($hcpQuery->orderBy('distance', 'ASC')->get());
        return $hcpQuery->orderBy('distance', 'ASC')->get();
//        return $hcpQuery->orderBy('distance', 'ASC')->paginate(25);
        //--------

    }

    public function get_hcp_expiry_document($type, $currentDate)
    {
        return $this->hcp->where(function ($query) use ($type, $currentDate) {
            //
        })
            ->whereHas('hcp_documents', function ($query) use ($type, $currentDate) {
                if ($type == 'notification') {
                    $query->where('send_notification_date', $currentDate);
                } else if ($type == 'expiry') {
                    // $query->where('expiry_date', $currentDate);
                    $query->where('expiry_date', '<=', $currentDate);
                    $query->where('document_approval', '!=', 'expired');
                }
                $query->with('document');
            })
            ->with(['hcp_documents' => function ($query) use ($type, $currentDate) {
                if ($type == 'notification') {
                    $query->where('send_notification_date', $currentDate);
                } else if ($type == 'expiry') {
                    // $query->where('expiry_date', $currentDate);
                    $query->where('expiry_date', '<=', $currentDate);
                    $query->where('document_approval', '!=', 'expired');
                }
                $query->with('document');
            }])->orderBy('id', 'DESC')->get();
    }

    public function get_hcp_marked_by_facility($where, $request = null, $facilityId = null, $type = null)
    {
        $paginationLimit = null;

        if (isset($request['hcp_per_page']) && !empty($request['hcp_per_page'])) {
            $paginationLimit = $request['hcp_per_page'];
        } else {
            $paginationLimit = config('constants.pagination_limit');
        }


        $hcpQuery = $this->hcp->where(function ($query) use ($where, $request) {

            if (isset($request['availability']) && $request['availability'] != '') {
                $query->where('available_for_job', $request['availability']);
            }

            if (!empty($request['states'])) {
                $query->whereIn('state_id', $request['states']);
            }

            if (!empty($request['state_id'])) {
                $query->where('state_id', $request['state_id']);
            }

            if (!empty($request['category']) || !empty($request['profession'])) {
                $query->whereHas('hcp_professions', function ($query) use ($request) {
                    if (!empty($request['category'])) {
                        $query->where('category_id', $request['category']);
                    }

                    if (!empty($request['profession'])) {
                        $query->where('profession_id', $request['profession']);
                    }

                });
            }

            if (!empty($request['hcp_search'])) {
                if (is_numeric($request['hcp_search'])) {
                    $query->where('id', 'LIKE', "%{$request['hcp_search']}%");
                } else {
                    $query->where(DB::raw('concat(first_name," ",last_name)'), 'like', '%' . strtolower($request['hcp_search']) . '%');
                }
            }

            $query->where($where);
        })
            ->with([
                'hcp_professions' => function ($query) use ($where, $request) {
                    if (!empty($request['category'])) {
                        $query->where('category_id', $request['category']);
                    }

                    if (!empty($request['profession'])) {
                        $query->where('profession_id', $request['profession']);
                    }
                },
            ])
            ->whereHas('favourites_and_blocklists', function ($query) use ($request, $facilityId, $type) {
                $query->where('facility_id', $facilityId);
                $query->where('marked_by', 'facility');
                $query->where('type', $type);

            });

        if (isset($request['hcp_export']) && $request['hcp_export'] == true) {
            return $hcpQuery->orderBy('id', 'DESC')->get();
        } else {
            return $hcpQuery->orderBy('id', 'DESC')->paginate($paginationLimit)->onEachSide(2);
        }
    }

    public function get_hcp_for_shift_invoice_generate($request = null)
    {
        return $this->hcp->where(function ($query) use ($request) {

            $query->whereHas('shifts', function ($query) use ($request) {
                $query->where('shift_status', '6');
                $query->where('hcp_id', '!=', null);

                if (isset($request['shift_approved_time']) && !empty($request['shift_approved_time'])) {
                    $query->whereDate('shift_approved_time', '<=', $request['shift_approved_time']);
                }

                $query->whereHas('transactions', function ($query) use ($request) {
                    $query->where('amount_for', 'hcp');
                    $query->where('amount_type', 'shift');
                    $query->where('hcp_payment_status', 'pending');
                    $query->where('statement_id', null);
                });
            });
        })
            ->with(['shifts' => function ($query) use ($request) {
                $query->where('shift_status', '6');
                $query->where('hcp_id', '!=', null);
                $query->select('id', 'shift_prefix', 'facility_id', 'hcp_id', 'shift_status', 'shift_approved_time');

                if (isset($request['shift_approved_time']) && !empty($request['shift_approved_time'])) {
                    $query->whereDate('shift_approved_time', '<=', $request['shift_approved_time']);
                }

                $query->whereHas('transactions', function ($query) use ($request) {
                    $query->where('amount_for', 'hcp');
                    $query->where('amount_type', 'shift');
                    $query->where('hcp_payment_status', 'pending');
                    $query->where('statement_id', null);
                });

                $query->with(['transactions' => function ($query) use ($request) {
                    $query->where('amount_for', 'hcp');
                    $query->where('amount_type', 'shift');
                    $query->where('hcp_payment_status', 'pending');
                    $query->where('statement_id', '=', null);
                }]);

            }])->with(['referral_transactions' => function ($query) use ($request) {
                $query->where('amount_for', 'hcp');
                $query->where('amount_type', 'refer-and-earn');
                $query->where('hcp_payment_status', 'pending');
                $query->where('statement_id', '=', null);
            }])->where('id', 85)->select('id', 'hcp_prefix', 'user_id', 'first_name', 'last_name', 'contact_number', 'address', 'status')->get();
    }

    public function get_hcp_shift_hours($request = null)
    {
        return $this->hcp->where(function ($query) use ($request) {
            $query->where('id', $request['hcp_id']);
        })
            ->with(['current_date_shifts' => function ($query) use ($request) {
                $query->where(function ($query) use ($request) {
                    $query->where('shift_status', '!=', '3');
                    $query->where('hcp_id', $request['hcp_id']);
                });

                $query->where(function ($query) use ($request) {
                    if (isset($request['current_day']) && !empty($request['current_day'])) {
                        $query->whereDate('start_time', '=', Carbon::parse($request['current_day']));
                    }
                });
                $query->select('id', 'hcp_id', 'shift_status', 'hours', 'working_hours', 'start_time', 'end_time');
            }])
            ->with(['fortnight_shifts' => function ($query) use ($request) {
                $query->where(function ($query) use ($request) {
                    $query->where('shift_status', '!=', '3');
                    $query->where('hcp_id', $request['hcp_id']);
                });

                $query->where(function ($query) use ($request) {
                    if (isset($request['fortnight_start']) && !empty($request['fortnight_start']) && isset($request['fortnight_end']) && !empty($request['fortnight_end'])) {
                        $query->whereDate('start_time', '>=', Carbon::parse($request['fortnight_start']))
                            ->whereDate('start_time', '<=', Carbon::parse($request['fortnight_end']));
                    }
                });

                $query->select('id', 'hcp_id', 'shift_status', 'hours', 'working_hours', 'start_time', 'end_time');
            }])->first();
    }

    public function get_hcp_shift_history($request = null)
    {
        return $this->hcp->where(function ($query) use ($request) {
            $query->where('id', $request['hcp_id']);

        })->withCount([
            'hcp_shift_transfers' => function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
            },
        ])->withCount([
            'hcp_transfer_shift_transfers' => function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
                $query->where('status', '!=', 'pending');
            },
        ])->withCount([
            'shift_hcp_swap' => function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
            },
        ])->withCount([
            'swap_hcp_shift' => function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
                $query->where('status', '!=', 'cancelled');
                $query->where('status', '!=', 'pending');
            },
        ])->withCount([
            'hcp_swap_rejected_request' => function ($query) use ($request) {
                $query->where('shift_id', $request['shift_id']);
            },
        ])->first();
    }

    public function get_hcp_for_notification($request = null, $in_array = false)
    {
        $hcps =  $this->hcp->where(function ($query) use ($request) {
            $query->where('status', '!=', 'archived');
            if (!empty($request['state_id'])) {
                $query->where('state_id', $request['state_id']);
            }

            $query->whereHas('user', function ($query) use ($request) {
                $query->where('device_id', '!=', null);
            });

        })
            ->with([
                'user' => function ($query) use ($request) {
                    $query->where('device_id', '!=', null);
                },
            ])->select('id', 'user_id', 'contact_number')->get();
         return !$in_array ? $hcps :  $hcps->pluck("user.device_id")->toArray();
    }
}

