<?php

namespace App\Repositories;
use App\Interfaces\ShiftInterface;
use App\Models\Shift;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Pagination\LengthAwarePaginator;
use DB;

class ShiftRepository implements ShiftInterface
{
    private $shift;
    public function __construct(Shift $shift){
        $this->shift = $shift;
    }

    public function find_shift($id){
        return $this->shift->with('category', 'profession', 'level', 'facility', 'hcp', 'facility_group', 'cancelled_shift', 'buddy_hcp', 'shift_transferred','rating_reviews', 'shift_swap', 'state')->find($id);
    }

    public function getAll($where, $options = false, $request = null){
        $paginationLimit = null;

        if(isset($request['shift_per_page']) && !empty($request['shift_per_page'])) {
            $paginationLimit = $request['shift_per_page'];
        } else {
            $paginationLimit = config('constants.pagination_limit');
        }

        if($options){
            $shiftQuery = $this->shift->where(function ($query) use ($where, $request) {

                if(!empty($request['states'])) {
                    $query->whereIn('state_id', $request['states']);
                }

                if(!empty($request['state_id'])) {
                    $query->where('state_id', $request['state_id']);
                }

                if(isset($request['status']) && $request['status'] !='' && $request['status'] != 'all') {
                    $query->where('shift_status', $request['status']);
                }

                if(!empty($request['facility'])) {
                    $query->where('facility_id', $request['facility']);
                }

                if(!empty($request['profession'])) {
                    $query->where('profession_id', $request['profession']);
                }

                if(!empty($request['category'])) {
                    $query->where('category_id', $request['category']);
                }

                if(!empty($request['level'])) {
                    $query->where('hcp_level_id', $request['level']);
                }

                if(!empty($request['shift_type'])) {
                    $query->where('shift_type', $request['shift_type']);
                }

                if(!empty($request['date_range'])) {
                    $date = explode('to', $request['date_range']);
                    $query->whereBetween('start_time', [trim($date[0]), trim($date[1])]);
                }

                if(!empty($request['date_from'])) {
                    $query->whereDate('start_time', '>=', $request['date_from']);
                }

                if(!empty($request['date_to'])) {
                    $query->whereDate('start_time', '<=', $request['date_to']);
                }

                if(!empty($request['assigned_facilities'])) {
                    $query->whereIn('facility_id', $request['assigned_facilities']);
                }

                if(!empty($request['facility_group_ids'])) {
                    $query->whereIn('facility_group_id', $request['facility_group_ids']);
                }

                if(isset($request['last_days']) && !empty($request['last_days'])){
                    $startDate = date('Y-m-d',strtotime("-".$request['last_days']." day",strtotime(date('M d, Y'))));
                    $endDate =  date('Y-m-d');
                    $query->whereBetween('created_at',[$startDate,$endDate]);
                }

                if(!empty($request['filter_mints'])) {
                    $startTime = Carbon::now()->addMinutes($request['filter_mints'])->format('Y-m-d H:i');
                    $currentTime = Carbon::now()->format('Y-m-d H:i');
                    $query->where('start_time', '<', $startTime)->where('start_time', '>', $currentTime);
                }

                if(!empty($request['shift_filter_traffic_lights'])) {
                    if ($request['shift_filter_traffic_lights'] == 'blue') {
                        $query->whereIn('id', explode(",", $request['blue_traffic_lights_shifts']));
                    } else if($request['shift_filter_traffic_lights'] == 'amber') {
                        $query->whereIn('id', explode(",", $request['amber_traffic_lights_shifts']));
                    } else if($request['shift_filter_traffic_lights'] == 'red') {
                        $query->whereIn('id', explode(",", $request['red_traffic_lights_shifts']));
                    }
                }

                if(!empty($request['shift_hcp_search'])) {
                    if(is_numeric($request['shift_hcp_search'])) {
                        $query->where('id', 'LIKE', "%{$request['shift_hcp_search']}%");
                    } else {
                        $query->whereHas('hcp', function ($query) use ($request) {
                            $query->where(DB::raw('concat(first_name," ",last_name)'), 'like', '%'.strtolower($request['shift_hcp_search']).'%');
                        });
                    }
                }

                if(!empty($request['shift_facility_search'])) {
                    if(is_numeric($request['shift_facility_search'])) {
                        $query->where('id', 'LIKE', "%{$request['shift_facility_search']}%");
                    } else {
                        $query->whereHas('facility', function ($query) use ($request) {
                            $query->where('name', 'LIKE', "%{$request['shift_facility_search']}%");
                        });
                    }
                }

                if(!empty($request['latitude']) && !empty($request['longitude']) && !empty($request['radius'])) {
                    $query->whereHas('facility', function ($query) use ($request) {

                        if(!empty($request['latitude']) && !empty($request['longitude']) && !empty($request['radius'])) {
                            $query->select('*', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS('.$request['latitude'].')) * COS(RADIANS(latitude)) * COS( RADIANS('.$request['longitude'].' - longitude)) + SIN(RADIANS('.$request['latitude'].')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<", $request['radius']);
                        } else {
                            $query->select('*');
                        }
                    });
                }

                if(!empty($request['shift_paid_filter']) && $request['shift_paid_filter'] == 'paid') {
                    $query->whereHas('transactions', function ($query) use ($request) {
                        $query->where('status', 'completed');
                        $query->where('amount_for', 'facility');
                    });
                }

                if(!empty($request['shift_paid_filter']) && $request['shift_paid_filter'] == 'unpaid') {
                    $query->whereHas('transactions', function ($query) use ($request) {
                        $query->where('status', 'pending');
                        $query->where('amount_for', 'facility');
                    });
                }

                $query->where($where);

            })
            ->with('category', 'profession', 'facility')
            ->with(['hcp' => function ($query) use ($request) {
                if(!empty($request['shift_hcp_search']) && !is_numeric($request['shift_hcp_search'])) {
                    $query->where(DB::raw('concat(first_name," ",last_name)'), 'like', '%'.strtolower($request['shift_hcp_search']).'%');
                }
            }])
            ->with(['transactions' => function ($query) use ($request) {
                if(!empty($request['shift_paid_filter']) && $request['shift_paid_filter'] == 'unpaid') {
                    $query->where('status', 'pending');
                }
                if(!empty($request['shift_paid_filter']) && $request['shift_paid_filter'] == 'paid') {
                    $query->where('status', 'completed');
                }
                $query->where('amount_for', 'facility');
            }])
            ->with(['facility' => function ($query) use($request) {
                if(!empty($request['latitude']) && !empty($request['longitude']) && !empty($request['radius'])) {
                    $query->select('*', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS('.$request['latitude'].')) * COS(RADIANS(latitude)) * COS( RADIANS('.$request['longitude'].' - longitude)) + SIN(RADIANS('.$request['latitude'].')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<", $request['radius']);
                } else {
                    $query->select('*');
                }
            }]);

            if(isset($request['shortby_filter']) && !empty($request['shortby_filter'])) {
                if ($request['shortby_filter'] == '1') {
                    $shiftQuery->orderBy('start_time', 'ASC');
                } else if($request['shortby_filter'] == '2') {
                    $shiftQuery->orderBy('start_time', 'DESC');
                } else {
                    $shiftQuery->orderBy('id', 'DESC');
                }
            }
            elseif(!empty($where) && isset($where['shift_status']) && ($where['shift_status']=='0' || $where['shift_status']=='1')){
                $shiftQuery->orderBy('start_time', 'ASC');
            }
            elseif(!empty($where) && isset($where['shift_status']) && ($where['shift_status']=='3' || $where['shift_status']=='4')){
                $shiftQuery->orderBy('updated_at', 'DESC');
            } else {
                $shiftQuery->orderBy('id', 'DESC');
            }

            if(isset($request['shift_export']) && $request['shift_export'] == true) {
                return $shiftQuery->get();
            } else {
                return $shiftQuery->paginate($paginationLimit)->fragment('shift')->onEachSide(2);
            }

        } else {
            return $this->shift->all();
        }
    }

    public function store($array){
        return $this->shift->create($array);
    }

    public function update($array, $where){
        return $this->shift->where($where)->update($array);
    }

    public function delete($id){
        return $this->shift->destroy($id);
    }

    public function shift_where($where){
        return $this->shift->with('category', 'profession', 'facility', 'hcp', 'buddy_hcp', 'cancelled_shifts', 'state')->where($where)->get();
    }

    public function api_shift_where($where){
        return $this->shift->with('category', 'profession', 'facility', 'hcp', 'buddy_hcp', 'state')->where($where)->first();
    }

    public function api_get_shifts($request, $user){
        $paginationLimit = config('constants.pagination_limit');

        if($request['shift_status'] == 'upcoming') {

            $upcomingShifts =  $this->shift->where(function ($query) use ($request, $user) {
                $query->where('start_time', '<=',  Carbon::now($user->hcp->state->time_zone_name)->addHours(12)->format('Y-m-d H:i'));
                $query->where('shift_status', '1')->where('hcp_id', $user->hcp->id);
            })->with(['hcp' => function ($query) {
                $query->select('id', 'first_name', 'last_name');
            }])->with(['category' => function ($query) {
                $query->select('id', 'name');
            }])->with(['profession' => function ($query) {
                $query->select('id', 'name');
            }])->with(['level' => function ($query) {
                $query->select('id', 'name');
            }])->with(['facility' => function ($query) {
                $query->select('id', 'name', 'facility_group_id', 'address');
            }])
            ->with('state')
            ->orderBy('start_time', 'ASC')->paginate($paginationLimit);

            $runningShifts = $this->shift->where(function ($query) use ($request, $user) {

                $query->where('shift_status', '2')->where('hcp_id', $user->hcp->id);

            })->with(['hcp' => function ($query) {
                $query->select('id', 'first_name', 'last_name');
            }])->with(['category' => function ($query) {
                $query->select('id', 'name');
            }])->with(['profession' => function ($query) {
                $query->select('id', 'name');
            }])->with(['level' => function ($query) {
                $query->select('id', 'name');
            }])->with(['facility' => function ($query) {
                $query->select('id', 'name', 'facility_group_id', 'address');
            }])
            ->with('state')
            ->orderBy('start_time', 'ASC')->paginate($paginationLimit);

            if (count($upcomingShifts) > 0 || count($runningShifts) > 0) {
                return ['upcoming_shifts' => $upcomingShifts, 'running_shifts' => $runningShifts];
            } else {
                return $upcomingShifts;
            }

        } else if($request['shift_status'] == 'available') {
            $shiftReturn = [];

            // General Configuration
            $generalConfiguration = get_general_configuration();
            $radius = $generalConfiguration['radius_km_for_shift_broadcast'];
            //----------------------

            // Get Hcp latitude & longitude
            $latitude = $user->hcp['latitude'];
            $longitude = $user->hcp['longitude'];
            //-----------------------------

            // Get Hcp Profession
            $hcpProfessionsIds = [];
            if(!empty($user->hcp->hcp_professions)){
                foreach ($user->hcp->hcp_professions as $hcpProfessions) {
                    $hcpProfessionsIds['category_ids'][] = $hcpProfessions['category_id'];
                    $hcpProfessionsIds['profession_ids'][] = $hcpProfessions['profession_id'];
                    $hcpProfessionsIds['level_ids'][] = $hcpProfessions['level_id'] ?? 0;
                }
            }
            //-------------------

            // Get Hcp Shift
            $overlappingShifts = [];
            $hcpShifts = $this->shift->where(function ($query) use ($request, $user) {
                $query->where('hcp_id', '=', $user->hcp->id);
            })
            ->where(function ($query) use ($request, $user) {
                $query->where('shift_status', '!=', '3');
                $query->where('shift_status', '!=', '6');
            })->select('id', 'shift_prefix', 'start_time', 'end_time', 'shift_type', 'shift_status', 'hcp_id')->orderBy('start_time', 'ASC')->get()->toArray();

            if(!empty($hcpShifts)) {
                foreach ($hcpShifts as $hcpShift) {
                    $startTime = $hcpShift['start_time'];
                    $endTime = $hcpShift['end_time'];
                    $shiftRecords = $this->shift->where(function ($query) use ($startTime, $endTime) {
                        $query->where('shift_status', '=', '0');
                    })
                    ->where(function ($query) use ($startTime, $endTime) {

                        if (!empty($startTime) && !empty($endTime)) {
                            $query->orwhere(function ($query) use ($startTime, $endTime) {
                                $query->where('start_time', '<=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($startTime));
                            });
                            $query->orwhere(function ($query) use ($startTime, $endTime) {
                                $query->where('start_time', '>=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($endTime))->where('start_time', '<=', Carbon::parse($endTime));
                            });
                            $query->orwhere(function ($query) use ($startTime, $endTime) {
                                $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)])
                                ->orWhereBetween('end_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                            });
                        }
                    })
                    ->select('id', 'start_time', 'end_time', 'shift_status', 'hcp_id')->orderBy('start_time', 'ASC')->get();
                    if(count($shiftRecords)) {
                        foreach ($shiftRecords as $shiftRecord) {
                            $overlappingShifts[] = $shiftRecord->id;
                        }
                    }
                }
            }
            //--------------

            // Available Shifts
            $availableShifts =  $this->shift->where(function ($query) use ($request, $user, $latitude, $longitude, $radius, $hcpProfessionsIds, $overlappingShifts, $generalConfiguration) {
                $query->where('shift_status', '0');

                if(count($overlappingShifts) && !empty($overlappingShifts)) {
                    $query->whereNotIn('id', $overlappingShifts);
                }

                if($generalConfiguration['show_accept_lower_level_job_checkbox'] == '1' && $user->hcp['accept_lower_level_job'] == 1) {

                    $query->where(function ($query) use ($request, $user, $hcpProfessionsIds) {
                        if (isset($hcpProfessionsIds['category_ids'])) {
                            $query->whereIn('category_id', $hcpProfessionsIds['category_ids']);
                        }
                        if (isset($hcpProfessionsIds['profession_ids'])) {
                            $query->whereIn('profession_id', $hcpProfessionsIds['profession_ids']);
                        }
                    });

                    $query->where(function ($query) use ($request, $user, $hcpProfessionsIds) {
                        if (isset($hcpProfessionsIds['level_ids'])) {
                            // $query->orWhereIn('hcp_level_id', $hcpProfessionsIds['level_ids']);
                            foreach ($hcpProfessionsIds['level_ids'] as $level) {
                                if ($level != '0') {
                                    $query->orWhere('hcp_level_id', '<=',  $level);
                                } else {
                                    $query->orWhere('hcp_level_id',  null);
                                }
                            }
                        }
                    });
                } else {
                    $query->where(function ($query) use ($request, $user, $hcpProfessionsIds) {
                        if (isset($hcpProfessionsIds['category_ids'])) {
                            $query->whereIn('category_id', $hcpProfessionsIds['category_ids']);
                        }
                        if (isset($hcpProfessionsIds['profession_ids'])) {
                            $query->whereIn('profession_id', $hcpProfessionsIds['profession_ids']);
                        }
                    });

                    $query->where(function ($query) use ($request, $user, $hcpProfessionsIds) {
                        if (isset($hcpProfessionsIds['level_ids'])) {
                            // $query->orWhereIn('hcp_level_id', $hcpProfessionsIds['level_ids']);
                            foreach ($hcpProfessionsIds['level_ids'] as $level) {
                                if ($level != '0') {
                                    $query->orWhere('hcp_level_id',  $level);
                                } else {
                                    $query->orWhere('hcp_level_id',  null);
                                }
                            }
                        }
                    });
                }

                $query->whereHas('facility', function ($query) use ($request, $latitude, $longitude, $radius, $user) {

                    if(!empty($latitude) && !empty($longitude) && !empty($radius)) {
                        $query->select('id','name', 'facility_group_id', 'address', 'latitude', 'longitude', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS('.$latitude.')) * COS(RADIANS(latitude)) * COS( RADIANS('.$longitude.' - longitude)) + SIN(RADIANS('.$latitude.')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<", $radius);
                    } else {
                        $query->select('id', 'name', 'facility_group_id', 'address', 'latitude', 'longitude');
                    }

                    $query->doesntHave('blocklists', 'and', function($query) use ($user){
                        if (!empty($user->hcp->id)) {
                            $query->where('hcp_id', $user->hcp->id);
                            $query->where('type', 'blocked');
                            $query->where('marked_by', 'facility');
                        }
                    });
                });

                // $query->doesntHave('cancelled_shift', 'and', function($query) use($request, $user){
                //     $query->where('hcp_id', '=', $user->hcp->id);
                // });

                // $query->doesntHave('rejected_shifts', 'and', function($query) use($request, $user){
                //     $query->where('hcp_id', '=', $user->hcp->id);
                // });

                $query->whereHas('profession', function($query) use($request, $user){
                    $query->whereHas('buddy_shift_requests', function($query) use($request, $user){
                        $query->where('hcp_id', $user->hcp->id);
                        $query->where('status', "approved");
                    });
                });

            })
            ->with(['category' => function ($query) {
                $query->select('id', 'name');
            }])->with(['profession' => function ($query) use($request, $user) {
                $query->select('id', 'name');

                $query->whereHas('buddy_shift_requests', function($query) use($request, $user){
                    $query->where('hcp_id', $user->hcp->id);
                    $query->where('status', "approved");
                });

                $query->with(['buddy_shift_requests' => function ($query) use($request, $user) {
                    $query->where('hcp_id', $user->hcp->id);
                    $query->where('status', "approved");
                }]);

            }])->with(['level' => function ($query) {
                $query->select('id', 'name');
            }])->with(['facility' => function ($query) use($latitude, $longitude, $radius, $user) {

                if(!empty($latitude) && !empty($longitude) && !empty($radius)) {
                    $query->select('id','name', 'facility_group_id', 'address', 'latitude', 'longitude', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS('.$latitude.')) * COS(RADIANS(latitude)) * COS( RADIANS('.$longitude.' - longitude)) + SIN(RADIANS('.$latitude.')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<", $radius);
                } else {
                    $query->select('id', 'name', 'facility_group_id', 'address', 'latitude', 'longitude');
                }

                $query->with(['blocklists' => function ($query) use($user) {
                    if (!empty($user->hcp->id)) {
                        $query->where('hcp_id', $user->hcp->id);
                        $query->where('type', 'blocked');
                        $query->where('marked_by', 'facility');
                    }
                }]);

            }])->with(['cancelled_shift' => function ($query) use($request, $user) {
                $query->where('hcp_id', $user->hcp->id);
            }])->with(['rejected_shifts' => function ($query) use($request, $user) {
                $query->where('hcp_id', $user->hcp->id);
            }])
            ->with('state')
            ->orderBy('start_time', 'ASC')->paginate($paginationLimit);
            $shiftReturn['available_shifts'] = $availableShifts;
            //-----------------

            // Transfer Shifts
            $transferShifts =  $this->shift->where(function ($query) use ($request, $user) {
                $query->where('shift_status', '1');
                $query->where('shift_transfer_swap_status', 'transferred');

                $query->whereHas('shift_transfer_to', function($query) use($request, $user){
                    $query->where('transfer_hcp_id', '=', $user->hcp->id);
                    $query->where('status', '=', 'pending');
                    $query->with(['hcp' => function ($query) {
                        $query->select('id', 'first_name', 'last_name');
                    }]);
                });

                $query->whereHas('profession', function($query) use($request, $user){
                    $query->whereHas('buddy_shift_requests', function($query) use($request, $user){
                        $query->where('hcp_id', $user->hcp->id);
                        $query->where('status', "approved");
                    });
                });

                // $query->doesntHave('rejected_shifts', 'and', function($query) use($request, $user){
                //     $query->where('hcp_id', '=', $user->hcp->id);
                // });

            })->with(['category' => function ($query) {
                $query->select('id', 'name');
            }])->with(['profession' => function ($query) use($request, $user) {
                $query->select('id', 'name');

                $query->whereHas('buddy_shift_requests', function($query) use($request, $user){
                    $query->where('hcp_id', $user->hcp->id);
                    $query->where('status', "approved");
                });

                $query->with(['buddy_shift_requests' => function ($query) use($request, $user) {
                    $query->where('hcp_id', $user->hcp->id);
                    $query->where('status', "approved");
                }]);

            }])->with(['level' => function ($query) {
                $query->select('id', 'name');
            }])->with(['facility' => function ($query) {
                $query->select('id', 'name', 'facility_group_id', 'address');
            }])->with(['shift_transfer_to' => function ($query) use($request, $user) {
                $query->where('transfer_hcp_id', '=', $user->hcp->id);
                $query->where('status', '=', 'pending');
                $query->with(['hcp' => function ($query) {
                    $query->select('id', 'first_name', 'last_name');
                }]);
            }])->with(['rejected_shifts' => function ($query) use($request, $user) {
                $query->where('hcp_id', $user->hcp->id);
            }])
            ->with('state')
            ->orderBy('start_time', 'ASC')->paginate($paginationLimit);
            $shiftReturn['transfer_shifts'] = $transferShifts;
            //----------------

            // Swap Shifts
            $swapShifts = $this->shift->where(function ($query) use ($request, $user, $latitude, $longitude, $radius) {

                $query->where('shift_status', '1');
                $query->where('shift_transfer_swap_status', 'swapped');

                $query->whereHas('shift_swap', function($query) use($request, $user){
                    $query->where('status', '=', 'pending');
                    $query->where('hcp_id', '!=', $user->hcp->id);
                    $query->whereRaw('FIND_IN_SET('.$user->hcp->id.',request_sent_hcps)');
                    $query->with(['hcp' => function ($query) {
                        $query->select('id', 'first_name', 'last_name');
                    }]);
                });

                $query->whereHas('facility', function ($query) use ($request, $latitude, $longitude, $radius, $user) {
                    if(!empty($latitude) && !empty($longitude) && !empty($radius)) {
                        $query->select('id','name', 'facility_group_id', 'address', 'latitude', 'longitude', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS('.$latitude.')) * COS(RADIANS(latitude)) * COS( RADIANS('.$longitude.' - longitude)) + SIN(RADIANS('.$latitude.')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<", $radius);
                    } else {
                        $query->select('id', 'name', 'facility_group_id', 'address', 'latitude', 'longitude');
                    }

                    $query->doesntHave('blocklists', 'and', function($query) use ($user){
                        if (!empty($user->hcp->id)) {
                            $query->where('hcp_id', $user->hcp->id);
                            $query->where('type', 'blocked');
                            $query->where('marked_by', 'facility');
                        }
                    });
                });

                // $query->doesntHave('swap_reject_request', 'and', function($query) use($request, $user){
                //     $query->where('hcp_id', '=', $user->hcp->id);
                // });

                $query->whereHas('profession', function($query) use($request, $user){
                    $query->whereHas('buddy_shift_requests', function($query) use($request, $user){
                        $query->where('hcp_id', $user->hcp->id);
                        $query->where('status', "approved");
                    });
                });

            })->with(['category' => function ($query) {
                $query->select('id', 'name');
            }])->with(['profession' => function ($query) use($request, $user) {
                $query->select('id', 'name');

                $query->whereHas('buddy_shift_requests', function($query) use($request, $user){
                    $query->where('hcp_id', $user->hcp->id);
                    $query->where('status', "approved");
                });

                $query->with(['buddy_shift_requests' => function ($query) use($request, $user) {
                    $query->where('hcp_id', $user->hcp->id);
                    $query->where('status', "approved");
                }]);

            }])->with(['level' => function ($query) {
                $query->select('id', 'name');
            }])->with(['facility' => function ($query) use($latitude, $longitude, $radius, $user) {

                if(!empty($latitude) && !empty($longitude) && !empty($radius)) {
                    $query->select('id','name', 'facility_group_id', 'address', 'latitude', 'longitude', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS('.$latitude.')) * COS(RADIANS(latitude)) * COS( RADIANS('.$longitude.' - longitude)) + SIN(RADIANS('.$latitude.')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<", $radius);
                } else {
                    $query->select('id', 'name', 'facility_group_id', 'address', 'latitude', 'longitude');
                }

                $query->with(['blocklists' => function ($query) use($user) {
                    if (!empty($user->hcp->id)) {
                        $query->where('hcp_id', $user->hcp->id);
                        $query->where('type', 'blocked');
                        $query->where('marked_by', 'facility');
                    }
                }]);

            }])->with(['shift_swap' => function ($query) use($request, $user) {
                $query->where('status', '=', 'pending');
                $query->where('hcp_id', '!=', $user->hcp->id);
                $query->whereRaw('FIND_IN_SET('.$user->hcp->id.',request_sent_hcps)');
                $query->with(['hcp' => function ($query) {
                    $query->select('id', 'first_name', 'last_name');
                }]);
            }])->with(['swap_reject_request' => function ($query) use($request, $user) {
                $query->where('hcp_id', '=', $user->hcp->id);
            }])
            ->with('state')
            ->orderBy('start_time', 'ASC')->paginate($paginationLimit);
            $shiftReturn['swap_shifts'] = $swapShifts;
            //----------------

            if (count($shiftReturn['available_shifts']) > 0 || count($shiftReturn['transfer_shifts']) > 0 || count($shiftReturn['swap_shifts']) > 0)
            {
                return $shiftReturn;
            } else {
                return $shiftReturn['available_shifts'];
            }

        } else if($request['shift_status'] == 'completed') {

            // Approved Shifts
            $approvedShifts = $this->shift->where(function ($query) use ($request, $user) {

                if($request['shift_status'] == 'completed') {
                    $query->where('shift_status', '6')->where('hcp_id', $user->hcp->id);
                }

                if(isset($request['filter_start_date']) && isset($request['filter_end_date']) && !empty($request['filter_end_date'])&& !empty($request['filter_start_date'])) {

                    $query->whereDate('start_time', '>=', Carbon::parse($request['filter_start_date'])->format('Y-m-d'))
                    ->whereDate('start_time', '<=', Carbon::parse($request['filter_end_date'])->format('Y-m-d'));
                }

            })->with(['hcp' => function ($query) {
                $query->select('id', 'first_name', 'last_name');
            }])->with(['category' => function ($query) {
                $query->select('id', 'name');
            }])->with(['profession' => function ($query) {
                $query->select('id', 'name');
            }])->with(['level' => function ($query) {
                $query->select('id', 'name');
            }])->with(['facility' => function ($query) {
                $query->select('id', 'name', 'facility_group_id', 'address');
            }])->with(['cancelled_shift' => function ($query) use($request, $user) {
                $query->where('hcp_id', $user->hcp->id);
            }])->with(['shift_transfer_from' => function ($query) use($request, $user) {
                $query->where('hcp_id', $user->hcp->id);
                $query->with(['transfer_hcp' => function ($query) {
                    $query->select('id', 'first_name', 'last_name', 'image');
                }]);
            }])->with(['shift_transfer_to' => function ($query) use($request, $user) {
                $query->where('transfer_hcp_id', $user->hcp->id);
            }])
            ->with('state')
            ->orderBy('start_time', 'DESC')->paginate($paginationLimit);
            //----------------

            // Completed Shifts
            $completedShifts = $this->shift->where(function ($query) use ($request, $user) {

                if($request['shift_status'] == 'completed') {
                    $query->where('shift_status', '4')->where('hcp_id', $user->hcp->id);
                }

                if(isset($request['filter_start_date']) && isset($request['filter_end_date']) && !empty($request['filter_end_date'])&& !empty($request['filter_start_date'])) {

                    $query->whereDate('start_time', '>=', Carbon::parse($request['filter_start_date'])->format('Y-m-d'))
                    ->whereDate('start_time', '<=', Carbon::parse($request['filter_end_date'])->format('Y-m-d'));
                }

            })->with(['hcp' => function ($query) {
                $query->select('id', 'first_name', 'last_name');
            }])->with(['category' => function ($query) {
                $query->select('id', 'name');
            }])->with(['profession' => function ($query) {
                $query->select('id', 'name');
            }])->with(['level' => function ($query) {
                $query->select('id', 'name');
            }])->with(['facility' => function ($query) {
                $query->select('id', 'name', 'facility_group_id', 'address');
            }])->with(['cancelled_shift' => function ($query) use($request, $user) {
                $query->where('hcp_id', $user->hcp->id);
            }])->with(['shift_transfer_from' => function ($query) use($request, $user) {
                $query->where('hcp_id', $user->hcp->id);
                $query->with(['transfer_hcp' => function ($query) {
                    $query->select('id', 'first_name', 'last_name', 'image');
                }]);
            }])->with(['shift_transfer_to' => function ($query) use($request, $user) {
                $query->where('transfer_hcp_id', $user->hcp->id);
            }])
            ->with('state')
            ->orderBy('start_time', 'DESC')->paginate($paginationLimit);
            //-----------------

            if (count($approvedShifts) > 0 || count($completedShifts) > 0) {
                return ['approved_shifts' => $approvedShifts ,'completed_shifts' => $completedShifts];
            } else {
                return $approvedShifts;
            }

        } else {
            return $this->shift->where(function ($query) use ($request, $user) {

                if($request['shift_status'] == 'scheduled') {
                    $query->where('shift_status', '1')->where('hcp_id', $user->hcp->id);
                } else if($request['shift_status'] == 'running') {
                    $query->where('shift_status', '2')->where('hcp_id', $user->hcp->id);
                } else if($request['shift_status'] == 'cancelled') {

                    if(isset($request['filter_start_date']) && isset($request['filter_end_date']) && !empty($request['filter_end_date'])&& !empty($request['filter_start_date'])) {

                        $query->whereDate('start_time', '>=', Carbon::parse($request['filter_start_date'])->format('Y-m-d'))
                        ->whereDate('start_time', '<=', Carbon::parse($request['filter_end_date'])->format('Y-m-d'));
                    }

                    $query->whereHas('cancelled_shift', function($query) use($request, $user){
                        $query->where('hcp_id', $user->hcp->id);
                    });

                } else if($request['shift_status'] == 'transferred') {

                    if(isset($request['filter_start_date']) && isset($request['filter_end_date']) && !empty($request['filter_end_date'])&& !empty($request['filter_start_date'])) {

                        $query->whereDate('start_time', '>=', Carbon::parse($request['filter_start_date'])->format('Y-m-d'))
                        ->whereDate('start_time', '<=', Carbon::parse($request['filter_end_date'])->format('Y-m-d'));
                    }

                    $query->whereHas('shift_transfer_from', function($query) use($request, $user){
                        $query->where('hcp_id', $user->hcp->id);
                        $query->where('status', '!=', 'cancelled');
                        $query->where('status', '!=', 'reject');
                    });

                }

            })->with(['hcp' => function ($query) {
                $query->select('id', 'first_name', 'last_name');
            }])->with(['category' => function ($query) {
                $query->select('id', 'name');
            }])->with(['profession' => function ($query) {
                $query->select('id', 'name');
            }])->with(['level' => function ($query) {
                $query->select('id', 'name');
            }])->with(['facility' => function ($query) {
                $query->select('id', 'name', 'facility_group_id', 'address');
            }])->with(['cancelled_shift' => function ($query) use($request, $user) {
                $query->where('hcp_id', $user->hcp->id);
            }])->with(['shift_transfer_from' => function ($query) use($request, $user) {
                $query->where('hcp_id', $user->hcp->id);
                $query->with(['transfer_hcp' => function ($query) {
                    $query->select('id', 'first_name', 'last_name', 'image');
                }]);
                $query->where('status', '!=', 'cancelled');
                $query->where('status', '!=', 'reject');
            }])->with(['shift_transfer_to' => function ($query) use($request, $user) {
                $query->where('transfer_hcp_id', $user->hcp->id);
            }])
            ->with('state')
            ->orderBy('start_time', 'ASC')->paginate($paginationLimit);
        }
    }

    public function api_get_shift_details($request, $user){

        return $this->shift->select('shifts.*','facility_documents.id as fc_doc_id','facility_documents.document_name as facility_duty_document','facility_documents.profession_id as fc_doc_profession_id')->where(function ($query) use ($request, $user) {

            $query->where('shifts.id', $request['shift_id']);

            if (!empty($request['start_time']) && !empty($request['end_time'])) {
                $query->where('start_time', '>=', Carbon::parse($request['start_time']))->where('start_time', '<=', Carbon::parse($request['end_time']))
                ->orWhere('end_time', '>=', Carbon::parse($request['start_time']))->where('end_time', '<=', Carbon::parse($request['end_time']));
            }

        })->with(['hcp' => function ($query) {
            $query->select('id', 'first_name', 'last_name');
        }])->with(['buddy_hcp' => function ($query) {
            $query->select('id', 'first_name', 'last_name');
        }])->with(['rating_reviews' => function ($query) {
            $query->select('*');
        }])->with(['hcp_rating' => function ($query) {
            $query->select('*');
            $query->where('status', 'publish');
        }])->with(['facility_rating' => function ($query) {
            $query->select('*');
            $query->where('status', 'publish');
        }])->with(['category' => function ($query) {
            $query->select('id', 'name');
        }])->with(['profession' => function ($query) {
            $query->select('id', 'name');
        }])->with(['level' => function ($query) {
            $query->select('id', 'name');
        }])
        ->with(['facility' => function ($query) {
            $query->select('id', 'facility_prefix', 'name', 'facility_group_id', 'address');
        }])
        ->with(['shift_swap' => function ($query) use($request, $user) {
            $query->where('status', '=', 'pending');
            $query->where('hcp_id', '!=', $user->hcp->id);
            $query->with(['hcp' => function ($query) {
                $query->select('id', 'first_name', 'last_name');
            }]);
        }])
        ->with(['cancelled_shift' => function ($query) use($request, $user) {
            $query->where('hcp_id', $user->hcp->id);
        }])->with(['shift_transfer_from' => function ($query) use($request, $user) {
            $query->where('hcp_id', $user->hcp->id);
            $query->where('status', '!=', 'cancelled');
            $query->where('status', '!=', 'reject');

            $query->with(['hcp' => function ($query) {
                $query->select('id', 'first_name', 'last_name', 'image');
            }]);
            $query->with(['transfer_hcp' => function ($query) {
                $query->select('id', 'first_name', 'last_name', 'image');
            }]);
        }])->with(['shift_transfer_to' => function ($query) use($request, $user) {
            $query->where('transfer_hcp_id', $user->hcp->id);
            $query->where('status', '!=', 'cancelled');
            $query->where('status', '!=', 'reject');
            $query->with(['hcp' => function ($query) {
                $query->select('id', 'first_name', 'last_name', 'image');
            }]);
            $query->with(['transfer_hcp' => function ($query) {
                $query->select('id', 'first_name', 'last_name', 'image');
            }]);
        }])
        ->leftJoin('facility_documents', function ($join) {
            $join->on('shifts.facility_id', '=', 'facility_documents.facility_id');
            $join->on('shifts.profession_id', '=', 'facility_documents.profession_id');
        })
        ->with('state')
        ->first();
    }

    public function get_shift_for_cron($where){
        return $this->shift->where(function ($query) use ($where) {
            $query->where($where);
        })->with('facility', 'hcp', 'state')->with(['shift_hcp_loaction' => function ($q) {
                $q->select('shift_id','latitude', 'longitude');
            }])->get();
    }

    public function api_get_buddy_shifts($request, $user){
        $paginationLimit = config('constants.pagination_limit');

        return $this->shift->where(function ($query) use ($request, $user) {

            $query->where('shift_type', 'buddy')->where('hcp_id', $user->hcp->id);

        })->with(['hcp' => function ($query) {
            $query->select('id', 'first_name', 'last_name');
        }])->with(['category' => function ($query) {
            $query->select('id', 'name');
        }])->with(['profession' => function ($query) {
            $query->select('id', 'name');
        }])->with(['level' => function ($query) {
            $query->select('id', 'name');
        }])->with(['facility' => function ($query) {
            $query->select('id', 'name', 'facility_group_id', 'address');
        }])->orderBy('select_date', 'DESC')->paginate($paginationLimit);
    }

    public function get_shift_as_date($from,$to,$request){
        return $this->shift
        ->where(function ($query) use ($request) {

                if(!empty($request['state_id'])) {
                    $query->where('state_id', $request['state_id']);
                }

                if(!empty($request['facility'])) {
                    $query->where('facility_id', $request['facility']);
                }

                if(!empty($request['hcp'])) {
                    $query->where('hcp_id', $request['hcp']);
                }

                if(!empty($request['shift_status'])) {
                    $query->where('shift_status', $request['shift_status']);
                }
	if(!empty($request['assigned_facilities'])) {
                    $query->whereIn('facility_id', $request['assigned_facilities']);
                }

                if(!empty($request['facility_group_ids'])) {
                    $query->whereIn('facility_group_id', $request['facility_group_ids']);
                }
            })
        ->whereBetween('created_at',[$from,$to])->get();
    }

    public function get_shift_as_date_with_pagination($from,$to,$request){
        $paginationLimit = null;

        if(isset($request['shift_per_page']) && !empty($request['shift_per_page'])) {
            $paginationLimit = $request['shift_per_page'];
        } else {
            $paginationLimit = config('constants.pagination_limit');
        }

        return $this->shift
        ->where(function ($query) use ($request) {

                if(!empty($request['state_id'])) {
                    $query->where('state_id', $request['state_id']);
                }

                if($request['facility'] !='') {
                    $query->where('facility_id', $request['facility']);
                }

            })
        ->whereDate('created_at', '>=', $from)->whereDate('created_at', '<=', $to)->paginate($paginationLimit);
    }

    public function calculate_facility_revenue($facility_id){
        return $this->shift->select(DB::raw('sum(facility_amount) as total_facility_amount'))
                            ->where('facility_id',$facility_id)
                            ->where('shift_status','6')
                            ->groupBy('facility_id')
                            ->first();
    }

    public function calculate_hcp_revenue($hcp_id){
        return $this->shift->select(DB::raw('sum(hcp_amount) as total_hcp_amount'))
                            ->where('hcp_id',$hcp_id)
                            ->where('shift_status','6')
                            ->groupBy('hcp_id')
                            ->first();
    }

    public function find_shift_with_where($where){
        return $this->shift->where($where)->with('state')->first();
    }

    public function find_shift_with_date($startTime = null, $endTime = null, $whereArry = null){
        return $this->shift
            ->where('hcp_id', $whereArry['hcp_id'])
            ->where(function ($query) use ($startTime, $endTime, $whereArry) {
                $query->where('shift_status', '!=', '3');
                $query->where('shift_status', '!=', '6');
                // $query->where('shift_status', '!=', '4');
                // $query->orwhere('shift_status', '=', '1');
                // $query->orwhere('shift_status', '=', '2');
            })

            ->where(function ($query) use ($startTime, $endTime, $whereArry) {

                // if (!empty($startTime) && !empty($endTime)) {
                //     $query->orwhere(function ($query) use ($startTime, $endTime) {
                //         $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                //     });
                //     $query->orwhere(function ($query) use ($startTime, $endTime) {
                //         $query->whereBetween('end_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                //     });
                // }

                // if (!empty($startTime) && !empty($endTime)) {
                //     $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                // }

                // if (!empty($startTime) && !empty($endTime)) {
                //     $query->orwhere(function ($query) use ($startTime, $endTime, $whereArry) {
                //         $query->where('start_time', '<=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($startTime));
                //     });
                //     $query->orwhere(function ($query) use ($startTime, $endTime, $whereArry) {
                //         $query->where('start_time', '>=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($endTime))->where('start_time', '<=', Carbon::parse($endTime));
                //     });
                // }

                if (!empty($startTime) && !empty($endTime)) {
                    $query->orwhere(function ($query) use ($startTime, $endTime) {
                        $query->where('start_time', '<=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($startTime));
                    });
                    $query->orwhere(function ($query) use ($startTime, $endTime) {
                        $query->where('start_time', '>=', Carbon::parse($startTime))->where('end_time', '>=', Carbon::parse($endTime))->where('start_time', '<=', Carbon::parse($endTime));
                    });
                    $query->orwhere(function ($query) use ($startTime, $endTime) {
                        $query->whereBetween('start_time', [Carbon::parse($startTime), Carbon::parse($endTime)])
                        ->orWhereBetween('end_time', [Carbon::parse($startTime), Carbon::parse($endTime)]);
                    });
                }
            })
            ->orderBy('id', 'DESC')->first();
    }

    public function api_get_hcp_shift_detail($request, $user){

        return $this->shift->where('hcp_id', $user->hcp->id)->where(function ($query) use ($request, $user) {

            if (!empty($request['start_time']) && !empty($request['end_time'])) {
                $query->where('start_time', '>=', Carbon::parse($request['start_time']))->where('start_time', '<=', Carbon::parse($request['end_time']))
                ->orWhere('end_time', '>=', Carbon::parse($request['start_time']))->where('end_time', '<=', Carbon::parse($request['end_time']));
            }

        })
        ->with('state')
        ->with(['hcp' => function ($query) {
            $query->select('id', 'first_name', 'last_name');
        }])->with(['category' => function ($query) {
            $query->select('id', 'name');
        }])->with(['profession' => function ($query) {
            $query->select('id', 'name');
        }])->with(['level' => function ($query) {
            $query->select('id', 'name');
        }])->with(['facility' => function ($query) {
            $query->select('id', 'name', 'facility_group_id', 'address');
        }])->orderBy('id', 'DESC')->first();
    }

    public function find_swap_date_shift($startTime = null, $endTime = null, $shift = null, $radius = null, $hcpId = null){

        // General Configuration
        if(!empty($radius)) {
            $searchRadius = $radius;
        } else {
            $generalConfiguration = get_general_configuration();
            $searchRadius = $generalConfiguration['radius_km_for_shift_broadcast'];
        }
        //----------------------

        $latitude = $shift->hcp['latitude'];
        $longitude = $shift->hcp['longitude'];

        return $this->shift->where(function ($query) use ($startTime, $endTime, $shift, $latitude, $longitude, $radius, $hcpId) {

            $query->where('shift_status', '1');
            $query->where('shift_type', '!=', 'buddy');
            $query->where('start_time', '=', Carbon::parse($startTime))->where('end_time', '=', Carbon::parse($endTime));
            $query->where('category_id', $shift['category_id']);
            $query->where('profession_id', $shift['profession_id']);
            $query->where('hcp_level_id', $shift['hcp_level_id']);

            if(!empty($hcpId)) {
                $query->where('hcp_id', $hcpId);
            }

            $query->whereHas('facility', function ($query) use ($shift, $latitude, $longitude, $radius) {
                if(!empty($latitude) && !empty($longitude) && !empty($radius)) {
                    $query->select('id','name', 'facility_group_id', 'address', 'latitude', 'longitude', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS('.$latitude.')) * COS(RADIANS(latitude)) * COS( RADIANS('.$longitude.' - longitude)) + SIN(RADIANS('.$latitude.')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<", $radius);
                } else {
                    $query->select('id', 'name', 'facility_group_id', 'address', 'latitude', 'longitude');
                }
            });

        })->with(['facility' => function ($query) use($latitude, $longitude, $radius, $hcpId) {

            if(!empty($latitude) && !empty($longitude) && !empty($radius)) {
                $query->select('id','name', 'facility_group_id', 'address', 'latitude', 'longitude', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS('.$latitude.')) * COS(RADIANS(latitude)) * COS( RADIANS('.$longitude.' - longitude)) + SIN(RADIANS('.$latitude.')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<", $radius);
            } else {
                $query->select('id', 'name', 'facility_group_id', 'address', 'latitude', 'longitude');
            }
        }])
        ->select('id', 'shift_prefix', 'start_time', 'end_time', 'hcp_id', 'shift_status', 'facility_id', 'category_id', 'profession_id', 'hcp_level_id')->orderBy('select_date', 'DESC')->get();
    }

    public function find_swap_hcp_shift($startTime = null, $endTime = null, $shift = null, $radius = null, $hcpId = null){

        // General Configuration
        if(!empty($radius)) {
            $searchRadius = $radius;
        } else {
            $generalConfiguration = get_general_configuration();
            $searchRadius = $generalConfiguration['radius_km_for_shift_broadcast'];
        }
        //----------------------

        $latitude = $shift->hcp['latitude'];
        $longitude = $shift->hcp['longitude'];

        return $this->shift->where(function ($query) use ($startTime, $endTime, $shift, $latitude, $longitude, $radius, $hcpId) {

            $query->where('shift_status', '1');
            $query->where('shift_type', '!=', 'buddy');
            $query->where('start_time', '=', Carbon::parse($startTime))->where('end_time', '=', Carbon::parse($endTime));
            $query->where('category_id', $shift['category_id']);
            $query->where('profession_id', $shift['profession_id']);
            $query->where('hcp_level_id', $shift['hcp_level_id']);

            if(!empty($hcpId)) {
                $query->where('hcp_id', $hcpId);
            }

            $query->whereHas('facility', function ($query) use ($shift, $latitude, $longitude, $radius) {
                if(!empty($latitude) && !empty($longitude) && !empty($radius)) {
                    $query->select('id','name', 'facility_group_id', 'address', 'latitude', 'longitude', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS('.$latitude.')) * COS(RADIANS(latitude)) * COS( RADIANS('.$longitude.' - longitude)) + SIN(RADIANS('.$latitude.')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<", $radius);
                } else {
                    $query->select('id', 'name', 'facility_group_id', 'address', 'latitude', 'longitude');
                }
            });

        })->with(['facility' => function ($query) use($latitude, $longitude, $radius) {

            if(!empty($latitude) && !empty($longitude) && !empty($radius)) {
                $query->select('id','name', 'facility_group_id', 'address', 'latitude', 'longitude', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS('.$latitude.')) * COS(RADIANS(latitude)) * COS( RADIANS('.$longitude.' - longitude)) + SIN(RADIANS('.$latitude.')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<", $radius);
            } else {
                $query->select('id', 'name', 'facility_group_id', 'address', 'latitude', 'longitude');
            }
        }])
        ->select('id', 'shift_prefix', 'start_time', 'end_time', 'hcp_id', 'shift_status', 'facility_id', 'category_id', 'profession_id', 'hcp_level_id')->orderBy('select_date', 'DESC')->first();

    }

    public function find_transfer_hcp_shift($startTime = null, $endTime = null, $shift = null, $radius = null, $hcpId = null){

        // General Configuration
        if(!empty($radius)) {
            $searchRadius = $radius;
        } else {
            $generalConfiguration = get_general_configuration();
            $searchRadius = $generalConfiguration['radius_km_for_shift_broadcast'];
        }
        //----------------------

        $latitude = $shift->hcp['latitude'];
        $longitude = $shift->hcp['longitude'];

        return $this->shift->where(function ($query) use ($startTime, $endTime, $shift, $latitude, $longitude, $radius, $hcpId) {

            $query->where('shift_status', '1');
            $query->where('shift_type', '!=', 'buddy');
            $query->where('start_time', '=', Carbon::parse($startTime))->where('end_time', '=', Carbon::parse($endTime));
            $query->where('category_id', $shift['category_id']);
            $query->where('profession_id', $shift['profession_id']);
            $query->where('hcp_level_id', $shift['hcp_level_id']);

            if(!empty($hcpId)) {
                $query->where('hcp_id', $hcpId);
            }

            $query->whereHas('facility', function ($query) use ($shift, $latitude, $longitude, $radius) {
                if(!empty($latitude) && !empty($longitude) && !empty($radius)) {
                    $query->select('id','name', 'facility_group_id', 'address', 'latitude', 'longitude', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS('.$latitude.')) * COS(RADIANS(latitude)) * COS( RADIANS('.$longitude.' - longitude)) + SIN(RADIANS('.$latitude.')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<", $radius);
                } else {
                    $query->select('id', 'name', 'facility_group_id', 'address', 'latitude', 'longitude');
                }
            });

        })->with(['facility' => function ($query) use($latitude, $longitude, $radius) {

            if(!empty($latitude) && !empty($longitude) && !empty($radius)) {
                $query->select('id','name', 'facility_group_id', 'address', 'latitude', 'longitude', DB::raw('( 111.111 * ( DEGREES( ACOS( COS(RADIANS('.$latitude.')) * COS(RADIANS(latitude)) * COS( RADIANS('.$longitude.' - longitude)) + SIN(RADIANS('.$latitude.')) * SIN(RADIANS(latitude)))))) AS distance'))->having("distance", "<", $radius);
            } else {
                $query->select('id', 'name', 'facility_group_id', 'address', 'latitude', 'longitude');
            }
        }])
        ->select('id', 'shift_prefix', 'start_time', 'end_time', 'hcp_id', 'shift_status', 'facility_id', 'category_id', 'profession_id', 'hcp_level_id')->orderBy('select_date', 'DESC')->first();

    }

    public function get_shift_count_starting_hours($request = null){

        return $this->shift->where(function ($query) use ($request) {

            if(isset($request['shift_status']) && $request['shift_status'] !='') {
                $query->where('shift_status', $request['shift_status']);
            }

            if(!empty($request['shift_filter_traffic_lights'])) {

                $query->where('facility_id', $request['facility_id']);
                if ($request['shift_filter_traffic_lights'] == 'blue') {
                    $blue = config('constants.shifts.shift_filter_traffic_lights.BLUE');
                    $blueStartTime = date('Y-m-d H:i', strtotime('+'.$blue['max_value'].' minutes', strtotime($request['current_time'])));
                    $query->where('start_time', '>', $blueStartTime);

                } else if($request['shift_filter_traffic_lights'] == 'amber') {
                    $amber = config('constants.shifts.shift_filter_traffic_lights.AMBER');
                    $startTime = date('Y-m-d H:i', strtotime('+'.$amber['min_value'].' minutes', strtotime($request['current_time'])));
                    $endTime = date('Y-m-d H:i', strtotime('+'.$amber['max_value'].' minutes', strtotime($request['current_time'])));

                    $query->where(function ($query) use ($startTime, $endTime) {
                        $query->whereBetween('start_time', [$startTime, $endTime]);
                    });
                } else if($request['shift_filter_traffic_lights'] == 'red') {
                    $red = config('constants.shifts.shift_filter_traffic_lights.RED');
                    $redEndTime = date('Y-m-d H:i', strtotime('+'.$red['max_value'].' minutes', strtotime($request['current_time'])));
                    $query->where('start_time', '<', $redEndTime);
                }

            }
        })->select('id', 'start_time', 'shift_status')->get();

    }

    public function get_shift_invoice_generate($filter = null){
        return $this->shift->where(function ($query) use ($filter) {
//$filter['current_date'] = "2025-01-29";
            if (!empty($filter) && !empty($filter['current_date'])) {
                $query->whereDate('end_time', '<=', Carbon::parse($filter['current_date']));
                $invoiceStartDate = date('Y-m-d',strtotime("-7 day",strtotime($filter['current_date'])));
                $from=$invoiceStartDate.' 23:59:59';
                $to=$filter['current_date'].' 23:59:59';
                $from='2024-12-01 00:00:00';
                $to='2024-12-20 23:59:59';
                //$query->whereBetween('shift_approved_time', [$from, $to]);
            }

            if (!empty($filter) && !empty($filter['shift_status'])) {
                $query->where('shift_status', $filter['shift_status']);
            }

            if(!empty($filter['facility_id'])) {
                $query->where('facility_id', $filter['facility_id']);
            }

            if(!empty($filter['hcp_id'])) {
                $query->where('hcp_id', $filter['hcp_id']);
            }

            if(!empty($filter) && isset($filter['shift_ids']) && count($filter['shift_ids'])) {
                $query->whereIn('id', $filter['shift_ids']);
            }

            $query->whereHas('transactions', function ($query) use ($filter) {
                if(!empty($filter['facility_id'])) {
                    $query->where('invoice_id', null);
                }
                if(!empty($filter['hcp_id'])) {
                   $query->where('statement_id', null);
                }
            });

        })->with(['transactions' => function ($query) use($filter) {
            if(!empty($filter['facility_id'])) {
               $query->where('invoice_id', null);
            }

            if(!empty($filter['hcp_id'])) {
               $query->where('statement_id', null);
            }
        }])->orderBy('id', 'DESC')->get();
    }

    public function get_hcp_shift_invoice_generate($filter = null){

        return $this->shift->where(function ($query) use ($filter) {

            if (!empty($filter) && !empty($filter['current_date'])) {
                $query->where('start_time', '<', Carbon::parse($filter['current_date']));
            }

            if (!empty($filter) && !empty($filter['shift_status'])) {
                $query->where('shift_status', $filter['shift_status']);
            }

            if(!empty($filter['facility_id'])) {
                $query->where('facility_id', $filter['facility_id']);
            }

            if(!empty($filter['hcp_id'])) {
                $query->where('hcp_id', $filter['hcp_id']);
            }

            $query->whereHas('transactions', function ($query) use ($filter) {
                $query->where('statement_id', null);
            $query->where('amount_for','=','hcp');
            });

        })->with(['transactions' => function ($query) use($filter) {
            $query->where('statement_id', null);
            $query->where('amount_for','=','hcp');
        }])->orderBy('id', 'DESC')->get();
    }

    public function get_paid_facility_invoice($filter = null){

        return $this->shift->where(function ($query) use ($filter) {

            if (!empty($filter) && !empty($filter['current_date'])) {
                $query->where('start_time', '<', Carbon::parse($filter['current_date']));
            }

            if (!empty($filter) && !empty($filter['shift_status'])) {
                $query->where('shift_status', $filter['shift_status']);
            }

            if(!empty($filter['facility_id'])) {
                $query->where('facility_id', $filter['facility_id']);
            }

            if(!empty($filter['hcp_id'])) {
                $query->where('hcp_id', $filter['hcp_id']);
            }

            $query->whereHas('transactions', function ($query) use ($filter) {
                $query->where('status', '=', 'completed');
                $query->where('amount_for', '=', 'facility');
            });

        })->with(['transactions' => function ($query) use($filter) {
            $query->where('status', '=', 'completed');
            $query->where('amount_for', '=', 'facility');
        }])->orderBy('id', 'DESC')->get();
    }

    public function get_shift_for_account_card($filter = null){

        return $this->shift->where(function ($query) use ($filter) {

            if (!empty($filter) && !empty($filter['current_date'])) {
                $query->where('end_time', '<', Carbon::parse($filter['current_date']));
            }

            if (!empty($filter) && !empty($filter['start_date'])) {
                if (!empty($filter) && isset($filter['shift_status']) && $filter['shift_status'] == '6') {
                    $query->whereDate('shift_approved_time', '>=', Carbon::parse($filter['start_date']));
                } else if (!empty($filter) && isset($filter['shift_status']) && $filter['shift_status'] == '3') {
                    $query->whereDate('end_time', '>=', Carbon::parse($filter['start_date']));
                } else {
                    $query->whereDate('end_time', '>=', Carbon::parse($filter['start_date']));
                }
            }

            if (!empty($filter) && !empty($filter['end_date'])) {
                if (!empty($filter) && isset($filter['shift_status']) && $filter['shift_status'] == '6') {
                    $query->whereDate('shift_approved_time', '<=', Carbon::parse($filter['end_date']));
                } else if (!empty($filter) && isset($filter['shift_status']) && $filter['shift_status'] == '3') {
                    // $query->whereDate('shift_approved_time', '<=', Carbon::parse($filter['end_date']));
                } else {
                    $query->whereDate('end_time', '<=', Carbon::parse($filter['end_date']));
                }
            }

            if (!empty($filter) && !empty($filter['shift_status'])) {
                $query->where('shift_status', $filter['shift_status']);
            }

            if(!empty($filter['facility_id'])) {
                $query->where('facility_id', $filter['facility_id']);
            }

            if(!empty($filter['hcp_id'])) {
                $query->where('hcp_id', $filter['hcp_id']);
            }

            if (!empty($filter['invoice_status'])) {
                $query->whereHas('transactions', function ($query) use ($filter) {

                    if(isset($filter['invoice_status']) && !empty($filter['invoice_status'])) {
                        $query->where('status', '=', $filter['invoice_status']);
                    }

                    if(isset($filter['statement_status']) && !empty($filter['statement_status'])) {
                        $query->where('hcp_payment_status', '=', $filter['statement_status']);
                    }

                    if(isset($filter['facility_id']) && !empty($filter['facility_id'])) {
                        $query->where('amount_for', '=', 'facility');
                    }

                    if(isset($filter['hcp_id']) && !empty($filter['hcp_id'])) {
                        $query->where('amount_for', '=', 'hcp');
                    }

                    if (isset($filter['invoice_id'])) {
                        $query->where('invoice_id', '=', $filter['invoice_id'] ? 0 : null);
                    }

                    if (isset($filter['statement_id'])) {
                        $query->where('statement_id', '=', $filter['statement_id'] ? 0 : null);
                    }

                    $query->where('amount_type', '=', 'shift');
                });
            }

        })->with(['transactions' => function ($query) use($filter) {

            if(isset($filter['invoice_status']) && !empty($filter['invoice_status'])) {
                $query->where('status', '=', $filter['invoice_status']);
            }

            if(isset($filter['statement_status']) && !empty($filter['statement_status'])) {
                $query->where('hcp_payment_status', '=', $filter['statement_status']);
            }

            if(isset($filter['facility_id']) && !empty($filter['facility_id'])) {
                $query->where('amount_for', '=', 'facility');
            }

            if(isset($filter['hcp_id']) && !empty($filter['hcp_id'])) {
                $query->where('amount_for', '=', 'hcp');
            }

            if (isset($filter['invoice_id'])) {
                $query->where('invoice_id', '=', $filter['invoice_id'] ? 0 : null);
            }

            if (isset($filter['statement_id'])) {
                $query->where('statement_id', '=', $filter['statement_id'] ? 0 : null);
            }

            $query->where('amount_type', '=', 'shift');
        }])->orderBy('id', 'DESC')->get();
    }

    public function get_shift_for_broadcast($where){

//        return $this->shift->with('category', 'profession', 'facility', 'hcp', 'buddy_hcp', 'cancelled_shifts', 'state')->where($where)->get();
       return $this->shift->select('id','shift_prefix','facility_id','state_id','country_id','category_id','profession_id','hcp_level_id','start_time','end_time','created_by','shift_broadcast','shift_rebroadcast_time')->with([
            'cancelled_shifts' => function ($query) {
                $query->addSelect('id','shift_id','hcp_id');
            },
        ])->with([
            'state' => function ($query) {
                $query->addSelect('id','time_zone_name');
            },
        ])->where($where)->get();
    }

}

